<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Session;
use App\Models\User;
use App\Models\Customers;
use App\Models\Routes;
use App\Models\DeliveryDetails;
use Hash;
use DB;

class AuthController extends Controller
{

    public function index()
    {
        return view('auth.login');
    }  


    public function postLogin(Request $request)
    {
        $request->validate([
            'email' => 'required',
            'password' => 'required',
        ]);

        $credentials = $request->only('email', 'password');
        if (Auth::attempt($credentials)) {
            return redirect()->intended('dashboard')
            ->withSuccess('You have Successfully loggedin');
        }

        return redirect("login")->withSuccess('Oppes! You have entered invalid credentials');
    }


    public function dashboard()
    {
        if(Auth::check()){
            return view('auth.dashboard');
        }

        return redirect("login")->withSuccess('Opps! You do not have access');
    }

    public function check()
    {
        $routes = Routes::where('is_delete', 0)->orderBy('route_name')->pluck('route_name', 'id');
        return view('auth.check', compact('routes'));
    }

    public function checkRoute(Request $request)
    {
        $routes = Routes::where('is_delete', 0)->orderBy('route_name')->pluck('route_name', 'id');
        $route_id = $request->route_id;
        $month = $request->route_month;
        $first_day = "01-".$month;
        $first_day = date('Y-m-d', strtotime($first_day));
        $last_day = date('Y-m-t', strtotime($first_day));

        $route_model = Routes::find($route_id);

        $duplicate_date_list = [];
        $start_date = strtotime($first_day);
        $end_date = strtotime($last_day);
        while($start_date < $end_date){

            $date = date('Y-m-d', $start_date);
            $table_ids = [];

            $duplicateRecords = DB::table('delivery_details')->select('delivery_details.id', 'delivery_details.customer_id', 'delivery_details.product_id', 'delivery_details.created_at')
            ->join('customers','customers.id', '=','delivery_details.customer_id' )
            ->selectRaw('count(delivery_details.customer_id) as occurences')
            //->from('delivery_details')
            ->where('customers.is_delete', 0)
            ->where('customers.route_id', '=' ,$route_id)
            ->whereDate('customers.created_at', '<=' ,$date)
            ->whereDate('delivery_details.created_at', '=', $date)
            ->groupBy('customer_id', 'product_id', 'quantity')
            ->having('occurences', '>', 1)
            ->get();

            $table_ids = $duplicateRecords->pluck('id')->all();
            $customer_ids = $duplicateRecords->pluck('customer_id')->all();
            $product_ids = $duplicateRecords->pluck('product_id')->all();
            $delivered_date = $duplicateRecords->pluck('created_at')->all();

            if(!empty($table_ids)){
                $duplicate_date_list[] = ['date' => $date, 'ids'=>$table_ids, 'customer_ids'=>$customer_ids, 'product_ids'=>$product_ids, 'delivered_date'=>$delivered_date];
            }
            $start_date = strtotime("+1 day", $start_date);
        }

        return view('auth.check', compact('duplicate_date_list', 'routes'))->with('route_model', $route_model);
    }  

    public function checkCustomer(Request $request)
    {
        return view('auth.check_customer');
    }

    public function checkCustomerData(Request $request)
    {
        if(isset($request->customer_id)){
            $customer_id = $request->customer_id;

            $customer_details = Customers::findOrFail($customer_id);
            return view('auth.check_customer')->with('customer_details', $customer_details);

        }
        
        return view('auth.check_customer');
    }
    
    public function logout() {
        Session::flush();
        Auth::logout();

        return Redirect('login');
    }



    public function routelist()
    {
        $routes = Routes::where('is_delete', 0)->orderBy('route_name')->get();
        return view('auth.routelist', compact('routes'));
    }

    public function routeData(Request $request)
    {
        $route = Routes::find($request->route_id);
        $customers = Customers::where('route_id', $request->route_id)->where('is_delete', 0)->orderBy('customer_name')->get();
        return view('auth.route_customers', compact('route'))->with('customers', $customers);

    }

    public function submitData(Request $request)
    {
        $data = $request->all();
        $customers = $data['Customer_count'];

        foreach ($customers as $customer_id => $input) {
            // echo "Customer_id : ".$customer_id."<br>";
            // echo "Count : ".$data['Customer_count'][$customer_id]."<br><br>";

            $quantity = isset($data['Customer_count'][$customer_id])?$data['Customer_count'][$customer_id]:0;

            DB::table('delivery_details')
            ->updateOrInsert(
                ['customer_id' => $customer_id, 'product_id' => 1, 'created_at'=> '2021-05-31'],
                ['quantity' => $quantity, 'amount' => $quantity * 32, 'payment_status'=>0]
            );

            /*
            $delivery = DeliveryDetails::whereDate('created_at', '2021-05-31')->where('customer_id', $customer_id)->where('product_id', 1)->get();
            if(empty($delivery)){
                $delivery = new DeliveryDetails();
            }
            $delivery->customer_id  = $customer_id;
            $delivery->product_id   = 13;
            $delivery->quantity     = $quantity;
            $delivery->amount       = $quantity * 32;
            $delivery->created_at   = "2021-05-31";
            $delivery->payment_status = 0;
            $delivery->save();
            */
            

        }
        return redirect()->route('routelist')
        ->with('success', 'Delivery updated successfully');
    }



    public function supplyIndex()
    {
        $routes = Routes::where('is_delete', 0)->orderBy('route_name')->pluck('route_name', 'id');
        return view('supply.index', compact('routes'));
    }

    public function supplyDetails(Request $request)
    {
        $routes = Routes::find($request->route_id);
        $date = $request->route_date;

        $details = DB::table('delivery_details')->select('delivery_details.*')
        ->join('customers','customers.id', '=','delivery_details.customer_id' )
            //->selectRaw('count(delivery_details.customer_id) as occurences')
            //->from('delivery_details')
        ->where('customers.is_delete', 0)
        ->where('customers.route_id', '=' ,$request->route_id)
        ->whereDate('customers.created_at', '<=' ,$date)
        ->whereDate('delivery_details.created_at', '=', $date)
        ->orderBy('customer_id')
        ->get();

            //print("<pre>"); print_r($details); print("</pre>"); exit;
        return view('supply.details', compact('details'))->with('routes', $routes);
    } 


    public function deleteDelivery($delivery_id){

        $model = DeliveryDetails::find($delivery_id);
        $model->delete();
    }

}