<?php

namespace App\Http\Controllers;

use DB;
use Illuminate\Http\Request;
use App\Models\CustomerOrders;
use App\Models\Routes;
use Validator;
use Exception;
use App\Http\Controllers\BaseController as BaseController;

class CustomerOrderController extends BaseController
{

    public function orderList(Request $request)
    {
        $query = CustomerOrders::query();
        $query->select('customer_products.id','customer_products.customer_id','customer_products.product_id','customer_products.quantity', DB::raw("(customer_products.delivery_start_date) as order_date"), 'customer_products.created_at', 'customers.customer_name', 'products.product_name');
        $query->leftJoin('customers', 'customers.id', '=', 'customer_products.customer_id');
        $query->leftJoin('products', 'products.id', '=', 'customer_products.product_id');
        
        $query->orderBy('customer_products.created_at');
        if($request->has('start_date') && $request->start_date!= ''){
            $query = $query->whereDate('customer_products.delivery_start_date', '>=' ,$request->start_date);
        }
        if($request->has('end_date') && $request->end_date!= ''){
            $query = $query->whereDate('customer_products.delivery_start_date', '<=' ,$request->end_date);
        }
        $query->where('delivery_mode','order');
        $orders = $query->paginate(50);
        return $this->sendResponse($orders,'Order List');
    }


    public function addOrder(Request $request)
    {

        $input = $request->all(); 
        $validator = Validator::make($input, CustomerOrders::rules(), CustomerOrders::messages());
        if ($validator->fails()) { 
            return $this->sendValidationErrors($validator->errors()); 
        }

        $input['delivery_start_date'] = $request->order_date;
        $input['delivery_mode'] = 'order';
        if (CustomerOrders::create($input)){

            return $this->sendResponse('','Order Successfully Added');
        }else{
            return $this->sendError('Order Not Added');  
        }
    }

   
    
    public function updateOrder(Request $request)
    {

        $input  = $request->all(); 
        $id     = isset($input['id'])?$input['id']:0;
        $details = CustomerOrders::find($id);
        if($details){
            $validator = Validator::make($input, CustomerOrders::rules($id));
            if ($validator->fails()) { 
                return $this->sendValidationErrors($validator->errors()); 
            }
            
            $data = array('customer_id'=> $request->customer_id, 'product_id'=> $request->product_id, 'quantity'=> $request->quantity, 'delivery_start_date'=> $request->order_date);
            $updated = $details->fill($data)->save();

            if($updated){

                $details = CustomerOrders::select('customer_products.id','customer_products.customer_id','customer_products.product_id','customer_products.quantity', DB::raw("(customer_products.delivery_start_date) as order_date"), 'customer_products.created_at', 'customers.customer_name', 'products.product_name')
                    ->where('customer_products.id',$id)
                    ->leftJoin('customers', 'customers.id', '=', 'customer_products.customer_id')
                    ->leftJoin('products', 'products.id', '=', 'customer_products.product_id')
                    ->first();
                
                return $this->sendResponse($details,'Order Successfully Updated');
            }else{
                return $this->sendError('Order Not Updated');  
            }
        }else{
            return $this->sendError('Order Not Found');  
        }
        
    }

    public function deleteOrder($id)
    {
        $details = CustomerOrders::find($id);

        if($details){
            if($details->delete()){
                return $this->sendResponse('','Order Successfully Deleted');
            }else{
                return $this->sendError('Order Not Deleted');  
            }
        }else{
            return $this->sendError('Order Not Found');  
        }
    }

    public function viewOrder($id)
    {
        $details = CustomerOrders::select('customer_products.id','customer_products.customer_id','customer_products.product_id','customer_products.quantity', DB::raw("(customer_products.delivery_start_date) as order_date"), 'customer_products.created_at', 'customers.customer_name', 'products.product_name')
            ->where('customer_products.id',$id)
            ->leftJoin('customers', 'customers.id', '=', 'customer_products.customer_id')
            ->leftJoin('products', 'products.id', '=', 'customer_products.product_id')
            ->first();

        if($details){
                return $this->sendResponse($details,'Order Details');
        }else{
            return $this->sendError('Order Not Found');  
        }
    }
}
