<?php

namespace App\Http\Controllers;

use DB;
use App\Models\Customers;
use App\Models\Products;
use App\Models\Routes;
use App\Models\CustomerProducts;
use App\Models\DeliveryDetails;
use App\Models\DeliverySummary;
use App\Models\CustomerPayments;
use Illuminate\Http\Request;
use Validator;
use Exception;
use Illuminate\Support\Facades\Auth; 
use Illuminate\Support\Facades\Log;
use App\Http\Controllers\BaseController as BaseController;

class CustomersController extends BaseController
{

    public function customerList(Request $request)
    {
        $query = Customers::query();
        $query->select('customers.*', 'routes.route_name')->where('customers.is_delete', 0)->leftJoin('routes', 'routes.id', '=', 'customers.route_id');

        if($request->has('route') && $request->route!= ''){
            $query = $query->where('customers.route_id', '=' ,$request->route);
            $query->orderBy('customers.route_position');
        }else{
            $query->orderBy('customers.customer_name');
        }


        if($request->has('search') && $request->search!=''){
            if(is_numeric($request->search)){
                $query = $query->where('customers.phone_number', $request->search);
            }else{
                $query = $query->where('customers.customer_name','like','%'.$request->search.'%');
            }
        }
        // if($request->has('phone') && $request->phone!=''){
        //     $query = $query->where('customers.phone_number', $request->phone);
        // }
        
        $customers = $query->paginate(50);
        // $customers = Customers::select('customers.*', 'routes.route_name')->where('customers.status', 0)->where('customers.is_delete', 0)->orderBy('customer_name')->leftJoin('routes', 'routes.id', '=', 'customers.route_id')->paginate(5);          
        return $this->sendResponse($customers,'Customer List');
    }

    public function addCustomer(Request $request)
    {
        $input = $request->all();
        $id     = isset($input['id'])?$input['id']:0;
        $route_id = $input['route_id'];

        $customer_products  = json_decode($input['products'], true);

        $validator = Validator::make($input, Customers::rules($id, $route_id));
        //$product_validator = Validator::make($customer_products, Customers::pro_rules($id));
        if ($validator->fails()) { 
            return $this->sendValidationErrors($validator->errors());         
        }

        $input              = $request->all(); 
        $nearest_customer_id   = $input['nearest_customer_id']; //previous customer id
        $route_id           = $input['route_id'];
        $customer_products  = json_decode($input['products'], true); 
        

        
        if($nearest_customer_id == 0){
            $input['route_position'] = 1;
        }
        if(!$nearest_customer_id == 0){
            $model = Customers::find($nearest_customer_id);
            $near_position = $model->route_position;

            if($input['position_type']==0){ //before customer
                $input['route_position'] = $near_position;
            }else{
                $input['route_position'] = $near_position+1;
            }
        }

        if ($customer = Customers::create($input)){

            //save customer products
            if(!empty($customer_products)){
                foreach ($customer_products as $data) {
                    $data['customer_id'] = $customer->id;
                    CustomerProducts::create($data);
                }
            }

            $sql = 'UPDATE customers SET route_position = (route_position+1) WHERE route_position >='.$customer->route_position.' AND route_id ='.$route_id.' AND id !='.$customer->id;
            $update = DB::update($sql);

            return $this->sendResponse('','Customer Successfully Added'); 
        }else{
            return $this->sendError('Customer Not Added');   
        }
    }

    
    public function updateCustomer(Request $request)
    {
        $input  = $request->all(); 
        $id     = isset($input['id'])?$input['id']:0;
        $route_id = $input['route_id'];
        $customer = Customers::find($id);
        if($customer){
            $validator = Validator::make($input, Customers::rules($id, $route_id));
            if ($validator->fails()) { 
                return $this->sendValidationErrors($validator->errors()); 
            }
            
            $nearest_customer_id   = $input['nearest_customer_id']; //nearest customer id
            $route_id           = $input['route_id'];
            $customer_products  = json_decode($input['products'], true); 

            if($nearest_customer_id == 0){ //no customers in this route
                $input['route_position'] = 1;
            }

            if(!$nearest_customer_id == 0){
                $model = Customers::find($nearest_customer_id);
                $near_position = $model->route_position;

                if($input['position_type']==0){ //before customer
                    $input['route_position'] = $near_position;
                }else{
                    $input['route_position'] = $near_position+1;
                }
            }



            $updated = $customer->fill($input)->save();

            if($updated){

                $old_products_ids = DB::table('customer_products')->where('customer_id', $customer->id)->pluck('id')->toArray();
                //save customer products
                if(!empty($customer_products)){
                    $exist_ids = [];
                    foreach ($customer_products as $data) {
                        $data['customer_id'] = $customer->id;
                        if(isset($data['id'])){ //update old product details

                            $cust_prod = DB::table('customer_products')->where('id', $data['id'])->update($data);

                            $exist_ids[] = $data['id'];
                        }else{
                            //add new products
                            CustomerProducts::create($data);
                        }
                    }


                    $deleted_ids = array_diff($old_products_ids, $exist_ids);
                    $deleted_items = DB::table('customer_products')->whereIn('id', $deleted_ids)->delete(); 

                }

                $sql = 'UPDATE customers SET route_position = (route_position+1) WHERE route_position >='.$customer->route_position.' AND route_id ='.$route_id.' AND id !='.$customer->id;
                $update = DB::update($sql);

                //reorder position
                $route_customers = Customers::select('id', 'route_position')->where('route_id', $route_id)->orderBy('route_position')->get();
                foreach ($route_customers as $key => $customer_data) {
                    $customer_data->update(['route_position' => $key+1]);
                }


                return $this->sendResponse($customer,'Customer Successfully Updated');
            }else{
                return $this->sendError('Customer Not Updated');  
            }
        }else{
            return $this->sendError('Customer Not Found');  
        }
    }

    public function viewCustomer($id)
    {
        $customer = Customers::select('customers.*', 'routes.route_name')->where('customers.id', $id)->where('customers.is_delete', 0)
        ->groupBy('customers.id')
                        //->join('customer_products', 'customer_products.customer_id', '=', 'customers.id')
        ->leftJoin('routes', 'routes.id', '=', 'customers.route_id')
        ->first();

        $product_details = CustomerProducts::select('customer_products.*', 'products.product_name')->where('customer_products.customer_id', $id)->leftJoin('products', 'products.id', '=', 'customer_products.product_id')
        ->get();

        
        if($customer){

            $customer_data = $customer->toArray();

            $prev_customer = Customers::select('customers.id', 'customers.customer_name')->where('route_position', $customer->route_position-1)->where('route_id',$customer->route_id)->first();
            if(!empty($prev_customer)){
                $customer_data['nearest_customer_id'] = $prev_customer['id'];
                $customer_data['nearest_customer_name']     = $prev_customer['customer_name'];
                $customer_data['position_type']     = 1;
            }else{
                $after_customer = Customers::select('customers.id', 'customers.customer_name')->where('route_position', $customer->route_position+1)->where('route_id',$customer->route_id)->first();
                if(!empty($after_customer)){
                    $customer_data['nearest_customer_id'] = $after_customer['id'];
                    $customer_data['nearest_customer_name']     = $after_customer['customer_name'];
                    $customer_data['position_type']     = 0;
                }else{
                    $customer_data['nearest_customer_id'] = 0;
                    $customer_data['nearest_customer_name']     = '';
                    $customer_data['position_type']     = 0;
                }
            }
            

            $result = ['customer_details'=>$customer_data, 'products'=>$product_details->toArray()];
            return $this->sendResponse($result, 'Customer Details');
        }else{
            return $this->sendError('Customer Not Found');  
        }
    }

    public function deleteCustomer($id)
    {
        $customer = Customers::find($id);

        if($customer){
            $customer->is_delete = 1;
            if($customer->save()){
                return $this->sendResponse('','Customer Successfully Deleted');
            }else{
                return $this->sendError('Customer Not Deleted');  
            }
        }else{
            return $this->sendError('Customer Not Found');  
        } 
    }

    public function changeCustomerStatus(Request $request)
    {
        $input  = $request->all(); 
        $id     = isset($input['id'])?$input['id']:0;
        $status = isset($input['status'])?$input['status']:1;
        $customer = Customers::find($id);

        if($customer){
            $customer->status = $status;
            if(isset($input['inactive_till_date'])){
                $customer->inactive_till_date = $input['inactive_till_date'];
            }

            if($status == 1){
                $customer->inactive_till_date = NULL;
            }
            if($customer->save()){
                return $this->sendResponse('','Customer Status Changed Successfully');
            }else{
                return $this->sendError('Status Not Changed');  
            }
        }else{
            return $this->sendError('Customer Not Found');  
        }
    }

    public function submitDelivery(Request $request){

        $input = $request->all();
        
        $customer_products  = json_decode($input['data'], true);
        $delivery_date      = isset($input['delivery_date'])?$input['delivery_date']:date('Y-m-d');
        $delivery_date_time = $delivery_date." ".date('H:i:s');

        $validator = Validator::make($customer_products, DeliveryDetails::rules());

        if ($validator->fails()) { 
            return $this->sendValidationErrors($validator->errors()); 
        }

        $user = Auth::user();
        $route_model = Routes::where('route_user_id', $user->id)->first();

        Log::channel('delivery')->info("Route Name : ".$route_model->route_name);
        Log::channel('delivery')->info($input);

        $delivery_details = [];

        $product_price = Products::all()->keyBy('id')->toArray();
        foreach ($customer_products as $key => $delivery_data) {
            $customer_id = $delivery_data['customer_id'];
            $delivery_products = $delivery_data['details'];
            foreach ($delivery_products as $key => $product_data) {
                $product_id = $product_data['product_id'];
                $quantity   = $product_data['quantity'];

                $supply_date = $delivery_date;
                $exist = DeliveryDetails::where([ 
                    ['customer_id', '=', $customer_id],
                    ['product_id', '=', $product_id],
                    ['quantity', '=', $quantity],
                ])
                ->whereDate('created_at', '=' ,$supply_date)->first();
                if(!$exist){
                    $model = new DeliveryDetails;
                    $model->customer_id = $customer_id;
                    $model->product_id  = $product_id;
                    $model->quantity    = $quantity;
                    $model->amount      = isset($product_price[$product_id]['price'])?($product_price[$product_id]['price'] * $quantity):0;
                    $model->payment_status = 0;
                    $model->created_at = $delivery_date_time;
                    $model->save();

                    $delivery_details[$product_id] =  isset($delivery_details[$product_id])? ($delivery_details[$product_id] + $quantity) : $quantity;

                }       
                
            }
        }

        $date   = $delivery_date;
        //$day    = strtolower(date("D"));
        $day    = strtolower(date("D", strtotime($date)));
        $products = Products::select('products.id', DB::raw("SUM(customer_products.quantity) AS quantity"))
        ->join('customer_products', 'customer_products.product_id', '=', 'products.id')
        ->join('customers', 'customer_products.customer_id', '=', 'customers.id')
        ->join('routes', 'customers.route_id', '=', 'routes.id')
        ->where('customers.is_delete', 0)
        ->where('customers.status', 1)
        ->where(function ($query) use ($day, $date) {
            $query->where([ ['customer_products.delivery_mode', '=', 'daily'] ])->orWhere([ ['customer_products.delivery_mode', '=', 'custom'],['customer_products.'.$day, '=', 1] ])->orWhere([ ['customer_products.delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', customer_products.delivery_start_date), 2)"), '=', 0] ]);
        })
        ->where('routes.route_user_id', $user->id)
        ->groupBy('products.id')
        ->get();
        $actual_products = $products->toArray();

        $actual_product_ids = array_column($actual_products, 'id');
        $delivered_product_ids = array_keys($delivery_details); //key of this array

        $all_product_ids = array_unique(array_merge($actual_product_ids, $delivered_product_ids), SORT_REGULAR);

        //print("<pre>"); print_r($products->toArray()); print("</pre>"); 
        //print("<pre>"); print_r($delivery_details); print("</pre>");
        //print("<pre>"); print_r($all_product_ids); print("</pre>"); exit; exit;
        foreach ($all_product_ids as $pid) {

            $index = array_search($pid, array_column($actual_products, 'id'));
            if(false !== $index){
                $actual_product_quantity = $actual_products[$index]['quantity'];
            }else{
                $actual_product_quantity = 0;
            }
            $delivered_quantity = isset($delivery_details[$pid])?$delivery_details[$pid]:0;

            $summary_exist = DeliverySummary::where([ 
                ['route_id', '=', $route_model->id],
                ['product_id', '=', $pid],
            ])
            ->whereDate('created_at', '=' ,$delivery_date)->first();
            if(!$summary_exist){
                $summary_model = new DeliverySummary;
                $summary_model->route_id = $route_model->id;
                $summary_model->product_id = $pid;
                $summary_model->delivered_quantity = $delivered_quantity;
                $summary_model->actual_quantity = $actual_product_quantity;
                $summary_model->created_at = $delivery_date_time;
                $summary_model->save();
            }
            
        }
        
        return $this->sendResponse('','Product Delivery Successfull');
    }


    public function productPayment(Request $request){

        $input = $request->all();
        $validator = Validator::make($input, CustomerPayments::rules());

        if ($validator->fails()) { 
            return $this->sendValidationErrors($validator->errors()); 
        }

        $customer_id    = $input['customer_id'];
        $amount         = $input['amount'];
        $payment_type   = $input['payment_type'];

        if ($payment = CustomerPayments::create($input)){

            $flag = true;
            $customer_advance_amount = $payment->customer->advance_amount;
            $total_amount  = $customer_advance_amount + $amount;

            while($total_amount > 0 && $flag == true){
                $purchase = DeliveryDetails::where('customer_id', $customer_id)->where('payment_status', 0)->first();
                if($purchase !=null){
                    $purchase_amount = $purchase->amount;

                    if($purchase_amount <= $total_amount){
                        $purchase->payment_status = 1;
                        $purchase->save();

                        $total_amount = $total_amount - $purchase_amount;
                    }else{
                        $flag = false;
                    }
                }else{
                    $flag = false;
                }
            }

            $customer = Customers::find($customer_id);
            $customer->advance_amount = $total_amount;
            $customer->save();

            return $this->sendResponse('','Payment Successfull');
        }else{
            return $this->sendError('Payment Not Added');  
        }
    }


    public function submitPayment(Request $request){

        $input = $request->all();
        $json = utf8_encode($input['data']);
        $payment_details = json_decode($json, true);    
        $validator = Validator::make($payment_details, CustomerPayments::rules());

        if ($validator->fails()) { 
            return $this->sendValidationErrors($validator->errors()); 
        }

        foreach ($payment_details as $key => $input) {
            $customer_id        = $input['customer_id'];
            $amount             = $input['amount'];
            $payment_type       = $input['payment_type'];
            $transaction_id     = $input['transaction_id'];
            $remarks            = $input['remarks'];

            if ($payment = CustomerPayments::create($input)){

                $flag = true;
                $customer_advance_amount = $payment->customer->advance_amount;
                $total_amount  = $customer_advance_amount + $amount;

                while($total_amount > 0 && $flag == true){
                    $purchase = DeliveryDetails::where('customer_id', $customer_id)->where('payment_status', 0)->first();
                    if($purchase !=null){
                        $purchase_amount = $purchase->amount;

                        if($purchase_amount <= $total_amount){
                            $purchase->payment_status = 1;
                            $purchase->save();

                            $total_amount = $total_amount - $purchase_amount;
                        }else{
                            $flag = false;
                        }
                    }else{
                        $flag = false;
                    }
                }

                if($payment_type=="Cash"){
                    $login_user_id = auth('api')->user()->id;
                    $model = Routes::where('route_user_id', $login_user_id)->first();
                    $model->petty_cash += $amount;
                    $model->save();
                }

                $customer = Customers::find($customer_id);
                $customer->advance_amount = $total_amount;
                $customer->save();
            }
        }
        return $this->sendResponse('','Payment Successfull');

    }


    public function deleteDelivery(Request $request){
        if($request->has('route_id') && $request->route_id!= ''){

            $query = DeliveryDetails::leftJoin('customers', 'customers.id', '=', 'delivery_details.customer_id')
            ->where('customers.route_id', '=' ,$request->route_id);

            if($request->has('delivery_date') && $request->delivery_date!= ''){
                $query->whereDate('delivery_details.created_at', '=' ,$request->delivery_date);
            }

            $query->delete();
            return $this->sendResponse('','Delete Successfull');

        }else{
            return $this->sendError('Invalid Route'); 
        }

    }



}
