<?php

namespace App\Http\Controllers;

use DB;
use App\Models\Customers;
use App\Models\Products;
use App\Models\Routes;
use App\Models\CustomerProducts;
use App\Models\DeliveryDetails;
use App\Models\DeliverySummary;
use App\Models\CustomerPayments;
use App\Models\PettyCashDetails;
use App\Models\RouteEmployees;
use App\Models\CustomerImages;
use App\Models\Role;
use Illuminate\Http\Request;
use Validator;
use Exception;
use Illuminate\Support\Facades\Auth; 
use Illuminate\Support\Facades\Log;
use App\Http\Controllers\BaseController as BaseController;
use App\Http\Traits\ProductTrait;

class CustomersController extends BaseController
{
    use ProductTrait;

    public function customerList(Request $request)
    {
        $last_month_end = date('Y-m-d', strtotime('last day of last month'));
        $query = Customers::query();
        // $query->select('customers.*', 'routes.route_name', DB::raw("(SUM(delivery_details.amount) - customers.advance_amount) as pending_amount"))->where('customers.is_delete', 0)->leftJoin('routes', 'routes.id', '=', 'customers.route_id');

        $query->select('customers.*', 'routes.route_name')->where('customers.is_delete', 0)->leftJoin('routes', 'routes.id', '=', 'customers.route_id');

        // $query->leftJoin('delivery_details', function($join) use ($request,$last_month_end){
        //     $join->on('customers.id', '=', 'delivery_details.customer_id');
        //     $join->whereDate('delivery_details.created_at', '<=', $last_month_end);
        //     $join->where('delivery_details.payment_status', 0);
        //     $join->orWhere('delivery_details.created_at', '=', null);
        // });


        if($request->has('route') && $request->route!= ''){
            $query = $query->where('customers.route_id', '=' ,$request->route);
            $query->orderBy('customers.route_position');
        }else{
            $query->orderBy('customers.customer_name');
        }


        if($request->has('search') && $request->search!=''){
            if(is_numeric($request->search)){
                $query = $query->where('customers.phone_number', $request->search);
            }else{
                $query = $query->where('customers.customer_name','like','%'.$request->search.'%');
            }
        }
        // if($request->has('phone') && $request->phone!=''){
        //     $query = $query->where('customers.phone_number', $request->phone);
        // }
        $query->groupBy('customers.id');
        $query->orderBy('customers.route_position');
        
        $customers = $query->paginate(50);
        $result['payment_date'] = $last_month_end;
        $result['customer_list'] = $customers;
            
        // $customers = Customers::select('customers.*', 'routes.route_name')->where('customers.status', 0)->where('customers.is_delete', 0)->orderBy('customer_name')->leftJoin('routes', 'routes.id', '=', 'customers.route_id')->paginate(5);          
        return $this->sendResponse($result,'Customer List');
    }


    public function customerListForManager(Request $request)
    {
        $last_month_end = date('Y-m-d', strtotime('last day of last month'));
        $query = Customers::query();
        $query->select('customers.*', 'routes.route_name')->where('customers.is_delete', 0)->leftJoin('routes', 'routes.id', '=', 'customers.route_id');

        $query->join('route_employees', function($join){
            $join->on('route_employees.route_id', '=', 'customers.route_id');
            $join->where('route_employees.user_id', '=', auth('api')->user()->id);
        });

        if($request->has('route') && $request->route!= ''){
            $query = $query->where('customers.route_id', '=' ,$request->route);
            $query->orderBy('customers.route_position');
        }else{
            $query->orderBy('customers.customer_name');
        }


        if($request->has('search') && $request->search!=''){
            if(is_numeric($request->search)){
                $query = $query->where('customers.phone_number', $request->search);
            }else{
                $query = $query->where('customers.customer_name','like','%'.$request->search.'%');
            }
        }
      
        $query->groupBy('customers.id');
        $query->orderBy('customers.route_position');
        
        $customers = $query->paginate(50);
        $result['payment_date'] = $last_month_end;
        $result['customer_list'] = $customers;
        return $this->sendResponse($result,'Route Manager Customer List');
    }

    public function viewCustomerForManager($id)
    {
        $customer = Customers::select('customers.*', 'routes.route_name')->where('customers.id', $id)->where('customers.is_delete', 0)
        ->groupBy('customers.id')
                        //->join('customer_products', 'customer_products.customer_id', '=', 'customers.id')
        ->leftJoin('routes', 'routes.id', '=', 'customers.route_id')
        ->first();

        $check_employee_route = null;

        if($customer){
            $check_employee_route = RouteEmployees::where('user_id','=', auth('api')->user()->id)->where('route_id','=',$customer->route_id)->first();
        }
        
        

        $delivery_date = date('Y-m-d');
        $product_details = CustomerProducts::select('customer_products.*', 'products.product_name')->where('customer_products.customer_id', $id)
                    ->leftJoin('products', 'products.id', '=', 'customer_products.product_id')
                    ->where('delivery_mode','!=','order')->get();

        
        if($customer && $check_employee_route){

            $customer_data = $customer->toArray();

            $prev_customer = Customers::select('customers.id', 'customers.customer_name')->where('route_position', $customer->route_position-1)->where('route_id',$customer->route_id)->first();
            if(!empty($prev_customer)){
                $customer_data['nearest_customer_id'] = $prev_customer['id'];
                $customer_data['nearest_customer_name']     = $prev_customer['customer_name'];
                $customer_data['position_type']     = 1;
            }else{
                $after_customer = Customers::select('customers.id', 'customers.customer_name')->where('route_position', $customer->route_position+1)->where('route_id',$customer->route_id)->first();
                if(!empty($after_customer)){
                    $customer_data['nearest_customer_id'] = $after_customer['id'];
                    $customer_data['nearest_customer_name']     = $after_customer['customer_name'];
                    $customer_data['position_type']     = 0;
                }else{
                    $customer_data['nearest_customer_id'] = 0;
                    $customer_data['nearest_customer_name']     = '';
                    $customer_data['position_type']     = 0;
                }
            }
            

            $result = ['customer_details'=>$customer_data, 'products'=>$product_details->toArray()];
            return $this->sendResponse($result, 'Customer Details');
        }else{
            return $this->sendError('Customer Not Found');  
        }
    }

    public function addCustomer(Request $request)
    {
        $input = $request->all();
        $id     = isset($input['id'])?$input['id']:0;
        $route_id = $input['route_id'];

        $customer_products  = json_decode($input['products'], true);

        $validator = Validator::make($input, Customers::rules($id, $route_id));
        //$product_validator = Validator::make($customer_products, Customers::pro_rules($id));
        if ($validator->fails()) { 
            return $this->sendValidationErrors($validator->errors());         
        }

        $input              = $request->all(); 
        $nearest_customer_id   = $input['nearest_customer_id']; //previous customer id
        $route_id           = $input['route_id'];
        $customer_products  = json_decode($input['products'], true); 
        

        
        if($nearest_customer_id == 0){
            $input['route_position'] = 1;
        }
        if(!$nearest_customer_id == 0){
            $model = Customers::find($nearest_customer_id);
            $near_position = $model->route_position;

            if($input['position_type']==0){ //before customer
                $input['route_position'] = $near_position;
            }else{
                $input['route_position'] = $near_position+1;
            }
        }

        if ($customer = Customers::create($input)){

            //save customer products
            if(!empty($customer_products)){
                foreach ($customer_products as $data) {
                    $data['customer_id'] = $customer->id;
                    CustomerProducts::create($data);
                }
            }

            $sql = 'UPDATE customers SET route_position = (route_position+1) WHERE route_position >='.$customer->route_position.' AND route_id ='.$route_id.' AND id !='.$customer->id;
            $update = DB::update($sql);

            return $this->sendResponse('','Customer Successfully Added'); 
        }else{
            return $this->sendError('Customer Not Added');   
        }
    }

    
    public function updateCustomer(Request $request)
    {
        $input  = $request->all(); 
        $id     = isset($input['id'])?$input['id']:0;
        $route_id = $input['route_id'];
        $customer = Customers::find($id);
        if($customer){
            $validator = Validator::make($input, Customers::rules($id, $route_id));
            if ($validator->fails()) { 
                return $this->sendValidationErrors($validator->errors()); 
            }
            
            $nearest_customer_id   = $input['nearest_customer_id']; //nearest customer id
            $route_id           = $input['route_id'];
            $customer_products  = json_decode($input['products'], true); 

            if($nearest_customer_id == 0){ //no customers in this route
                $input['route_position'] = 1;
            }

            if(!$nearest_customer_id == 0){
                $model = Customers::find($nearest_customer_id);
                $near_position = $model->route_position;

                if($input['position_type']==0){ //before customer
                    $input['route_position'] = $near_position;
                }else{
                    $input['route_position'] = $near_position+1;
                }
            }



            $updated = $customer->fill($input)->save();
            $old_products_ids = array();
            
            if($updated){

                $old_products_ids = DB::table('customer_products')->where('delivery_mode','!=','order')->where('customer_id', $customer->id)->pluck('id')->toArray();
                //save customer products
                if(!empty($customer_products)){
                    $exist_ids = [];
                    foreach ($customer_products as $data) {
                        $data['customer_id'] = $customer->id;
                        if(isset($data['id'])){ //update old product details

                            $cust_prod = DB::table('customer_products')->where('id', $data['id'])->update($data);

                            $exist_ids[] = $data['id'];
                        }else{
                            //add new products
                            CustomerProducts::create($data);
                        }
                    }


                    $deleted_ids = array_diff($old_products_ids, $exist_ids);
                    $deleted_items = DB::table('customer_products')->whereIn('id', $deleted_ids)->delete(); 

                }

                $sql = 'UPDATE customers SET route_position = (route_position+1) WHERE route_position >='.$customer->route_position.' AND route_id ='.$route_id.' AND id !='.$customer->id;
                $update = DB::update($sql);

                //reorder position
                $route_customers = Customers::select('id', 'route_position')->where('route_id', $route_id)->orderBy('route_position')->get();
                foreach ($route_customers as $key => $customer_data) {
                    $customer_data->update(['route_position' => $key+1]);
                }


                return $this->sendResponse($customer,'Customer Successfully Updated');
            }else{
                return $this->sendError('Customer Not Updated');  
            }
        }else{
            return $this->sendError('Customer Not Found');  
        }
    }

    public function viewCustomer($id)
    {
        $customer = Customers::select('customers.*', 'routes.route_name')->where('customers.id', $id)->where('customers.is_delete', 0)
        ->groupBy('customers.id')
                        //->join('customer_products', 'customer_products.customer_id', '=', 'customers.id')
        ->leftJoin('routes', 'routes.id', '=', 'customers.route_id')
        ->first();

        //$product_details = CustomerProducts::select('customer_products.*', 'products.product_name')->where('customer_products.customer_id', $id)->leftJoin('products', 'products.id', '=', 'customer_products.product_id')->get();

        $delivery_date = date('Y-m-d');
        $product_details = CustomerProducts::select('customer_products.*', 'products.product_name')->where('customer_products.customer_id', $id)
                    ->leftJoin('products', 'products.id', '=', 'customer_products.product_id')
                    ->where('delivery_mode','!=','order')->get();

        
        if($customer){

            $customer_data = $customer->toArray();

            $prev_customer = Customers::select('customers.id', 'customers.customer_name')->where('route_position', $customer->route_position-1)->where('route_id',$customer->route_id)->first();
            if(!empty($prev_customer)){
                $customer_data['nearest_customer_id'] = $prev_customer['id'];
                $customer_data['nearest_customer_name']     = $prev_customer['customer_name'];
                $customer_data['position_type']     = 1;
            }else{
                $after_customer = Customers::select('customers.id', 'customers.customer_name')->where('route_position', $customer->route_position+1)->where('route_id',$customer->route_id)->first();
                if(!empty($after_customer)){
                    $customer_data['nearest_customer_id'] = $after_customer['id'];
                    $customer_data['nearest_customer_name']     = $after_customer['customer_name'];
                    $customer_data['position_type']     = 0;
                }else{
                    $customer_data['nearest_customer_id'] = 0;
                    $customer_data['nearest_customer_name']     = '';
                    $customer_data['position_type']     = 0;
                }
            }
            

            $result = ['customer_details'=>$customer_data, 'products'=>$product_details->toArray()];
            return $this->sendResponse($result, 'Customer Details');
        }else{
            return $this->sendError('Customer Not Found');  
        }
    }

    public function deleteCustomer($id)
    {
        $customer = Customers::find($id);

        if($customer){
            $customer->is_delete = 1;
            if($customer->save()){
                return $this->sendResponse('','Customer Successfully Deleted');
            }else{
                return $this->sendError('Customer Not Deleted');  
            }
        }else{
            return $this->sendError('Customer Not Found');  
        } 
    }

    public function updateCustomerStatus(Request $request)
    {
        $input  = $request->all(); 
        $id     = isset($input['id'])?$input['id']:0;
        $status = isset($input['status'])?$input['status']:1;
        $inactive_type = $input['inactive_type'];
        $customer = Customers::find($id);

        if($customer){
            $customer->status = $status;
            $customer->inactive_type = $inactive_type;
            if(isset($input['inactive_till_date'])){
                $customer->inactive_till_date   = $input['inactive_till_date'];
                $customer->inactive_date        = date('Y-m-d');
            }

            if(isset($input['inactive_from_date'])){
                $customer->inactive_from_date   = $input['inactive_from_date'];
                $customer->inactive_date        = $input['inactive_from_date'];
            }

            if($status == 1){
                $customer->inactive_till_date = NULL;
                $customer->inactive_date      = NULL;
                $customer->inactive_from_date = NULL;
                $customer->inactive_type      = NULL;
            }
            if($customer->save()){
                return $this->sendResponse('','Customer Status Changed Successfully');
            }else{
                return $this->sendError('Status Not Changed');  
            }
        }else{
            return $this->sendError('Customer Not Found');  
        }
    }

    public function submitDelivery(Request $request){

        $input = $request->all();
        
        $customer_products  = json_decode($input['data'], true);
        $delivery_date      = isset($input['delivery_date'])?$input['delivery_date']:date('Y-m-d');
        $delivery_date_time = $delivery_date." ".date('H:i:s');

        $validator = Validator::make($customer_products, DeliveryDetails::rules());

        if ($validator->fails()) { 
            return $this->sendValidationErrors($validator->errors()); 
        }

        $user = Auth::user();
        $route_model = Routes::where('route_user_id', $user->id)->first();

        Log::channel('delivery')->info("Route Name : ".$route_model->route_name);
        Log::channel('delivery')->info($input);

        $delivery_details = [];

        $product_price = Products::all()->keyBy('id')->toArray();
        foreach ($customer_products as $key => $delivery_data) {
            $customer_id = $delivery_data['customer_id'];
            $delivery_products = $delivery_data['details'];
            foreach ($delivery_products as $key => $product_data) {
                $product_id = $product_data['product_id'];
                $quantity   = $product_data['quantity'];
                $actual_quantity   = $product_data['actual_quantity'];


                $supply_date = $delivery_date;
                $exist = DeliveryDetails::where([ 
                    ['customer_id', '=', $customer_id],
                    ['product_id', '=', $product_id],
                    ['quantity', '=', $quantity],
                ])
                ->whereDate('created_at', '=' ,$supply_date)->first();
                if(!$exist){
                    $model = new DeliveryDetails;
                    $model->customer_id = $customer_id;
                    $model->product_id  = $product_id;
                    $model->quantity    = $quantity;
                    $model->actual_quantity    = $actual_quantity;
                    $model->amount      = isset($product_price[$product_id]['price'])?($product_price[$product_id]['price'] * $quantity):0;
                    $model->payment_status = 0;
                    $model->created_at = $delivery_date_time;
                    $model->save();

                    //$delivery_details[$product_id] =  isset($delivery_details[$product_id])? ($delivery_details[$product_id] + $quantity) : $quantity;

                }       
                
            }
        }

        $delivery_details = $this->routeDeliveryProductCount($route_model->id, $delivery_date);

        $date   = $delivery_date;
        $products = $this->routeDayProducts($date, $user->id);
        $actual_products = $products->toArray();

        $actual_product_ids = array_column($actual_products, 'id');
        $delivered_product_ids = array_keys($delivery_details); //key of this array

        $all_product_ids = array_unique(array_merge($actual_product_ids, $delivered_product_ids), SORT_REGULAR);

        //print("<pre>"); print_r($products->toArray()); print("</pre>"); 
        //print("<pre>"); print_r($delivery_details); print("</pre>");
        //print("<pre>"); print_r($all_product_ids); print("</pre>"); exit; exit;
        foreach ($all_product_ids as $pid) {

            $index = array_search($pid, array_column($actual_products, 'id'));
            if(false !== $index){
                $actual_product_quantity = $actual_products[$index]['quantity'];
            }else{
                $actual_product_quantity = 0;
            }
            $delivered_quantity = isset($delivery_details[$pid])?$delivery_details[$pid]:0;

            $summary_exist = DeliverySummary::where([ 
                ['route_id', '=', $route_model->id],
                ['product_id', '=', $pid],
            ])
            ->whereDate('created_at', '=' ,$delivery_date)->first();
            if(!$summary_exist){
                $summary_model = new DeliverySummary;
                $summary_model->route_id = $route_model->id;
                $summary_model->product_id = $pid;
                $summary_model->delivered_quantity = $delivered_quantity;
                $summary_model->actual_quantity = $actual_product_quantity;
                $summary_model->created_at = $delivery_date_time;
                $summary_model->save();
            }else{
                $summary_exist->delivered_quantity = $delivered_quantity;
                $summary_exist->actual_quantity = $actual_product_quantity;
                $summary_exist->save();
            }
            
        }
        
        return $this->sendResponse('','Product Delivery Successfull');
    }


    public function productPayment(Request $request){

        $input = $request->all();
        $validator = Validator::make($input, CustomerPayments::rules());

        if ($validator->fails()) { 
            return $this->sendValidationErrors($validator->errors()); 
        }

        $customer_id    = $input['customer_id'];
        $amount         = $input['amount'];
        $payment_type   = $input['payment_type'];

        if ($payment = CustomerPayments::create($input)){

            $flag = true;
            $customer_advance_amount = $payment->customer->advance_amount;
            $total_amount  = $customer_advance_amount + $amount;

            while($total_amount > 0 && $flag == true){
                $purchase = DeliveryDetails::where('customer_id', $customer_id)->where('payment_status', 0)->first();
                if($purchase !=null){
                    $purchase_amount = $purchase->amount;

                    if($purchase_amount <= $total_amount){
                        $purchase->payment_status = 1;
                        $purchase->save();

                        $total_amount = $total_amount - $purchase_amount;
                    }else{
                        $flag = false;
                    }
                }else{
                    $flag = false;
                }
            }

            $customer = Customers::find($customer_id);
            $customer->advance_amount = $total_amount;
            $customer->save();

            return $this->sendResponse('','Payment Successfull');
        }else{
            return $this->sendError('Payment Not Added');  
        }
    }


    public function submitPayment(Request $request){

        $input = $request->all();
        $json = utf8_encode($input['data']);
        $payment_details = json_decode($json, true);    
        $validator = Validator::make($payment_details, CustomerPayments::rules());

        if ($validator->fails()) { 
            return $this->sendValidationErrors($validator->errors()); 
        }

        foreach ($payment_details as $key => $input) {
            $customer_id        = $input['customer_id'];
            $amount             = $input['amount'];
            $payment_type       = $input['payment_type'];
            $transaction_id     = $input['transaction_id'];
            $remarks            = $input['remarks'];

            if ($payment = CustomerPayments::create($input)){

               
                $flag = true;
                $customer_advance_amount = $payment->customer->advance_amount;
                $total_amount  = $customer_advance_amount + $amount;
                $last_paid_date = '';
                while($total_amount > 0 && $flag == true){
                    $purchase = DeliveryDetails::where('customer_id', $customer_id)->where('payment_status', 0)->orderBy('created_at', 'ASC')->first();
                    if($purchase !=null){
                        $purchase_amount = $purchase->amount;

                        if($purchase_amount <= $total_amount){
                            $purchase->payment_status = 1;
                            $purchase->save();

                            $total_amount = $total_amount - $purchase_amount;
                            //$last_paid_date = date('d/m/Y', strtotime($purchase->created_at));
                        }else{
                            $flag = false;
                        }
                    }else{
                        $flag = false;
                    }
                }

                if($payment_type=="Cash"){
                    $login_user_id = auth('api')->user()->id;
                    $model = Routes::where('route_user_id', $login_user_id)->first();
                    $model->petty_cash += $amount;
                    $model->save();
                }

                $last_payment_details = DeliveryDetails::where('customer_id', $customer_id)->where('payment_status', 1)->orderBy('created_at', 'DESC')->first();
                if($last_payment_details!=NULL){
                    $last_paid_date = date('d/m/Y', strtotime($last_payment_details->created_at));
                }

                 $this->sendSms($customer_id, $amount, $last_paid_date, $total_amount);

                $customer = Customers::find($customer_id);
                $customer->advance_amount = $total_amount;
                $customer->save(false);
            }
        }
        return $this->sendResponse('','Payment Successfull');

    }

    public function submitSinglePayment(Request $request){

        $input = $request->all();
        $validator = Validator::make($input, CustomerPayments::single_rules());
        if ($validator->fails()) { 
            return $this->sendValidationErrors($validator->errors()); 
        }
        $advance_amount = 0;
        $petty_cash = 0;
        $result = [];
        $customer_id        = $input['customer_id'];
        $amount             = $input['amount'];
        if($customer_id!=null && $amount!=null) {

            $payment_type       = $input['payment_type'];
            $transaction_id     = $input['transaction_id'];
            $remarks            = $input['remarks'];
            $input['collected_user_id'] = auth('api')->user()->id;
            $input['payment_flag'] = 1;

            $customer = Customers::find($customer_id);
            if($customer){
                $advance_amount = $customer->advance_amount;
            }else{
                return $this->sendError('Invalid Customer'); 
            }

            $input['advance_amount'] = $advance_amount;

            if ($payment = CustomerPayments::create($input)){

                //$this->sendSms($customer_id, $amount);
                $last_paid_date = '';
                $recent_id = $payment->id;
                $this->updateCustomerPayments($customer_id,$recent_id);

                $flag = true;
                $customer_advance_amount = $payment->customer->advance_amount;
                $total_amount  = $customer_advance_amount + $amount;

                while($total_amount > 0 && $flag == true){
                    $purchase = DeliveryDetails::where('customer_id', $customer_id)->where('payment_status', 0)->orderBy('created_at', 'ASC')->first();
                    if($purchase !=null){
                        $purchase_amount = $purchase->amount;

                        if($purchase_amount <= $total_amount){
                            $purchase->payment_status = 1;
                            $purchase->payment_id     = $recent_id;
                            $purchase->save();

                            $total_amount = $total_amount - $purchase_amount;
                             $last_paid_date = date('d/m/Y', strtotime($purchase->created_at));
                        }else{
                            $flag = false;
                        }
                    }else{
                        $flag = false;
                    }
                }

                if($payment_type=="Cash"){
                    $login_user_id = auth('api')->user()->id;
                    $model = Routes::where('route_user_id', $login_user_id)->first();
                    $model->petty_cash += $amount;
                    $model->save();
                    $petty_cash = $model->petty_cash;
                }

                $last_payment_details = DeliveryDetails::where('customer_id', $customer_id)->where('payment_status', 1)->orderBy('created_at', 'DESC')->first();
                if($last_payment_details!=NULL){
                    $last_paid_date = date('d/m/Y', strtotime($last_payment_details->created_at));
                }

                $this->sendSms($customer_id, $amount, $last_paid_date, $total_amount);

                $customer = Customers::find($customer_id);
                $customer->advance_amount = $total_amount;
                $customer->save();
            }
        }
        $result['petty_cash'] = $petty_cash;
        return $this->sendResponse($result,'Payment Successfull');

    }


    public function deleteDelivery(Request $request){
        if($request->has('route_id') && $request->route_id!= ''){

            $query = DeliveryDetails::leftJoin('customers', 'customers.id', '=', 'delivery_details.customer_id')
            ->where('customers.route_id', '=' ,$request->route_id);

            if($request->has('delivery_date') && $request->delivery_date!= ''){
                $query->whereDate('delivery_details.created_at', '=' ,$request->delivery_date);
            }

            $query->delete();
            return $this->sendResponse('','Delete Successfull');

        }else{
            return $this->sendError('Invalid Route'); 
        }

    }


    public function deliveredCount(){
        $this->routeDeliveryProductCount(1, '2021-06-25');
    }


    //from web cash payment - collected by admin
    public function submitAdminPayment(Request $request){

        $input = $request->all();
        $validator = Validator::make($input, CustomerPayments::single_rules());
        if ($validator->fails()) { 
            return $this->sendValidationErrors($validator->errors()); 
        }
        $petty_cash = 0;
        $result = [];
        $customer_id        = $input['customer_id'];
        $amount             = $input['amount'];
        if($customer_id!=null && $amount!=null) {

            $payment_type       = $input['payment_type'];
            $transaction_id     = $input['transaction_id'];
            $remarks            = $input['remarks'];
            $user = Auth::user();
            $input['collected_user_id'] = $user->id;
            $input['payment_flag'] = 1;

            $customer = Customers::find($customer_id);
            if($customer){
                $advance_amount = $customer->advance_amount;
            }else{
                return $this->sendError('Invalid Customer'); 
            }

            $input['advance_amount'] = $advance_amount;

            if ($payment = CustomerPayments::create($input)){

                //$this->sendSms($customer_id, $amount);
                
                $recent_id = $payment->id;
                $this->updateCustomerPayments($customer_id,$recent_id);
                $last_paid_date = '';
                $flag = true;
                $customer_advance_amount = $payment->customer->advance_amount;
                $total_amount  = $customer_advance_amount + $amount;

                while($total_amount > 0 && $flag == true){
                    $purchase = DeliveryDetails::where('customer_id', $customer_id)->where('payment_status', 0)->orderBy('created_at', 'ASC')->first();
                    if($purchase !=null){
                        $purchase_amount = $purchase->amount;

                        if($purchase_amount <= $total_amount){
                            $purchase->payment_status = 1;
                            $purchase->payment_id     = $recent_id;
                            $purchase->save();

                            $total_amount = $total_amount - $purchase_amount;
                            $last_paid_date = date('d/m/Y', strtotime($purchase->created_at));
                        }else{
                            $flag = false;
                        }
                    }else{
                        $flag = false;
                    }
                }

                if($payment_type=="Cash"){
                    $route_id = $customer->route_id;
                    $model = Routes::where('id', $route_id)->first();
                    $model->petty_cash += $amount;
                    if($model->save()){
                        $petty_cash = $model->petty_cash;

                        $petty_cash_model = new PettyCashDetails;
                        $petty_cash_model->route_user_id = $model->route_user_id;
                        $petty_cash_model->amount = $amount;
                        $petty_cash_model->created_at = date('Y-m-d H:i:s');
                        $petty_cash_model->status = 1;
                        $petty_cash_model->save();

                    }
                    
                }

                $last_payment_details = DeliveryDetails::where('customer_id', $customer_id)->where('payment_status', 1)->orderBy('created_at', 'DESC')->first();
                if($last_payment_details!=NULL){
                    $last_paid_date = date('d/m/Y', strtotime($last_payment_details->created_at));
                }

                if($payment_type!= 'Settlement'){
                    $this->sendSms($customer_id, $amount, $last_paid_date, $total_amount);
                }

                $customer = Customers::find($customer_id);
                $customer->advance_amount = $total_amount;
                $customer->save();
            }

            $total_pending_amount = DeliveryDetails::where('delivery_details.payment_status', 0)->where('delivery_details.customer_id', $customer_id)->sum('amount');
            $result['total_pending_amount'] = $total_pending_amount - $customer->advance_amount;
        }

        //$result['petty_cash'] = $petty_cash;
        return $this->sendResponse($result,'Payment Successfull');

    }

    //from web collection - Manager
    public function submitManagerPayment(Request $request){

        $input = $request->all();
        $validator = Validator::make($input, CustomerPayments::single_rules());
        if ($validator->fails()) { 
            return $this->sendValidationErrors($validator->errors()); 
        }
        $petty_cash = 0;
        $result = [];
        $customer_id        = $input['customer_id'];
        $amount             = $input['amount'];
        if($customer_id!=null && $amount!=null) {

            $payment_type       = $input['payment_type'];
            $transaction_id     = $input['transaction_id'];
            $remarks            = $input['remarks'];
            $user = Auth::user();
            $input['collected_user_id'] = $user->id;
            $input['payment_flag'] = 1;

            $customer = Customers::find($customer_id);
            if($customer){
                $advance_amount = $customer->advance_amount;
            }else{
                return $this->sendError('Invalid Customer'); 
            }

            $input['advance_amount'] = $advance_amount;

            if ($payment = CustomerPayments::create($input)){

                //$this->sendSms($customer_id, $amount);
                
                $recent_id = $payment->id;
                $this->updateCustomerPayments($customer_id,$recent_id);
                $last_paid_date = '';
                $flag = true;
                $customer_advance_amount = $payment->customer->advance_amount;
                $total_amount  = $customer_advance_amount + $amount;

                while($total_amount > 0 && $flag == true){
                    $purchase = DeliveryDetails::where('customer_id', $customer_id)->where('payment_status', 0)->orderBy('created_at', 'ASC')->first();
                    if($purchase !=null){
                        $purchase_amount = $purchase->amount;

                        if($purchase_amount <= $total_amount){
                            $purchase->payment_status = 1;
                            $purchase->payment_id     = $recent_id;
                            $purchase->save();

                            $total_amount = $total_amount - $purchase_amount;
                            $last_paid_date = date('d/m/Y', strtotime($purchase->created_at));
                        }else{
                            $flag = false;
                        }
                    }else{
                        $flag = false;
                    }
                }

                // if($payment_type=="Cash"){
                //     $route_id = $customer->route_id;
                //     $model = Routes::where('id', $route_id)->first();
                //     $model->petty_cash += $amount;
                //     if($model->save()){
                //         $petty_cash = $model->petty_cash;

                //         $petty_cash_model = new PettyCashDetails;
                //         $petty_cash_model->route_user_id = $model->route_user_id;
                //         $petty_cash_model->amount = $amount;
                //         $petty_cash_model->created_at = date('Y-m-d H:i:s');
                //         $petty_cash_model->status = 1;
                //         $petty_cash_model->save();

                //     }
                    
                // }

                $petty_cash_model = new PettyCashDetails;
                $petty_cash_model->route_user_id = null; 
                $petty_cash_model->collected_user_id = $user->id; //manager user id
                $petty_cash_model->amount = $amount;
                $petty_cash_model->payment_type = $payment_type;
                $petty_cash_model->transfer_type = 'manager_transfer';
                $petty_cash_model->from_route_manager = 0;
                $petty_cash_model->created_at = date('Y-m-d H:i:s');
                $petty_cash_model->created_by = $user->id;
                $petty_cash_model->status = 1;
                $petty_cash_model->save();

                $last_payment_details = DeliveryDetails::where('customer_id', $customer_id)->where('payment_status', 1)->orderBy('created_at', 'DESC')->first();
                if($last_payment_details!=NULL){
                    $last_paid_date = date('d/m/Y', strtotime($last_payment_details->created_at));
                }

                if($payment_type!= 'Settlement'){
                    $this->sendSms($customer_id, $amount, $last_paid_date, $total_amount);
                }

                $customer = Customers::find($customer_id);
                $customer->advance_amount = $total_amount;
                $customer->save();
            }

            $total_pending_amount = DeliveryDetails::where('delivery_details.payment_status', 0)->where('delivery_details.customer_id', $customer_id)->sum('amount');
            $result['total_pending_amount'] = $total_pending_amount - $customer->advance_amount;
        }

        //$result['petty_cash'] = $petty_cash;
        return $this->sendResponse($result,'Payment Successfull');

    }

    public function deletePayment(Request $request){

        $input = $request->all();
        $payment_id        = $input['payment_id'];
        $petty_cash = 0;
        $model = CustomerPayments::where('id', $payment_id)->first();
        $deleted_by = auth('api')->user()->id;
        
        if($model!=null && $deleted_by == 1){

            $customer_id = $model->customer_id;
            $flag = true;
            $customer = Customers::find($model->customer_id);
            $route_id = $customer->route_id;
            $route_user_id = $customer->route->route_user_id;

            $model->status = 'D';
            $model->deleted_by = $deleted_by;
            if($model->save()){
                //change recent payment flag
                $this->updateCustomerRecentPayment($customer_id);
                //change paid status to unpaid
                $this->unpaidDelivery($customer_id, $model->id);

                //update customer advance amount
                $customer->advance_amount = $model->advance_amount;
                $customer->save();

                $collected_user_id = $model->collected_user_id;
                $role_model = Role::where('user_id', $collected_user_id)->first();
                $route_manager_collection = ($role_model!=null && $role_model->role == 'route_manager')?true:false;


                if($model->payment_type=="cash" && $route_manager_collection == false){
                    $routes_model = Routes::where('route_user_id', $route_user_id)->first();
                    $routes_model->petty_cash = $routes_model->petty_cash - $model->amount;
                    $routes_model->save();
                    $petty_cash = $routes_model->petty_cash;
                }
            }

            $result['petty_cash'] = $petty_cash;
            return $this->sendResponse($result,'Payment Delete Successfull');

        }else{
            return $this->sendError('Invalid Payment');  
        }
        

    }


    public function deletePaymentOld(Request $request){

        $input = $request->all();
        $payment_id        = $input['payment_id'];
        $petty_cash = 0;
        $model = CustomerPayments::where('id', $payment_id)->first();
        $customer_id = $model->customer_id;
        
        if($model!=null){
            $flag = true;
            $customer = Customers::find($model->customer_id);
            $route_id = $customer->route_id;
            $route_user_id = $customer->route->route_user_id;

            $customer_advance_amount = $customer->advance_amount;
            $post_payment = $model->amount;
            $total_amount = $post_payment;

            //$total_amount = $post_payment - $customer_advance_amount;

            while($total_amount > 0 && $flag == true){
                $purchase = DeliveryDetails::where('customer_id', $model->customer_id)->where('payment_status', 1)->orderBy('created_at', 'desc')->orderBy('id', 'desc')->first();
                    if($purchase !=null){
                        $purchase_amount = $purchase->amount;

                        if($purchase_amount <= $total_amount){
                            $purchase->payment_status = 0;
                            $purchase->save();

                            $total_amount = $total_amount - $purchase_amount;
                        }else{
                            if($customer_advance_amount >= $total_amount){
                                $customer_advance_amount = $customer_advance_amount - $total_amount;
                            }else if($customer_advance_amount < $total_amount){
                                $customer_advance_amount = $total_amount - $customer_advance_amount;
                            }
                            $flag = false;
                        }
                    }else{
                        $flag = false;
                    }

                    if($customer_advance_amount >= $total_amount){
                        $customer_advance_amount = $customer_advance_amount - $total_amount;
                        $flag = false;
                    }
            }

            if($model->payment_type=="cash"){
                $routes_model = Routes::where('route_user_id', $route_user_id)->first();
                $routes_model->petty_cash = $routes_model->petty_cash - $post_payment;
                $routes_model->save();
                $petty_cash = $routes_model->petty_cash;
            }

            $customer->advance_amount = $customer_advance_amount;
            $customer->save();

            if($model->delete()){
                $this->updateCustomerRecentPayment($customer_id);
            }

            $result['petty_cash'] = $petty_cash;
            return $this->sendResponse($result,'Payment Delete Successfull');

        }else{
            return $this->sendError('Invalid Payment');  
        }
        

    }

    public function deletePayments(Request $request){

        $input = $request->all();
        $payment_id        = $input['payment_id'];
        $petty_cash = 0;
        $model = CustomerPayments::where('id', $payment_id)->first();
        if($model!=null){
            $flag = true;
            $customer = Customers::find($model->customer_id);
            $route_id = $customer->route_id;
            $route_user_id = $customer->route->route_user_id;

            $customer_advance_amount = $customer->advance_amount;
            $post_payment = $model->amount;

            $total_amount = $post_payment - $customer_advance_amount;

            while($total_amount > 0 && $flag == true){
                $purchase = DeliveryDetails::where('customer_id', $model->customer_id)->where('payment_status', 1)->orderBy('created_at', 'desc')->orderBy('id', 'desc')->first();
                    if($purchase !=null){
                        $purchase_amount = $purchase->amount;

                        if($purchase_amount <= $total_amount){
                            $purchase->payment_status = 0;
                            $purchase->save();

                            $total_amount = $total_amount - $purchase_amount;
                        }else{
                            $flag = false;
                        }
                    }else{
                        $flag = false;
                    }
            }

            if($model->payment_type=="cash"){
                $routes_model = Routes::where('route_user_id', $route_user_id)->first();
                $routes_model->petty_cash = $routes_model->petty_cash - $post_payment;
                $routes_model->save();
                $petty_cash = $routes_model->petty_cash;
            }

            $customer->advance_amount = $total_amount;
            $customer->save();

            $model->delete();

            $result['petty_cash'] = $petty_cash;
            return $this->sendResponse($result,'Payment Delete Successfull');

        }else{
            return $this->sendError('Invalid Payment');  
        }
        

    }

    public function addCustomerLocation(Request $request)
    {
        $input = $request->all();
        $customer_id     = isset($input['customer_id'])?$input['customer_id']:0;

        $validator = Validator::make($input, Customers::location_rules($customer_id));
        if ($validator->fails()) { 
            return $this->sendValidationErrors($validator->errors());         
        }

        $model = Customers::find($customer_id);
        if($model){
            $model->latitude = $input['latitude'];
            $model->longitude = $input['longitude'];
            $model->save();
            return $this->sendResponse('','Customer Location Added'); 
        }else{
            return $this->sendError('Customer Not Added'); 
        }
    }

    public function addCustomerImages(Request $request)
    {
        $input = $request->all();
        $customer_id     = isset($input['customer_id'])?$input['customer_id']:0;

        $validator = Validator::make($input, CustomerImages::rules($customer_id));
        if ($validator->fails()) { 
            return $this->sendValidationErrors($validator->errors());         
        }

        $user = Auth::user();
        $model = Customers::find($customer_id);
        if($model){

            $allowedfileExtension=['pdf','jpg','png'];
            $files = $request->file('file_name'); 
            $errors = [];

            $customer_images = [];
            foreach($request->file_name as $customer_image) {
 
                $name = time().rand(1,50).'.'.$customer_image->extension();
                $customer_image->move(public_path().'/images/customer_images/', $name);  

                $customer_model = new CustomerImages();
                $customer_model->customer_id = $customer_id;
                $customer_model->file_name = $name;
                $customer_model->status = 1;
                $customer_model->created_at = date('Y-m-d H:i:s');
                $customer_model->created_by = $user->id;
                $customer_model->save();
                //print("<pre>"); print_r($customer_model->toArray()); print("</pre>"); exit;
                //$customer_images[] = asset('images/customer_images/').'/'.$name;
                $customer_images[] = $customer_model->toArray();
            }
                
            $result['images'] = $customer_images;
            return $this->sendResponse($result,'Customer Images Added'); 
        }else{
            return $this->sendError('Customer Not Found'); 
        }
    }

    public function deleteCustomerImages(Request $request)
    {
        $input = $request->all();
        $image_id     = isset($input['image_id'])?$input['image_id']:0;

        $user = Auth::user();
        $model = CustomerImages::find($image_id);
        if($model){

            $image_name = $model->getRawOriginal('file_name');
            unlink(public_path().'/images/customer_images/'.$image_name);
            $model->delete();
              
            return $this->sendResponse('','Customer Images Deleted'); 
        }else{
            return $this->sendError('Image Not Found'); 
        }
    }


}
