<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\PettyCashDetails;
use App\Models\Routes;
use App\Models\Role;
use Illuminate\Support\Facades\Auth; 
use Illuminate\Support\Facades\Log;
use Validator;
use Exception;
use App\Http\Controllers\BaseController as BaseController;

class PettyCashController extends BaseController
{
     public function pettyCashList()
    {
        $details = PettyCashDetails::where('status', 1)->where('is_delete', 0)->orderBy('created_at')->paginate(10);          
        return $this->sendResponse($details,'Petty Cash List');
    }

    public function deletePettyCashDetails($id){
        $petty_details = PettyCashDetails::find($id);

        $valid_flag = 0;
        if($petty_details){

            if($petty_details->is_delete == 0 && $petty_details->status == 1){
                $petty_details->is_delete = 1;
                $petty_details->deleted_by = auth('api')->user()->id;
                if($petty_details->save()){

                    if($petty_details->route_user_id != NULL){
                        //add the received amount to route, when route user id is route supplier user id
                        $route_user_id = $petty_details->route_user_id;
                        $role_model = Role::where('user_id', '=', $route_user_id)->first();
                        if($role_model){
                            if($role_model->role == 'supplier'){
                                $route_model = Routes::where('route_user_id', $route_user_id)->first();
                                $route_model->petty_cash = $route_model->petty_cash + $petty_details->amount;
                                if($route_model->save()){
                                    $valid_flag = 1;
                                }else{
                                    $valid_flag = 0;
                                    $petty_details->is_delete = 0;
                                    $petty_details->save();
                                }
                                
                            }else{
                                $valid_flag = 1;
                            }
                        }else{
                            return $this->sendError('Petty Cash Not Deleted - Role not found');
                        }

                        if($valid_flag == 1){
                            Log::channel('petty_cash_delete')->info("Petty Cash Delete From : ".$route_user_id." By : ".auth('api')->user()->id);

                            Log::channel('petty_cash_delete')->info(json_encode($petty_details));

                            return $this->sendResponse('','Petty Cash Successfully Deleted');
                        }else{
                            return $this->sendError('Petty Cash Not Deleted');  
                        }
                    }else{
                        return $this->sendResponse('','Petty Cash Successfully Deleted');
                    }
                    
                    
                }else{
                    return $this->sendError('Petty Cash Not Deleted');  
                }
            }else{
                return $this->sendError('Petty Cash Not Found'); 
            }
            
        }else{
            return $this->sendError('Petty Cash Not Found');  
        }
    }

    public function receivePettyCash(Request $request)
    {

        $input = $request->all(); 
        // $validator = Validator::make($input, PettyCashDetails::rules());

        // if ($validator->fails()) { 
        //     return $this->sendValidationErrors($validator->errors()); 
        // }

        //case 1 - admin collect from supplier
        //case 2 - admin collect for manager from supplier
        //case 3 - admin collect from manager
        //case 4 - manager collect from supplier

        $collected_user_id = auth('api')->user()->id;
        $collected_user_scope = json_decode(auth('api')->user()->role);
        $collected_user_role = $collected_user_scope->role;

        $route_flag = 0;
        $collected_manager_user_id = null;
        $route_user_id = null;
        $collect_from_supplier = 0;
        $admin_transfer_flag = 0;
        $is_route_admin = null;
        $manager_user_id = null;
        
        //collected by admin - for 1, collected by manager - for 0
        $is_admin_collect = ($input['is_admin_collect'] && $input['is_admin_collect']==1)?$input['is_admin_collect']:0;
        if(isset($input['route_admin_id'])){
            $is_route_admin = ($input['route_admin_id']!=null)?$input['route_admin_id']:null; //collect from manager to admin
        }
        
        if(isset($input['manager_user_id'])){
            $manager_user_id = ($input['manager_user_id']!=null)?$input['manager_user_id']:null;
        }
        
        $input['from_route_manager'] = 0;
        //admin collect from supplier
        if($is_admin_collect && $is_route_admin == null){
            $input['route_user_id'] = $input['route_user_id'];
            $input['collected_user_id'] = $collected_user_id;
            $collect_from_supplier = 1;
            $admin_transfer_flag = 1;
            $transfer_type = 'admin_transfer';
            Log::channel('petty_cash')->info("Petty Cash Collection : admin collect from supplier");

        }else if($is_admin_collect ==0 &&  $manager_user_id!=null){ //admin collect for manager
            $input['route_user_id'] = $input['route_user_id'];
            $input['collected_user_id'] = $manager_user_id;
            $collect_from_supplier = 1;
            $transfer_type = 'manager_transfer';
            Log::channel('petty_cash')->info("Petty Cash Collection : admin collect for manager");

        }else if($manager_user_id!=null && $collected_user_id ==  $manager_user_id){ //manager collect from supplier
            $input['route_user_id'] = $input['route_user_id'];
            $input['collected_user_id'] = $manager_user_id;
            $collect_from_supplier = 1;
            $transfer_type = 'manager_transfer';
            Log::channel('petty_cash')->info("Petty Cash Collection : manager collect from supplier");

        }else if($is_admin_collect ==1 && $is_route_admin!=null){ //admin collect from manager
            $input['route_user_id'] = $input['route_admin_id'];
            $input['collected_user_id'] = $collected_user_id;
            $input['from_route_manager'] = 1;
            $admin_transfer_flag = 1;
            $transfer_type = 'admin_transfer';
            Log::channel('petty_cash')->info("Petty Cash Collection : admin collect from manager");
        }

        $route_user_id = ($input['route_user_id'] && $input['route_user_id']!=null)?$input['route_user_id']:null;
       
        $input['transfer_type'] = $transfer_type;
        $input['created_by'] = auth('api')->user()->id;

        //return json_encode($input); exit;

        $validator = Validator::make($input, PettyCashDetails::rules());

        if ($validator->fails()) { 
            return $this->sendValidationErrors($validator->errors()); 
        }

        if($input['collected_user_id'] != null){
            if ($petty = PettyCashDetails::create($input)){

                Log::channel('petty_cash')->info($input);

                if($collect_from_supplier){
                    $model = Routes::where('route_user_id', $petty->route_user_id)->first();
                    if($model){
                        $model->petty_cash = $model->petty_cash - $petty->amount;
                        if($model->save()){
                            Log::channel('petty_cash')->info("Petty cash collected from route supplier : ".$petty->route_user_id);
                        }
                    }
                }
                
                return $this->sendResponse('','Petty Cash Successfully Added');
            }else{
                return $this->sendError('Petty Cash Not Added');  
            }
        }else{
            Log::channel('petty_cash_error')->info("Petty cash error by the collected user : ".$collected_user_id);
            return $this->sendError('Petty Cash Not Added');  
        }

        
    }

    public function receivePettyCashForManager(Request $request)
    {

        $input = $request->all(); 
        // $validator = Validator::make($input, PettyCashDetails::route_admin_rules());

        // if ($validator->fails()) { 
        //     return $this->sendValidationErrors($validator->errors()); 
        // }

        $collected_user_id = auth('api')->user()->id;
        $collected_user_scope = json_decode(auth('api')->user()->role);
        $collected_user_role = $collected_user_scope->role;

        $input['route_user_id'] = $input['route_admin_user_id'];
        $input['collected_user_id'] = $collected_user_id;
        $input['from_route_manager'] = 0;
        $input['transfer_type'] = 'route_admin_transfer';
        $input['created_by'] = auth('api')->user()->id;

        $validator = Validator::make($input, PettyCashDetails::route_admin_rules());

        if ($validator->fails()) { 
            return $this->sendValidationErrors($validator->errors()); 
        }
        if($input['collected_user_id'] != null){
            if ($petty = PettyCashDetails::create($input)){

                Log::channel('petty_cash')->info("Petty Cash Collection : Route Manager collect from another manager");
                Log::channel('petty_cash')->info($input);

                return $this->sendResponse('','Petty Cash Successfully Transffered');
            }else{
                return $this->sendError('Petty Cash Not Added');  
            }
        }else{
            Log::channel('petty_cash_error')->info("Petty cash error by the collected user : ".$collected_user_id);
            return $this->sendError('Petty Cash Not Added');  
        }
    }




    public function managerPettyCashReceive(Request $request)
    {

        $input = $request->all(); 
        // $validator = Validator::make($input, PettyCashDetails::manager_rules());

        // if ($validator->fails()) { 
        //     return $this->sendValidationErrors($validator->errors()); 
        // }

        $transfer_type = 'manager_transfer';
        $collected_user_id = auth('api')->user()->id;
        $collected_user_scope = json_decode(auth('api')->user()->role);
        $collected_user_role = $collected_user_scope->role;

        $input['route_user_id'] = $input['route_user_id'];
        $input['collected_user_id'] = $collected_user_id;
        $input['from_route_manager'] = 0;
        $input['transfer_type'] = $transfer_type;
        $input['created_by'] = auth('api')->user()->id;

        $route_user_id = ($input['route_user_id'] && $input['route_user_id']!=null)?$input['route_user_id']:null;

        $validator = Validator::make($input, PettyCashDetails::manager_rules());

        if ($validator->fails()) { 
            return $this->sendValidationErrors($validator->errors()); 
        }

        if($input['collected_user_id'] != null){
            if ($petty = PettyCashDetails::create($input)){ 

                Log::channel('petty_cash')->info("Petty Cash Collection : Route Manager collect from supplier");
                Log::channel('petty_cash')->info($input);

                $model = Routes::where('route_user_id', $petty->route_user_id)->first();
                if($model){
                    $model->petty_cash = $model->petty_cash - $petty->amount;
                    if($model->save()){
                        Log::channel('petty_cash')->info("Petty cash collected from route supplier : ".$petty->route_user_id);
                    }
                }            
                return $this->sendResponse('','Petty Cash Successfully Added');
            }else{
                return $this->sendError('Petty Cash Not Added');  
            }
        }else{
            Log::channel('petty_cash_error')->info("Petty cash error by the collected user : ".$collected_user_id);
            return $this->sendError('Petty Cash Not Added');
        }
    }

    public function receivePettyCashCopy(Request $request)
    {

        $input = $request->all(); 
        $validator = Validator::make($input, PettyCashDetails::rules());

        if ($validator->fails()) { 
            return $this->sendValidationErrors($validator->errors()); 
        }

        //case 1 - admin collect from supplier
        //case 2 - admin collect for manager from supplier
        //case 3 - admin collect from manager
        //case 4 - manager collect from supplier

        $collected_user_id = auth('api')->user()->id;
        $collected_user_scope = json_decode(auth('api')->user()->role);
        $collected_user_role = $collected_user_scope->role;

        $route_flag = 0;
        $collected_manager_user_id = null;
        $route_user_id = null;
        $collect_from_supplier = 0;
        $admin_transfer_flag = 0;
        
        //collected by admin - for 1, collected by manager - for 0
        $is_admin_collect = ($input['is_admin_collect'] && $input['is_admin_collect']==1)?$input['is_admin_collect']:0;
        $is_route_admin = ($input['route_admin_id'] && $input['route_admin_id']!=null)?$input['route_admin_id']:0; //collect from manager to admin

        $manager_user_id = ($input['manager_user_id'] && $input['manager_user_id']!=null)?$input['manager_user_id']:null;

        //admin collect from supplier
        if($is_admin_collect && $is_route_admin == 0){
            $input['route_user_id'] = $input['route_user_id'];
            $input['collected_user_id'] = $collected_user_id;
            $collect_from_supplier = 1;
            $admin_transfer_flag = 1;
            $transfer_type = 'admin_transfer';

        }else if($is_admin_collect ==0 &&  $manager_user_id!=null){ //admin collect for manager
            $input['route_user_id'] = $input['route_user_id'];
            $input['collected_user_id'] = $manager_user_id;
            $collect_from_supplier = 1;
            $transfer_type = 'manager_transfer';

        }else if($manager_user_id!=null && $collected_user_id ==  $manager_user_id){ //manager collect from supplier
            $input['route_user_id'] = $input['route_user_id'];
            $input['collected_user_id'] = $manager_user_id;
            $collect_from_supplier = 1;
            $transfer_type = 'manager_transfer';

        }else if($is_admin_collect ==1 && $is_route_admin!=null && $is_route_admin!=0){
            $input['route_user_id'] = $input['route_admin_id'];
            $input['collected_user_id'] = $collected_user_id;
            $admin_transfer_flag = 1;
            $transfer_type = 'admin_transfer';
        }

        $route_user_id = ($input['route_user_id'] && $input['route_user_id']!=null)?$input['route_user_id']:null;
        if(in_array($collected_user_role, array('admin','super_admin'))){

            //admin collect from route user
            if($route_user_id!=null && $for_manager==0 && $is_admin_collect==1){
                $route_flag = 1;
                $transfer_type = 'admin_transfer';
                $input['collected_user_id'] = $collected_user_id;
            }else if($for_manager==1 && $collected_manager_user_id!=null){
                $route_flag = 1;
                $transfer_type = 'manager_transfer';
                $input['collected_user_id'] = $collected_manager_user_id;
            }

        }
        

        $transfer_type = null;
        $is_admin_collection = ($input['is_admin']==1)?$input['is_admin']:0;
        if($is_admin_collection){
            //collected by admin
            $collected_user_id = auth('api')->user()->id;
            $collected_user_scope = json_decode(auth('api')->user()->role);
            $collected_user_role = $collected_user_scope->role;
        }else{
            $collected_user_id = $input['manger_user_id'];
            $role_model = Role::select('role')->where('user_id', $collected_user_id)->first();
            $collected_user_role = $role_model->role;
        }
        
        if(in_array($collected_user_role, array('admin','super_admin'))){
            $transfer_type = 'admin_transfer';
        }else if(in_array($collected_user_role, array('route_manager'))){
            $transfer_type = 'manager_transfer';
        }
        
        $input['transfer_type'] = $transfer_type;
        $input['created_by'] = auth('api')->user()->id;

        if ($petty = PettyCashDetails::create($input)){

            if($transfer_type == 'manager_transfer'){
                $model = Routes::where('route_user_id', $petty->route_user_id)->first();
                if($model){
                    $model->petty_cash = $model->petty_cash - $petty->amount;
                    $model->save();
                }
            }
            
            return $this->sendResponse('','Petty Cash Successfully Added');
        }else{
            return $this->sendError('Petty Cash Not Added');  
        }
    }

   
    
    public function updatePettyCash(Request $request)
    {

        $input  = $request->all(); 
        $id     = isset($input['id'])?$input['id']:0;
        $details = PettyCashDetails::find($id);
        if($details){
            $validator = Validator::make($input, PettyCashDetails::rules($id));
            if ($validator->fails()) { 
                return $this->sendValidationErrors($validator->errors()); 
            }
            
            $updated = $details->fill($request->all())->save();

            if($updated){
                return $this->sendResponse($details,'Petty Cash Successfully Updated');
            }else{
                return $this->sendError('Petty Cash Not Updated');  
            }
        }else{
            return $this->sendError('Petty Cash Not Found');  
        }
        
    }

    public function deletePettyCash($id)
    {
        $details = PettyCashDetails::find($id);

        if($details){
            if($details->delete()){
                return $this->sendResponse('','Petty Cash Successfully Deleted');
            }else{
                return $this->sendError('Petty Cash Not Deleted');  
            }
        }else{
            return $this->sendError('Petty Cash Not Found');  
        }
    }

    public function viewPettyCash($id)
    {
        $details = PettyCashDetails::find($id);

        if($details){
                return $this->sendResponse($details->toArray(),'Petty Cash Details');
        }else{
            return $this->sendError('Petty Cash Not Found');  
        }
    }

    
}
