<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\Products;
use Validator;
use Exception;
use App\Http\Controllers\BaseController as BaseController;

class ProductsController extends BaseController
{
    
    public function productList()
    {
        $products = Products::where('status', 1)->where('is_delete', 0)->orderBy('product_name')->paginate(10);          
        return $this->sendResponse($products,'Product List');
    }


    public function productListData()
    {
        $products = Products::select('id', 'product_name')->where('status', 1)->where('is_delete', 0)->orderBy('product_name')->get();
        return $this->sendResponse($products,'Product List');
    }


    public function addProduct(Request $request)
    {

        $input = $request->all(); 
        $validator = Validator::make($input, Products::rules());

        if ($validator->fails()) { 
            return $this->sendValidationErrors($validator->errors()); 
        }

        
        if ($route = Products::create($input)){
            return $this->sendResponse('','Product Successfully Added');
            //return response()->json(['success'=>true,'message' => 'Route Successfully Added'], $this->successStatus);
        }else{
            return $this->sendError('Product Not Added');  
        }
    }

    public function updateProducts(Request $request, $id)
    {
    }

    
    public function updateProduct(Request $request)
    {

        $input  = $request->all(); 
        $id     = isset($input['id'])?$input['id']:0;
        $products = Products::find($id);
        if($products){
            $validator = Validator::make($input, Products::rules($id));
            if ($validator->fails()) { 
                return $this->sendValidationErrors($validator->errors()); 
            }
            
            $updated = $products->fill($request->all())->save();

            if($updated){
                return $this->sendResponse($products,'Product Successfully Updated');
            }else{
                return $this->sendError('Product Not Updated');  
            }
        }else{
            return $this->sendError('Product Not Found');  
        }
        
    }

    public function deleteProductOld($id)
    {
        $product = Products::find($id);

        if($product){
            if($product->delete()){
                return $this->sendResponse('','Product Successfully Deleted');
            }else{
                return $this->sendError('Product Not Deleted');  
            }
        }else{
            return $this->sendError('Product Not Found');  
        }
    }

    public function deleteRoute($id)
    {
        $product = Products::find($id);
        if($product){
            $product->is_delete = 1;
            if($product->save()){
                return $this->sendResponse('','Product Successfully Deleted');
            }else{
                return $this->sendError('Product Not Deleted');  
            }
        }else{
            return $this->sendError('Product Not Found');  
        }
    }

    public function viewProduct($id)
    {
        $product = Products::find($id);

        if($product){
                return $this->sendResponse($product->toArray(),'Product Details');
        }else{
            return $this->sendError('Product Not Found');  
        }
    }
}
