<?php

namespace App\Http\Controllers;

use DB;
use PDF;
use App\Models\Routes;
use App\Models\User; 
use App\Models\Role; 
use App\Models\Customers;
use App\Models\Products;
use App\Models\CustomerPayments;
use App\Models\DeliveryDetails;
use App\Models\DeliverySummary;
use App\Models\PettyCashDetails;
use Illuminate\Http\Request;
use App\Http\Controllers\BaseController as BaseController;


class ReportsController extends BaseController
{
	private $objCustomer;

	public function __construct(Customers $customers){
		$this->objCustomer = $customers;
	}

	public function routeCollection(Request $request){

		$result = [];
		$query = CustomerPayments::query();
		$query->select('customer_payments.*', 'customers.customer_name', 'users.name as collected_user');

		$query->leftJoin('customers', function($join){
			$join->on('customers.id', '=', 'customer_payments.customer_id');
		});

		$query->leftJoin('users', function($join){
			$join->on('users.id', '=', 'customer_payments.collected_user_id');
		});

		if($request->has('customer_name') && $request->customer_name!=''){
            $query = $query->where('customers.customer_name','like','%'.$request->customer_name.'%');
        }

		if($request->has('start_date') && $request->start_date!= ''){
			$query = $query->whereDate('customer_payments.created_at', '>=' ,$request->start_date);
		}
		if($request->has('end_date') && $request->end_date!= ''){
			$query = $query->whereDate('customer_payments.created_at', '<=' ,$request->end_date);
		}
		if($request->has('customer_pay_type') && $request->customer_pay_type!= 'all' && $request->customer_pay_type!= ''){
			$query = $query->where('customer_payments.payment_type', '=' ,$request->customer_pay_type);
		}
		if($request->has('route_id') && $request->route_id!= ''){
			$query = $query->where('customers.route_id', '=' ,$request->route_id);
		}

		$query = $query->where('customer_payments.status', '=' ,'A');

		$query->orderBy('customer_payments.created_at','desc');
		$collection = $query->paginate(50);

		$amount = $query->sum('customer_payments.amount');

		$result['collection_amount'] 	= $amount;
		$result['collection_data'] 		= $collection;
		return $this->sendResponse($result,'Collection List');
		

	}

	public function routeCollectionForManager(Request $request){

		$result = [];
		$query = CustomerPayments::query();
		$query->select('customer_payments.*', 'customers.customer_name', 'users.name as collected_user');

		$query->leftJoin('customers', function($join){
			$join->on('customers.id', '=', 'customer_payments.customer_id');
		});

		$query->leftJoin('users', function($join){
			$join->on('users.id', '=', 'customer_payments.collected_user_id');
		});

		if($request->has('customer') && $request->customer!=''){
            $query = $query->where('customers.customer_name','like',$request->customer.'%');
        }

		if($request->has('start_date') && $request->start_date!= ''){
			$query = $query->whereDate('customer_payments.created_at', '>=' ,$request->start_date);
		}
		if($request->has('end_date') && $request->end_date!= ''){
			$query = $query->whereDate('customer_payments.created_at', '<=' ,$request->end_date);
		}
		if($request->has('payment_type') && $request->payment_type!= 'all'){
			$query = $query->where('customer_payments.payment_type', '=' ,$request->payment_type);
		}
		if($request->has('route_id') && $request->route_id!= ''){
			$query = $query->where('customers.route_id', '=' ,$request->route_id);
		}

		$query = $query->where('customer_payments.status', '=' ,'A');

		$query->orderBy('customer_payments.created_at','desc');
		$collection = $query->paginate(50);

		$amount = $query->sum('customer_payments.amount');

		$result['collection_amount'] 	= $amount;
		$result['collection_data'] 		= $collection;
		return $this->sendResponse($result,'Collection List');
		

	}


	public function salesReports(Request $request){

		$result = [];
		$query = Customers::query();
		$query->select('customers.id', 'customers.customer_name', 'customers.phone_number', 'customers.address', 'customers.customer_pay_type', DB::raw("(SUM(delivery_details.amount) - customers.advance_amount) as pending_amount"));

		$query->leftJoin('delivery_details', function($join) use ($request){
			$join->on('customers.id', '=', 'delivery_details.customer_id');

			if($request->has('start_date') && $request->start_date!= ''){
				$join->whereDate('delivery_details.created_at', '>=' ,$request->start_date);
				//$join->orWhere('delivery_details.created_at', '=', null);
				$join->where('delivery_details.payment_status', 0);
			}

			if($request->has('end_date') && $request->end_date!= ''){
				$join->whereDate('delivery_details.created_at', '<=' ,$request->end_date);
				$join->orWhere('delivery_details.created_at', '=', null);
			}
		});

		$query->leftJoin('customer_payments', function($join){
			$join->on('customers.id', '=','customer_payments.customer_id');
			$join->where('customer_payments.status', '=' ,'A');
			$join->orWhere('customer_payments.customer_id', '=', null);

		});

		
		

		if($request->has('route_id') && $request->route_id!= ''){
			$query->where('customers.route_id', '=' ,$request->route_id);
		}

		if($request->has('customer_pay_type') && $request->customer_pay_type!= ''){
			$query->where('customers.customer_pay_type', '=' ,$request->customer_pay_type);
		}

		$query->groupBy('customers.id');
		$query->orderBy('customers.route_position');
		$collection = $query->paginate(50);

		//$amount = $query->sum('customer_payments.amount');

		//$result['collection_amount'] 	= $amount;
		$result['sales_report'] 		= $collection;
		return $this->sendResponse($result,'Sales Report');
		

	}

	public function salesReportOrginal(Request $request){

		$date = date('Y-m-d');
		$result = [];
		$query = DeliveryDetails::query();
		$query->select('customers.id', 'customers.customer_name', 'customers.phone_number', 'customers.address', 'customers.advance_amount','customers.customer_pay_type', DB::raw("(SUM(delivery_details.amount) - customers.advance_amount) as pending_amount"), DB::raw("SUM(delivery_details.amount) as total_pending_amount")
			);
		$query->selectRaw("(CASE WHEN (customers.status = 1 OR (customers.status = 0 AND customers.inactive_till_date < '$date') ) THEN 1 ELSE 0 END) AS status");
		

		$query->rightJoin('customers', function($join) use ($request, $date){
			$join->on('customers.id', '=', 'delivery_details.customer_id');
			

			$join->where('delivery_details.payment_status', 0);

			if($request->has('start_date') && $request->start_date!= ''){
				$join->whereDate('delivery_details.created_at', '>=' ,$request->start_date);
				//$join->orWhere('delivery_details.created_at', '=', null);
				
			}

			if($request->has('end_date') && $request->end_date!= ''){
				$join->whereDate('delivery_details.created_at', '<=' ,$request->end_date);
				$join->orWhere('delivery_details.created_at', '=', null);
			}
		});
		
		if($request->has('customer_name') && $request->customer_name!= ''){
			$query = $query->where('customers.customer_name','like','%'.$request->customer_name.'%');
		}

		if($request->has('route_id') && $request->route_id!= ''){
			$query->where('customers.route_id', '=' ,$request->route_id);
		}

		if($request->has('customer_pay_type') && $request->customer_pay_type!= ''){
			$query->where('customers.customer_pay_type', '=' ,$request->customer_pay_type);
		}

		if($request->has('valid_pending') && $request->valid_pending == 1){
			$query->having('total_pending_amount', '>' ,0);
		}

		$query->groupBy('customers.id');
		$query->orderBy('customers.route_position');
		$collection = $query->paginate(300);

		$pending_amount = $query->pluck('pending_amount')->all();

		//print("<pre>"); print_r($pending_amount); print("</pre>"); exit;

		//$amount = $query->sum('customer_payments.amount');

		$result['pending_amount'] 	= array_sum($pending_amount);
		$result['sales_report'] 		= $collection;
		return $this->sendResponse($result,'Sales Report');
	}

	public function salesReport(Request $request){

		$date = date('Y-m-d');
		$valid_from = false;
		$valid_to = false;
		if($request->has('start_date') && $request->start_date!= ''){
			$start_date = date('Y-m-d', strtotime($request->start_date)).' 00:00:00';
			$valid_from = true;
		}

		if($request->has('end_date') && $request->end_date!= ''){
			$end_date = date('Y-m-d', strtotime($request->end_date)).' 23:59:59';
			$valid_to = true;
		}
		
		$result = [];
		$query = Customers::query();
		$query->select('customers.id', 'customers.customer_name', 'customers.phone_number', 'customers.address', 'customers.advance_amount','customers.customer_pay_type', 'customers.route_id'
			);
		$query->leftjoin('delivery_details', function($join) use($request){
			$join->on('customers.id', '=', 'delivery_details.customer_id');

			if($request->has('start_date') && $request->start_date!= ''){
				//$join->whereDate('delivery_details.created_at', '>=' ,$request->start_date);
			}
			if($request->has('end_date') && $request->end_date!= ''){
				// $join->where(function($join) use($request, $join){
				// 	$join->whereDate('delivery_details.created_at', '<=' ,$request->end_date)
				// 			->orWhereNull('delivery_details.customer_id');
				// });
				//$join->whereDate('delivery_details.created_at', '<=' ,$request->end_date);
			}

			$join->orWhereNull('delivery_details.customer_id');

			
		});

		if($valid_from && $valid_to){
			$query->selectRaw("SUM(CASE WHEN (delivery_details.payment_status = 0 AND delivery_details.created_at >= '$start_date' AND delivery_details.created_at <= '$end_date') THEN delivery_details.amount ELSE 0 END) - customers.advance_amount AS pending_amount");
			

		}else if($valid_from && !$valid_to){
			$query->selectRaw("SUM(CASE WHEN (delivery_details.payment_status = 0 AND delivery_details.created_at >= '$start_date') THEN delivery_details.amount ELSE 0 END) - customers.advance_amount AS pending_amount");
		}else if(!$valid_from && $valid_to){
			$query->selectRaw("SUM(CASE WHEN (delivery_details.payment_status = 0  AND delivery_details.created_at <= '$end_date') THEN delivery_details.amount ELSE 0 END) - customers.advance_amount AS pending_amount");
		}else{
			$query->selectRaw("SUM(CASE WHEN (delivery_details.payment_status = 0) THEN delivery_details.amount ELSE 0 END) - customers.advance_amount AS pending_amount");
		}

		$query->selectRaw("(CASE WHEN (customers.status = 1 OR (customers.status = 0 AND customers.inactive_till_date < '$date') ) THEN 1 ELSE 0 END) AS status");

		//$query->selectRaw("SUM(CASE WHEN (delivery_details.payment_status = 0) THEN delivery_details.amount ELSE 0 END) - customers.advance_amount AS pending_amount");

		//$query->selectRaw("(CASE WHEN( (SUM(CASE WHEN (delivery_details.payment_status = 0) THEN delivery_details.amount ELSE 0 END) - customers.advance_amount) > 0) THEN (SUM(CASE WHEN (delivery_details.payment_status = 0) THEN delivery_details.amount ELSE 0 END) - customers.advance_amount) ELSE 0 END) AS pending_amount");

		$query->selectRaw("SUM(CASE WHEN (delivery_details.payment_status = 0) THEN delivery_details.amount ELSE 0 END) AS total_pending_amount");
		
		if($request->has('route_id') && $request->route_id!= ''){
			$query->where('customers.route_id', '=' ,$request->route_id);
		}

		//$query->leftJoin('delivery_details', 'delivery_details.customer_id', '=', 'customers.id');
		//$query->where('delivery_details.payment_status', 0);

		
		if($request->has('customer_name') && $request->customer_name!= ''){
			$query->where('customers.customer_name','like','%'.$request->customer_name.'%');
		}

		if($request->has('customer_pay_type') && $request->customer_pay_type!= ''){
			$query->where('customers.customer_pay_type', '=' ,$request->customer_pay_type);
		}

		if($request->has('valid_pending') && $request->valid_pending == 1){
			$query->having('pending_amount', '>' ,0);
		}
		$query->groupBy('customers.id');
		$query->orderBy('customers.route_position');
		$all_query = $query->get();

		$collection = $query->paginate(300);
		$pending_amount = $all_query->pluck('pending_amount')->all();

		$result['pending_amount'] 	= array_sum($pending_amount);
		$result['sales_report'] 		= $collection;
		return $this->sendResponse($result,'Sales Report');
	}

	public function salesReportForManagersOrginal(Request $request){

		$date = date('Y-m-d');
		$result = [];
		$query = DeliveryDetails::query();
		$query->select('customers.id', 'customers.customer_name', 'customers.phone_number', 'customers.address', 'customers.advance_amount','customers.customer_pay_type', DB::raw("(SUM(delivery_details.amount) - customers.advance_amount) as pending_amount"), DB::raw("SUM(delivery_details.amount) as total_pending_amount"));

		$query->selectRaw("(CASE WHEN (customers.status = 1 OR (customers.status = 0 AND customers.inactive_till_date < '$date') ) THEN 1 ELSE 0 END) AS status");

		$query->rightJoin('customers', function($join) use ($request){
			$join->on('customers.id', '=', 'delivery_details.customer_id');

			$join->where('delivery_details.payment_status', 0);

			if($request->has('start_date') && $request->start_date!= ''){
				$join->whereDate('delivery_details.created_at', '>=' ,$request->start_date);
				//$join->orWhere('delivery_details.created_at', '=', null);
				
			}

			if($request->has('end_date') && $request->end_date!= ''){
				$join->whereDate('delivery_details.created_at', '<=' ,$request->end_date);
				$join->orWhere('delivery_details.created_at', '=', null);
			}
		});

		$query->join('route_employees', function($join){
            $join->on('route_employees.route_id', '=', 'customers.route_id');
            $join->where('route_employees.user_id', '=', auth('api')->user()->id);
        });
		
		if($request->has('customer_name') && $request->customer_name!= ''){
			$query = $query->where('customers.customer_name','like','%'.$request->customer_name.'%');
		}

		if($request->has('route_id') && $request->route_id!= ''){
			$query->where('customers.route_id', '=' ,$request->route_id);
		}

		if($request->has('customer_pay_type') && $request->customer_pay_type!= ''){
			$query->where('customers.customer_pay_type', '=' ,$request->customer_pay_type);
		}

		$query->groupBy('customers.id');
		$query->orderBy('customers.route_position');
		$collection = $query->paginate(300);

		$pending_amount = $query->pluck('pending_amount')->all();

		//print("<pre>"); print_r($pending_amount); print("</pre>"); exit;

		//$amount = $query->sum('customer_payments.amount');

		$result['pending_amount'] 	= array_sum($pending_amount);
		$result['sales_report'] 		= $collection;
		return $this->sendResponse($result,'Sales Report');
	}

	public function salesReportForManagers(Request $request){

		$date = date('Y-m-d');
		$start_date = date('Y-m-d', strtotime($request->start_date)).' 00:00:00';
		$result = [];
		$query = Customers::query();
		$query->select('customers.id', 'customers.customer_name', 'customers.phone_number', 'customers.address', 'customers.advance_amount','customers.customer_pay_type', 'customers.route_id'
			);
		$query->leftjoin('delivery_details', function($join) use($request){
			$join->on('customers.id', '=', 'delivery_details.customer_id');

			if($request->has('start_date') && $request->start_date!= ''){
				$join->whereDate('delivery_details.created_at', '>=' ,$request->start_date);
			}
			if($request->has('end_date') && $request->end_date!= ''){
				$join->whereDate('delivery_details.created_at', '<=' ,$request->end_date);
			}
			$join->orWhereNull('delivery_details.customer_id');
		});

		$query->join('route_employees', function($join){
            $join->on('route_employees.route_id', '=', 'customers.route_id');
            $join->where('route_employees.user_id', '=', auth('api')->user()->id);
        });

		$query->selectRaw("(CASE WHEN (customers.status = 1 OR (customers.status = 0 AND customers.inactive_till_date < '$date') ) THEN 1 ELSE 0 END) AS status");

		$query->selectRaw("SUM(CASE WHEN (delivery_details.payment_status = 0) THEN delivery_details.amount ELSE 0 END) - customers.advance_amount AS pending_amount");

		$query->selectRaw("SUM(CASE WHEN (delivery_details.payment_status = 0) THEN delivery_details.amount ELSE 0 END) AS total_pending_amount");
		
		if($request->has('route_id') && $request->route_id!= ''){
			$query->where('customers.route_id', '=' ,$request->route_id);
		}

		
		if($request->has('customer_name') && $request->customer_name!= ''){
			$query->where('customers.customer_name','like','%'.$request->customer_name.'%');
		}

		if($request->has('customer_pay_type') && $request->customer_pay_type!= ''){
			$query->where('customers.customer_pay_type', '=' ,$request->customer_pay_type);
		}

		if($request->has('valid_pending') && $request->valid_pending == 1){
			$query->having('total_pending_amount', '>' ,0);
		}
		$query->groupBy('customers.id');
		$query->orderBy('customers.route_position');
		$collection = $query->paginate(300);
		$pending_amount = $query->pluck('pending_amount')->all();
		$result['pending_amount'] 	= array_sum($pending_amount);
		$result['sales_report'] 		= $collection;
		return $this->sendResponse($result,'Sales Report');
	}

	public function supplyDetails(Request $request){
		if($request->has('customer_id') && $request->customer_id!= ''){

			$result = [];
			$query = DeliveryDetails::query();
			$join_query = $query;
			$query->select('delivery_details.*', 'products.product_name');

			$query->leftJoin('customers', function($join) use ($request){
				$join->on('customers.id', '=', 'delivery_details.customer_id');
			});

			$query->leftJoin('products', function($join) use ($request){
				$join->on('products.id', '=', 'delivery_details.product_id');
			});



			if($request->has('start_date') && $request->start_date!= ''){
				$start_date = date('Y-m-d', strtotime($request->start_date)).' 00:00:00';

				$query->whereDate('delivery_details.created_at', '>=' ,$start_date);
				//$join->orWhere('delivery_details.created_at', '=', null);
			}

			if($request->has('end_date') && $request->end_date!= ''){

				$end_date = date('Y-m-d', strtotime($request->end_date)).' 23:59:59';
				$query->whereDate('delivery_details.created_at', '<=' ,$end_date);
			}

			if($request->has('customer_id') && $request->customer_id!= ''){
				$query->where('customers.id', '=' ,$request->customer_id);
			}


			$query->orderBy('customers.route_position');
			
			$supply_query = $query;

			$collection = $query->paginate(300);

			$customer_data = Customers::select('customers.*')->where('id',$request->customer_id)->first();


			$range_pending_amount = $supply_query->where('delivery_details.payment_status', 0)->where('customers.id', $request->customer_id)->sum('amount');

			$total_pending_amount =DeliveryDetails::where('delivery_details.payment_status', 0)->where('delivery_details.customer_id', $request->customer_id)->sum('amount');

			$result['customer_data'] 		= $customer_data;
			$result['advance_amount'] 		= $customer_data->advance_amount;
			$result['range_pending_amount'] = $range_pending_amount;
			$result['total_pending_amount'] = $total_pending_amount - $customer_data->advance_amount;
			$result['supply_report'] 		= $collection;
			return $this->sendResponse($result,'Supply Report');
		}else{
			return $this->sendError('Invalid Customer'); 
		}
	}

	public function supplyDetailsForManager(Request $request){
		if($request->has('customer_id') && $request->customer_id!= ''){

			$result = [];
			$query = DeliveryDetails::query();
			$join_query = $query;
			$query->select('delivery_details.*', 'products.product_name');

			$query->rightJoin('customers', function($join) use ($request){
				$join->on('customers.id', '=', 'delivery_details.customer_id');
			});

			$query->leftJoin('products', function($join) use ($request){
				$join->on('products.id', '=', 'delivery_details.product_id');
			});



			if($request->has('start_date') && $request->start_date!= ''){
				$query->whereDate('delivery_details.created_at', '>=' ,$request->start_date);
				//$join->orWhere('delivery_details.created_at', '=', null);
			}

			if($request->has('end_date') && $request->end_date!= ''){
				$query->whereDate('delivery_details.created_at', '<=' ,$request->end_date);
			}

			if($request->has('customer_id') && $request->customer_id!= ''){
				$query->where('customers.id', '=' ,$request->customer_id);
			}

			$query->orderBy('customers.route_position');
			
			$supply_query = $query;

			$collection = $query->paginate(300);

			$customer_data = Customers::select('customers.*')->where('id',$request->customer_id)->first();


			$range_pending_amount = $supply_query->where('delivery_details.payment_status', 0)->where('customers.id', $request->customer_id)->sum('amount');

			$total_pending_amount =DeliveryDetails::where('delivery_details.payment_status', 0)->where('delivery_details.customer_id', $request->customer_id)->sum('amount');

			$result['customer_data'] 		= $customer_data;
			$result['advance_amount'] 		= $customer_data->advance_amount;
			$result['range_pending_amount'] = $range_pending_amount;
			$result['total_pending_amount'] = $total_pending_amount - $customer_data->advance_amount;
			$result['supply_report'] 		= $collection;
			return $this->sendResponse($result,'Supply Report');
		}else{
			return $this->sendError('Invalid Customer'); 
		}
	}


	public function routeSupplyDetailss(Request $request){
		if($request->has('route_id') && $request->route_id!= ''){


			$this->objCustomer->deliveryDate = $request->supply_date;

			//check product delivered in the date
			$delivery_query = Customers::query();
			$delivery_query->where('customers.is_delete', 0);
			$delivery_query->where('customers.route_id', '=' ,$request->route_id);
			$delivery_query->whereDate('customers.created_at', '<=' ,$request->supply_date);
			$delivery_query->join('delivery_details', function($join) use($request){
				$join->on('delivery_details.customer_id', '=', 'customers.id');
				$join->whereDate('delivery_details.created_at', '=', $request->supply_date);
			});

			$date_delivery = $delivery_query->get();

			$route_model = Routes::find($request->route_id);
			if($route_model->created_at > $request->supply_date){
				$supply_mode = 'invalid';
			}else{
				$supply_mode = (count($date_delivery) > 0)?'delivered':'future';
			}
			
			if($request->supply_date <= date('Y-m-d') && count($date_delivery) > 0 && $supply_mode!='invalid'){
				$result = [];
				$query = Customers::query();
				$query->select('customers.id', 'customers.customer_name');
				$query->where('customers.route_id', '=' ,$request->route_id);
				$query->where('customers.is_delete', 0);
				$query->orderBy('customers.route_position');

				$query->with(['delivery_details' => function($query) use ($request) {
					if($request->has('supply_date') && $request->supply_date!= ''){
						$query->whereDate('delivery_details.created_at', '=' ,$request->supply_date);
					}
				}]);

				$collection = $query->paginate(50);

				$summary_query = DeliveryDetails::join('customers', 'customers.id', '=', 'delivery_details.customer_id')
				->join('products', 'products.id', '=', 'delivery_details.product_id')
				->select('products.id AS product_id' ,'products.product_name', DB::raw("SUM(delivery_details.quantity) as quantity"))
				->where('customers.is_delete', 0)
				->where('customers.route_id', '=' ,$request->route_id)
				->whereDate('delivery_details.created_at', '=' ,$request->supply_date)
				->groupBy('products.id')->get();

				$result['supply_mode'] 			= $supply_mode;
				$result['supply_summary'] 		= $summary_query;
				$result['supply_report'] 		= $collection;
			}else{

				$day = strtolower(date("D", strtotime($request->supply_date)));
				$date = $request->supply_date;

				$query = Customers::query();
				$query->select('customers.id', 'customers.customer_name')
				->leftJoin('customer_products', 'customer_products.customer_id', '=', 'customers.id')
				->join('routes', 'routes.id', '=', 'customers.route_id')
				//->with('product_summary')
				->whereDate('customers.created_at', '<=' ,$request->supply_date)
				->with('product_summary', function($query) use ($day, $date){
					$query->select('product_id', 'customer_id', 'quantity')
					->where([ ['delivery_mode', '=', 'daily']])
					->orWhere([ ['delivery_mode', '=', 'custom'],[$day, '=', 1] ])
					->orWhere([ ['delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', delivery_start_date), 2)"), '=', 0] ]);
				})
				->whereHas('product', function($query) use ($day, $date){
					$query
					->where([ ['delivery_mode', '=', 'daily']])
					->orWhere([ ['delivery_mode', '=', 'custom'],[$day, '=', 1] ])
					->orWhere([ ['delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', delivery_start_date), 2)"), '=', 0] ]);
				})
				->where('customers.route_id', $request->route_id)
				->leftJoin('delivery_details', function($join){
					$join->on('delivery_details.customer_id', '=', 'customers.id');
					$join->whereDate('delivery_details.created_at', '=', date('Y-m-d'));
				})
				->whereNull('delivery_details.customer_id')
				->where('customers.is_delete', 0)
				->where('customers.status', 1)
				->groupBy('customers.id')
				->orderBy('route_position')
				->get();

				$customers = $query;

				$collection = $query->paginate(50);

				$customer_ids = $customers->pluck('id')->all();

				$products = Products::select('products.id as product_id', 'products.product_name', DB::raw("SUM(customer_products.quantity) AS quantity"))
				->join('customer_products', 'customer_products.product_id', '=', 'products.id')
				->whereIn('customer_products.customer_id', $customer_ids)
				->where(function ($query) use ($day, $date) {
					$query->where([ ['customer_products.delivery_mode', '=', 'daily']])
					->orWhere([ ['customer_products.delivery_mode', '=', 'custom'],['customer_products.'.$day, '=', 1] ])
					->orWhere([ ['customer_products.delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', customer_products.delivery_start_date), 2)"), '=', 0] ]);

				})
				->groupBy('products.id')
				->get();

				$result['supply_mode'] 			= $supply_mode;
				$result['supply_summary'] 		= $products;
				$result['supply_report'] 		= $collection;
			}
			
			return $this->sendResponse($result,'Supply Report');
		}else{
			return $this->sendError('Invalid Route'); 
		}
	}


	public function routeSupplyDetails(Request $request){
		if($request->has('route_id') && $request->route_id!= ''){

			$result = [];
			$route_model = Routes::find($request->route_id);
			$request_date 	= date("Y-m-d", strtotime($request->supply_date));
			$route_date 	= date("Y-m-d", strtotime($route_model->created_at));

			if($request_date < $route_date){
				$supply_mode = 'invalid';
				$result['supply_mode'] 			= $supply_mode;
			}
			else
			{

				$day = strtolower(date("D", strtotime($request->supply_date)));
				$date = $request->supply_date;

				//Customer Products 
				$products = Products::select('products.id as product_id', 'products.product_name', DB::raw("SUM(customer_products.quantity) AS actual_quantity"))
				->join('customer_products', 'customer_products.product_id', '=', 'products.id')
				->join('customers', 'customers.id', '=', 'customer_products.customer_id')
				->where(function ($query) use ($day, $date) {

					$query->where([ ['customer_products.delivery_mode', '=', 'daily']])
		                ->orWhere([ ['customer_products.delivery_mode', '=', 'custom'],['customer_products.'.$day, '=', 1] ])
		                ->orWhere([ ['customer_products.delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', customer_products.delivery_start_date), 2)"), '=', 0] ])
		                ->orWhere([ ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date] ]);

					/*
					$query->where([ ['customer_products.delivery_mode', '=', 'daily']])
					->orWhere([ ['customer_products.delivery_mode', '=', 'custom'],['customer_products.'.$day, '=', 1] ])
					->orWhere([ ['customer_products.delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', customer_products.delivery_start_date), 2)"), '=', 0] ]);
					*/

				})
				->where('customers.route_id', $request->route_id)
				->whereDate('customer_products.created_at', '<=', $request->supply_date)
				->where(function ($query)  use ($date){
					$query->where('customers.status', '=', 1)
		                ->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]])
		                ->orWhere([['customers.status', '=', 0 ], ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date]]);
					/*
					$query->where('customers.status', '=', 1)
					->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]]);
					*/
				})
				->where('customers.is_delete', 0)
				->groupBy('products.id')
				->get();
				//$result['customer_products_summary'] = $products;

				$product_list = $products->toArray();
				$customer_product_ids = array_column($product_list, 'product_id');

				
				//check product delivered in the date
				$delivery_query = Customers::query();
				$delivery_query->where('customers.is_delete', 0);
				$delivery_query->where('customers.route_id', '=' ,$request->route_id);
				$delivery_query->whereDate('customers.created_at', '<=' ,$request->supply_date);
				$delivery_query->join('delivery_details', function($join) use($request){
					$join->on('delivery_details.customer_id', '=', 'customers.id');
					$join->whereDate('delivery_details.created_at', '=', $request->supply_date);
				});
				$date_delivery = $delivery_query->get();

				if(count($date_delivery) > 0){

					$supply_mode = 'delivered';
					$result['supply_mode'] 			= $supply_mode;

					$query = Customers::query();
					$query->select('customers.id', 'customers.customer_name');
					$query->where('customers.route_id', '=' ,$request->route_id);
					$query->where('customers.is_delete', 0);
					$query->orderBy('customers.route_position');

					$query->with(['delivery_details' => function($query) use ($request) {
						if($request->has('supply_date') && $request->supply_date!= ''){
							$query->whereDate('delivery_details.created_at', '=' ,$request->supply_date);
						}
					}]);
					

					$collection = $query->paginate(300);


					$env_summary_date = env('DELIVERY_SUMMARY_DATE');
							//consolidated data saved in summary table after env date
					if($request->supply_date > $env_summary_date){
						//consolidated
						$delivery_consolidated = DeliverySummary::join('products', 'products.id', '=', 'delivery_summary.product_id')
						->select('products.id AS product_id' ,'products.product_name', 'actual_quantity', 'delivered_quantity')
						->where('delivery_summary.route_id', '=' ,$request->route_id)
						->whereDate('delivery_summary.created_at', '=' ,$request->supply_date)
						->groupBy('products.id')->get();

						$result['supply_summary'] 		= $delivery_consolidated;

					}else{
						$summary_query = DeliveryDetails::join('customers', 'customers.id', '=', 'delivery_details.customer_id')
						->join('products', 'products.id', '=', 'delivery_details.product_id')
						->select('products.id AS product_id' ,'products.product_name', DB::raw("SUM(delivery_details.quantity) as delivered_quantity"))
						->where('customers.is_delete', 0)
						->where('customers.route_id', '=' ,$request->route_id)
						->whereDate('delivery_details.created_at', '=' ,$request->supply_date)
						->groupBy('products.id')->get();

						$delivered_list = $summary_query->toArray();
						$delivered_product_ids = array_column($delivered_list, 'product_id');

						$product_ids = array_unique(array_merge($customer_product_ids, $delivered_product_ids));

						$supply_summary = $this->getProductData($product_list, $delivered_list);

						$result['supply_summary'] 		= $supply_summary;
					}

					$result['supply_report'] 		= $collection;
				}
				else if(count($products) >0 && count($date_delivery) == 0){

					if($request->supply_date > date('Y-m-d')){
						$supply_mode = 'future';
					}else{
						$supply_mode = 'not delivered';
					}

					$result['supply_mode'] 			= $supply_mode;

					$query = Customers::query();
					$query->select('customers.id', 'customers.customer_name')
					->leftJoin('customer_products', 'customer_products.customer_id', '=', 'customers.id')
					->join('routes', 'routes.id', '=', 'customers.route_id')
					->whereDate('customers.created_at', '<=' ,$request->supply_date)
					->with('product_summary', function($query) use ($day, $date){
						$query->select('product_id', 'customer_id', 'quantity')
			                 ->where(function ($query)  use ($day, $date){
			                    $query->where([ ['customer_products.delivery_mode', '=', 'daily']])
			                    ->where('customers.status', '=', 1)
			                    ->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]])
			                    ->orWhere([['customers.status', '=', 0 ], ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date]]);
			                })
			                ->orWhere(function ($query)  use ($day, $date){
			                    $query->where([ ['customer_products.delivery_mode', '=', 'custom'],['customer_products.'.$day, '=', 1] ])
			                    ->where('customers.status', '=', 1)
			                    ->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]])
			                    ->orWhere([['customers.status', '=', 0 ], ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date]]);
			                })
			                ->orWhere(function ($query)  use ($day, $date){
			                    $query->where([ ['customer_products.delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', customer_products.delivery_start_date), 2)"), '=', 0] ])
			                    ->where('customers.status', '=', 1)
			                    ->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]])
			                    ->orWhere([['customers.status', '=', 0 ], ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date]]);
			                })
			                ->orWhere([ ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date] ])
			                // ->groupBy('customer_products.product_id', 'customer_products.customer_id')
			                ;
						/*
						$query->select('product_id', 'customer_id', 'quantity')
						->where([ ['delivery_mode', '=', 'daily']])
						->orWhere([ ['delivery_mode', '=', 'custom'],[$day, '=', 1] ])
						->orWhere([ ['delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', delivery_start_date), 2)"), '=', 0] ]);
						*/
					})
					->whereHas('product', function($query) use ($day, $date){
						$query->where([ ['delivery_mode', '=', 'daily']])
			                ->orWhere([ ['delivery_mode', '=', 'custom'],[$day, '=', 1] ])
			                ->orWhere([ ['delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', delivery_start_date), 2)"), '=', 0] ])
			                ->orWhere([ ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date] ]);
						/*
						$query
						->where([ ['delivery_mode', '=', 'daily']])
						->orWhere([ ['delivery_mode', '=', 'custom'],[$day, '=', 1] ])
						->orWhere([ ['delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', delivery_start_date), 2)"), '=', 0] ]);
						*/
					})
					->where('customers.route_id', $request->route_id)
					->where('customers.is_delete', 0)
					->where(function ($query)  use ($date){
						$query->where('customers.status', '=', 1)
				                ->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]])
				                ->orWhere([['customers.status', '=', 0 ], ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date]]);
						/*
						$query->where('customers.status', '=', 1)
						->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]]);
						*/
					})
					->groupBy('customers.id')
					->orderBy('route_position')
					->get();

					$customers = $query;
					$collection = $query->paginate(300);

					$result['supply_summary'] = $products;
					//$result['supply_report'] 		= $collection;

				}else{
					$result['supply_mode'] 			= 'invalids';
				}
			}
			return $this->sendResponse($result,'Supply Report');
		}
		else{
			return $this->sendError('Invalid Route'); 
		}
	}


	//supply details of all routes in a date
	public function allSupplyDetails(Request $request){
		if($request->has('supply_date') && $request->supply_date!= ''){

			$request_date 	= date("Y-m-d", strtotime($request->supply_date));
			$routes_model = Routes::where('status', 1)->where('is_delete', 0)->orderBy('route_name')->get();
			if($routes_model!=null){
				$result = [];

				foreach ($routes_model as $key => $route_model) {
					$route_date 	= date("Y-m-d", strtotime($route_model->created_at));
					$result[$key]['route_name'] = $route_model->route_name;

					if($request_date < $route_date){
						$supply_mode = 'invalid';
						$result[$key]['supply_mode'] 			= $supply_mode;
					}
					else
					{

						$day = strtolower(date("D", strtotime($request->supply_date)));
						$date = $request->supply_date;

						//Customer Products 
						$products = Products::select('products.id as product_id', 'products.product_name', DB::raw("SUM(customer_products.quantity) AS actual_quantity"))
						->join('customer_products', 'customer_products.product_id', '=', 'products.id')
						->join('customers', 'customers.id', '=', 'customer_products.customer_id')
						->where(function ($query) use ($day, $date) {

							$query->where(function ($query)  use ($day, $date){
			                    $query->where([ ['customer_products.delivery_mode', '=', 'daily']])
			                    ->where('customers.status', '=', 1)
			                    ->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]])
			                    ->orWhere([['customers.status', '=', 0 ], ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date]]);
			                })
			                ->orWhere(function ($query)  use ($day, $date){
			                    $query->where([ ['customer_products.delivery_mode', '=', 'custom'],['customer_products.'.$day, '=', 1] ])
			                    ->where('customers.status', '=', 1)
			                    ->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]])
			                    ->orWhere([['customers.status', '=', 0 ], ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date]]);
			                })
			                ->orWhere(function ($query)  use ($day, $date){
			                    $query->where([ ['customer_products.delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', customer_products.delivery_start_date), 2)"), '=', 0] ])
			                    ->where('customers.status', '=', 1)
			                    ->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]])
			                    ->orWhere([['customers.status', '=', 0 ], ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date]]);
			                })
			                ->orWhere([ ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date] ])
			                // ->groupBy('customer_products.product_id', 'customer_products.customer_id')
			                ;
							/*
							$query->where([ ['customer_products.delivery_mode', '=', 'daily']])
							->orWhere([ ['customer_products.delivery_mode', '=', 'custom'],['customer_products.'.$day, '=', 1] ])
							->orWhere([ ['customer_products.delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', customer_products.delivery_start_date), 2)"), '=', 0] ]);
							*/

						})
						->where('customers.route_id', $route_model->id)
						->whereDate('customer_products.created_at', '<=', $request->supply_date)
						->where(function ($query)  use ($date){
							$query->where('customers.status', '=', 1)
				                ->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]])
				                ->orWhere([['customers.status', '=', 0 ], ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date]]);
							/*
							$query->where([['customers.status', '=', 1]])
							->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]]);
							*/
						})
						->where('customers.is_delete', 0)
						->groupBy('products.id')
						->get();
						//$result['customer_products_summary'] = $products;

						$product_list = $products->toArray();
						$customer_product_ids = array_column($product_list, 'product_id');


						//check product delivered in the date
						$delivery_query = Customers::query();
						$delivery_query->where('customers.is_delete', 0);
						$delivery_query->where('customers.route_id', '=' ,$route_model->id);
						$delivery_query->whereDate('customers.created_at', '<=' ,$request->supply_date);
						$delivery_query->join('delivery_details', function($join) use($request){
							$join->on('delivery_details.customer_id', '=', 'customers.id');
							$join->whereDate('delivery_details.created_at', '=', $request->supply_date);
						});
						$date_delivery = $delivery_query->get();

						if(count($date_delivery) > 0){

							$supply_mode = 'delivered';
							$result[$key]['supply_mode'] 			= $supply_mode;

							$env_summary_date = env('DELIVERY_SUMMARY_DATE');
							//consolidated data saved in summary table after env date
							if($request->supply_date > $env_summary_date){
								//consolidated
								$delivery_consolidated = DeliverySummary::join('products', 'products.id', '=', 'delivery_summary.product_id')
								->select('products.id AS product_id' ,'products.product_name', 'actual_quantity', 'delivered_quantity')
								->where('delivery_summary.route_id', '=' ,$route_model->id)
								->whereDate('delivery_summary.created_at', '=' ,$request->supply_date)
								->groupBy('products.id')->get();

								$result[$key]['supply_summary'] 		= $delivery_consolidated;

							}else{
								$summary_query = DeliveryDetails::join('customers', 'customers.id', '=', 'delivery_details.customer_id')
								->join('products', 'products.id', '=', 'delivery_details.product_id')
								->select('products.id AS product_id' ,'products.product_name', DB::raw("SUM(delivery_details.quantity) as delivered_quantity"))
								->where('customers.is_delete', 0)
								->where('customers.route_id', '=' ,$route_model->id)
								->whereDate('delivery_details.created_at', '=' ,$request->supply_date)
								->groupBy('products.id')->get();

								$delivered_list = $summary_query->toArray();
								$delivered_product_ids = array_column($delivered_list, 'product_id');

								$product_ids = array_unique(array_merge($customer_product_ids, $delivered_product_ids));

								$supply_summary = $this->getProductData($product_list, $delivered_list);

								$result[$key]['supply_summary'] 		= $supply_summary;
							}

						}
						else if(count($products) >0 && count($date_delivery) == 0){

							if($request->supply_date > date('Y-m-d')){
								$supply_mode = 'future';
							}else{
								$supply_mode = 'not delivered';
							}

							$result[$key]['supply_mode'] 			= $supply_mode;

							$query = Customers::query();
							$query->select('customers.id', 'customers.customer_name')
							->leftJoin('customer_products', 'customer_products.customer_id', '=', 'customers.id')
							->join('routes', 'routes.id', '=', 'customers.route_id')
							->whereDate('customers.created_at', '<=' ,$request->supply_date)
							->with('product_summary', function($query) use ($day, $date){

								$query->select('product_id', 'customer_id', 'quantity')
			                 ->where(function ($query)  use ($day, $date){
			                 	$query->select('product_id', 'customer_id', 'quantity')
			                    	->where([ ['customer_products.delivery_mode', '=', 'daily']])
					                    ->where('customers.status', '=', 1)
					                    ->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]])
					                    ->orWhere([['customers.status', '=', 0 ], ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date]]);
					                })
					                ->orWhere(function ($query)  use ($day, $date){
					                    $query->where([ ['customer_products.delivery_mode', '=', 'custom'],['customer_products.'.$day, '=', 1] ])
					                    ->where('customers.status', '=', 1)
					                    ->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]])
					                    ->orWhere([['customers.status', '=', 0 ], ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date]]);
					                })
					                ->orWhere(function ($query)  use ($day, $date){
					                    $query->where([ ['customer_products.delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', customer_products.delivery_start_date), 2)"), '=', 0] ])
					                    ->where('customers.status', '=', 1)
					                    ->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]])
					                    ->orWhere([['customers.status', '=', 0 ], ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date]]);
					                })
					                ->orWhere([ ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date] ])
					                // ->groupBy('customer_products.product_id', 'customer_products.customer_id')
					                ;

								/*
								$query->select('product_id', 'customer_id', 'quantity')
								->where([ ['delivery_mode', '=', 'daily']])
								->orWhere([ ['delivery_mode', '=', 'custom'],[$day, '=', 1] ])
								->orWhere([ ['delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', delivery_start_date), 2)"), '=', 0] ]);
								*/
							})
							->whereHas('product', function($query) use ($day, $date){
								
								$query->where([ ['delivery_mode', '=', 'daily']])
					                ->orWhere([ ['delivery_mode', '=', 'custom'],[$day, '=', 1] ])
					                ->orWhere([ ['delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', delivery_start_date), 2)"), '=', 0] ])
					                ->orWhere([ ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date] ]);
								/*$query
								->where([ ['delivery_mode', '=', 'daily']])
								->orWhere([ ['delivery_mode', '=', 'custom'],[$day, '=', 1] ])
								->orWhere([ ['delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', delivery_start_date), 2)"), '=', 0] ]);
								*/
							})
							->where('customers.route_id', $route_model->id)
							->where('customers.is_delete', 0)
							->where(function ($query)  use ($date){
								
								$query->where('customers.status', '=', 1)
				                ->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]])
				                ->orWhere([['customers.status', '=', 0 ], ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date]]);
								/*
								$query->where('customers.status', '=', 1)
								->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]]);
								*/
							})
							->groupBy('customers.id')
							->orderBy('route_position')
							->get();


							$customers = $query;
							$collection = $query->paginate(300);

							$result[$key]['supply_summary'] = $products;
							//$result['supply_report'] 		= $collection;

						}else{
							$result[$key]['supply_mode'] 			= 'invalid';
						}
					}
				}


				return $this->sendResponse($result,'Supply Report');
			}else{
				return $this->sendError('Active Routes Not Exists'); 
			}

		}
		else{
			return $this->sendError('Invalid Route'); 
		}
	}


	//supply details of all routes in a date
	public function allFutureSupplyDetails(Request $request){
		if($request->has('supply_date') && $request->supply_date!= ''){

			$request_date 	= date("Y-m-d", strtotime($request->supply_date));
			$routes_model = Routes::where('status', 1)->where('is_delete', 0)->get();
			if($routes_model!=null){
				$result = [];

				foreach ($routes_model as $key => $route_model) {
					$route_date 	= date("Y-m-d", strtotime($route_model->created_at));
					$result[$key]['route_name'] = $route_model->route_name;

					if($request_date < $route_date){
						$supply_mode = 'invalid';
						$result[$key]['supply_mode'] 			= $supply_mode;
					}
					else
					{

						$day = strtolower(date("D", strtotime($request->supply_date)));
						$date = $request->supply_date;

						//Customer Products 
						$products = Products::select('products.id as product_id', 'products.product_name', DB::raw("SUM(customer_products.quantity) AS actual_quantity"))
						->join('customer_products', 'customer_products.product_id', '=', 'products.id')
						->join('customers', 'customers.id', '=', 'customer_products.customer_id')
						->where(function ($query) use ($day, $date) {
							$query->where([ ['customer_products.delivery_mode', '=', 'daily']])
							->orWhere([ ['customer_products.delivery_mode', '=', 'custom'],['customer_products.'.$day, '=', 1] ])
							->orWhere([ ['customer_products.delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', customer_products.delivery_start_date), 2)"), '=', 0] ]);

						})
						->where('customers.route_id', $route_model->id)
						->whereDate('customer_products.created_at', '<=', $request->supply_date)
						->where(function ($query)  use ($date){
							$query->where([['customers.status', '=', 1]])
							->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]]);
						})
						->where('customers.is_delete', 0)
						->groupBy('products.id')
						->get();
						//$result['customer_products_summary'] = $products;


						if(count($products) >0){

							if($request->supply_date > date('Y-m-d')){
								$supply_mode = 'future';
							}else{
								$supply_mode = 'not delivered';
							}

							$result[$key]['supply_mode'] 			= $supply_mode;

							$query = Customers::query();
							$query->select('customers.id', 'customers.customer_name')
							->leftJoin('customer_products', 'customer_products.customer_id', '=', 'customers.id')
							->join('routes', 'routes.id', '=', 'customers.route_id')
							->whereDate('customers.created_at', '<=' ,$request->supply_date)
							->with('product_summary', function($query) use ($day, $date){
								$query->select('product_id', 'customer_id', 'quantity')
								->where([ ['delivery_mode', '=', 'daily']])
								->orWhere([ ['delivery_mode', '=', 'custom'],[$day, '=', 1] ])
								->orWhere([ ['delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', delivery_start_date), 2)"), '=', 0] ]);
							})
							->whereHas('product', function($query) use ($day, $date){
								$query
								->where([ ['delivery_mode', '=', 'daily']])
								->orWhere([ ['delivery_mode', '=', 'custom'],[$day, '=', 1] ])
								->orWhere([ ['delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', delivery_start_date), 2)"), '=', 0] ]);
							})
							->where('customers.route_id', $route_model->id)
							->where('customers.is_delete', 0)
							->where(function ($query)  use ($date){
								$query->where('customers.status', '=', 1)
								->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]]);
							})
							
							->groupBy('customers.id')
							->orderBy('route_position')
							->get();

							//print("<pre>"); print_r($query->toSql()); print("</pre>"); exit;

							$customers = $query;
							$collection = $query->paginate(300);

							$result[$key]['supply_summary'] = $products;
							//$result['supply_report'] 		= $collection;

						}else{
							$result[$key]['supply_mode'] 			= 'invalid';
						}
					}
				}


				return $this->sendResponse($result,'Supply Report');
			}else{
				return $this->sendError('Active Routes Not Exists'); 
			}

		}
		else{
			return $this->sendError('Invalid Route'); 
		}
	}


	public function getProductData($new_product_list, $delivered_list){
		if(is_array($new_product_list) && is_array($delivered_list)){
			foreach ($delivered_list as $delivered_data) {

				$key = array_search($delivered_data['product_id'], array_column($new_product_list, 'product_id'));

				if(false !== $key){
					$new_product_list[$key]['delivered_quantity'] = $delivered_data['delivered_quantity'];
				}else{

					$delivered_data['actual_count'] = 0;
					$new_product_list[] = $delivered_data;
				}
			}
			return $new_product_list;
		}

	}

	public function routeSalesConsolidateReport(Request $request){
		if($request->has('start_date') && $request->start_date!= ''){

			$start_date = date('Y-m-d', strtotime($request->start_date)).' 00:00:00';
			$end_date = date('Y-m-d', strtotime($request->end_date)).' 23:59:59';

			$query =  DB::table('routes');
			$query->leftJoin('customers', 'routes.id', '=' ,'customers.route_id');
			$query->leftjoin('delivery_details', function($join) use($request, $start_date, $end_date){
				$join->on('customers.id', '=', 'delivery_details.customer_id');
				if($request->has('start_date') && $request->start_date!= ''){
					//$join->whereDate('delivery_details.created_at', '>=' , $start_date);
				}
				if($request->has('end_date') && $request->end_date!= ''){
					//$join->whereDate('delivery_details.created_at', '<=' , $end_date);
				}
				$join->orWhereNull('delivery_details.customer_id');
			});
			$query->select('routes.id','routes.route_name','routes.petty_cash', DB::raw("(SELECT SUM(customers.advance_amount) FROM customers WHERE routes.id = customers.route_id) as total_advance_amount"));
		    $query->selectRaw("SUM(CASE WHEN (delivery_details.created_at >= '$start_date' AND delivery_details.created_at <= '$end_date') THEN delivery_details.amount ELSE 0 END) AS total_sales");
		    $query->selectRaw("SUM(CASE WHEN (delivery_details.payment_status = 0 AND delivery_details.created_at >= '$start_date' AND delivery_details.created_at <= '$end_date') THEN delivery_details.amount ELSE 0 END) AS outstanding_amount");
		    $query->selectRaw("SUM(CASE WHEN (delivery_details.payment_status = 0 AND delivery_details.created_at < '$start_date') THEN delivery_details.amount ELSE 0 END) AS prev_outstanding_amount");

			$query->where('routes.status', 1)->where('routes.is_delete', 0);
			$query->orderBy('routes.route_name');
			$query->groupBy('customers.route_id');
			$query->groupBy('routes.id');
				
			$collection = $query->paginate(300);
			$total_sales_sum = $query->pluck('total_sales')->all();
			$outstanding_amount_sum = $query->pluck('outstanding_amount')->all();
			$prev_outstanding_amount_sum = $query->pluck('prev_outstanding_amount')->all();
			$total_sales_amount = $query->pluck('total_sales')->all();
			$petty_cash_sum = $query->pluck('petty_cash')->all();
			$total_advance_amount = $query->pluck('total_advance_amount')->all();

			$result['total_sales_sum'] 	= array_sum($total_sales_sum);
			$result['collection_sum'] 	= array_sum($total_sales_sum) - array_sum($outstanding_amount_sum);
			$result['outstanding_sum'] 	= array_sum($outstanding_amount_sum);
			$result['prev_outstanding_sum'] 	= array_sum($prev_outstanding_amount_sum);
			$result['total_outstanding_sum'] 	= array_sum($outstanding_amount_sum) + array_sum($prev_outstanding_amount_sum);
			$result['petty_cash_sum'] 	= array_sum($petty_cash_sum);
			$result['total_advance_sum'] 	= array_sum($total_advance_amount);
			$result['grand_sum'] 	= (array_sum($outstanding_amount_sum) + array_sum($prev_outstanding_amount_sum) + array_sum($petty_cash_sum)) - array_sum($total_advance_amount);

			$result['sales_report'] = $collection;
			return $this->sendResponse($result,'Route Sales Consolidated Report');
		}
		else{
			return $this->sendError('Invalid Period'); 
		}
	}


	public function routeSalesConsolidateReportPdf(Request $request){
		if($request->has('start_date') && $request->start_date!= ''){

				$start_date = date('Y-m-d', strtotime($request->start_date)).' 00:00:00';
				$end_date = date('Y-m-d', strtotime($request->end_date)).' 23:59:59';
				// $query = Routes::query();
				// $query->join('customers', 'routes.id', '=' ,'customers.route_id')
				// 		->select('routes.id','routes.route_name','routes.petty_cash')
				// 		->where(function ($query) use ($request) {
			    //             if($request->has('route_id') && $request->route_id!= ''){
			    //             	$query->where('customers.route_id', '=' ,$request->route_id);
			    //             }
			    //         });
			    // $query->selectRaw("SUM(CASE WHEN (delivery_details.created_at >= '$start_date' AND delivery_details.created_at <= '$end_date') THEN delivery_details.amount ELSE 0 END) AS total_sales");

			    // $query->selectRaw("SUM(CASE WHEN (delivery_details.payment_status = 0 AND delivery_details.created_at >= '$start_date' AND delivery_details.created_at <= '$end_date') THEN delivery_details.amount ELSE 0 END) AS outstanding_amount");

			    // $query->selectRaw("SUM(CASE WHEN (delivery_details.payment_status = 0 AND delivery_details.created_at < '$start_date') THEN delivery_details.amount ELSE 0 END) AS prev_outstanding_amount");

		        // $query->leftJoin('delivery_details', function($join) use ($request){
				// 	$join->on('customers.id', '=', 'delivery_details.customer_id');
				// 	$join->orWhereNull('delivery_details.customer_id');
				// });
				// //$query->with('route_customers');
				// $query->where('routes.status', 1)->where('routes.is_delete', 0);
				// $query->orderBy('routes.route_name');
				// $query->groupBy('routes.id');

			$query =  DB::table('routes');
			$query->leftJoin('customers', 'routes.id', '=' ,'customers.route_id');
			$query->leftjoin('delivery_details', function($join) use($request, $start_date, $end_date){
				$join->on('customers.id', '=', 'delivery_details.customer_id');
				if($request->has('start_date') && $request->start_date!= ''){
					//$join->whereDate('delivery_details.created_at', '>=' , $start_date);
				}
				if($request->has('end_date') && $request->end_date!= ''){
					//$join->whereDate('delivery_details.created_at', '<=' , $end_date);
				}
				$join->orWhereNull('delivery_details.customer_id');
			});
			$query->select('routes.id','routes.route_name','routes.petty_cash', DB::raw("(SELECT SUM(customers.advance_amount) FROM customers WHERE routes.id = customers.route_id) as total_advance_amount"));
		    $query->selectRaw("SUM(CASE WHEN (delivery_details.created_at >= '$start_date' AND delivery_details.created_at <= '$end_date') THEN delivery_details.amount ELSE 0 END) AS total_sales");
		    $query->selectRaw("SUM(CASE WHEN (delivery_details.payment_status = 0 AND delivery_details.created_at >= '$start_date' AND delivery_details.created_at <= '$end_date') THEN delivery_details.amount ELSE 0 END) AS outstanding_amount");
		    $query->selectRaw("SUM(CASE WHEN (delivery_details.payment_status = 0 AND delivery_details.created_at < '$start_date') THEN delivery_details.amount ELSE 0 END) AS prev_outstanding_amount");

			$query->where('routes.status', 1)->where('routes.is_delete', 0);
			$query->orderBy('routes.route_name');
			$query->groupBy('customers.route_id');
			$query->groupBy('routes.id');

			$collection = $query->get();
			$result['sales_report'] = $collection;
			$result['start_date'] = date('d-m-Y', strtotime($request->start_date));
			$result['end_date'] = date('d-m-Y', strtotime($request->end_date));
			$result['generate_time'] = date('d-m-Y h:i A');
			//return (array)$result;
			$pdf = PDF::loadview('reports.route_sale_consolidated_pdf', $result);
			return $pdf->download('Route Sale Consolidated Report - '.date('Y-m-d').'.pdf');
		}
		else{
			return $this->sendError('Invalid Period'); 
		}
	}

	public function itemConsolidatedReport(Request $request){
		if($request->has('start_date') && $request->start_date!= ''){

				$start_date = date('Y-m-d', strtotime($request->start_date)).' 00:00:00';
				$end_date = date('Y-m-d', strtotime($request->end_date)).' 23:59:59';
				$query = Products::query();
				
				$query->select('products.id','products.product_name', DB::raw("SUM(delivery_details.quantity) as sales_quantity"), DB::raw("SUM(delivery_details.amount) as sales_amount"))
						->where(function ($query) use ($request) {
			                if($request->has('route_id') && $request->route_id!= ''){
			                	$query->where('customers.route_id', '=' ,$request->route_id);
			                }
			            });

		        $query->leftJoin('delivery_details', function($join) use ($request,$start_date, $end_date){
					$join->on('delivery_details.product_id', '=', 'products.id');

					if($request->has('start_date') && $request->start_date!= ''){
						$join->whereDate('delivery_details.created_at', '>=' ,$start_date);
					}

					if($request->has('end_date') && $request->end_date!= ''){
						$join->whereDate('delivery_details.created_at', '<=' ,$end_date);
					}
				});

				$query->leftJoin('customers', 'customers.id', '=' ,'delivery_details.customer_id');
				$query->orderBy('products.product_name');
				$query->groupBy('products.id');
				
			$collection = $query->paginate(300);

			$sales_quantity_array = $query->pluck('sales_quantity')->all();
			$sales_amount_array = $query->pluck('sales_amount')->all();

			$result['sales_quantity_sum'] 	= array_sum($sales_quantity_array);
			$result['sales_amount_sum'] 	= array_sum($sales_amount_array);

			$result['item_sale_report'] = $collection;


			return $this->sendResponse($result,'Route Sales Consolidated Report');
		}
		else{
			return $this->sendError('Invalid Period'); 
		}
	}

	public function itemConsolidatedReportPdf(Request $request){
		if($request->has('start_date') && $request->start_date!= ''){

				$start_date = date('Y-m-d', strtotime($request->start_date)).' 00:00:00';
				$end_date = date('Y-m-d', strtotime($request->end_date)).' 23:59:59';
				$query = Products::query();
				
				$query->select('products.id','products.product_name', DB::raw("SUM(delivery_details.quantity) as sales_quantity"), DB::raw("SUM(delivery_details.amount) as sales_amount"))
						->where(function ($query) use ($request) {
			                if($request->has('route_id') && $request->route_id!= ''){
			                	$query->where('customers.route_id', '=' ,$request->route_id);
			                }
			            });

		        $query->leftJoin('delivery_details', function($join) use ($request, $start_date, $end_date){
					$join->on('delivery_details.product_id', '=', 'products.id');

					if($request->has('start_date') && $request->start_date!= ''){
						$join->whereDate('delivery_details.created_at', '>=' ,$start_date);
					}

					if($request->has('end_date') && $request->end_date!= ''){
						$join->whereDate('delivery_details.created_at', '<=' ,$end_date);
					}
				});

				$query->leftJoin('customers', 'customers.id', '=' ,'delivery_details.customer_id');
				$query->orderBy('products.product_name');
				$query->groupBy('products.id');
				
			$collection = $query->get();

			$result['item_sale_report'] = $collection;
			$result['start_date'] = date('d-m-Y', strtotime($request->start_date));
			$result['end_date'] = date('d-m-Y', strtotime($request->end_date));
			$result['generate_time'] = date('d-m-Y h:i A');
			$pdf = PDF::loadview('reports.item_consolidated_pdf', $result);
			return $pdf->download('Item Consolidated Report - '.date('Y-m-d').'.pdf');
		}
		else{
			return $this->sendError('Invalid Period'); 
		}
	}

	public function pettyCashReport(Request $request){


        $result = [];
		$query = PettyCashDetails::query();
		$query->select('petty_cash_details.id','petty_cash_details.route_user_id as from_user_id', 'usr.name as from_username', 'petty_cash_details.collected_user_id as to_user_id', 'user.name as to_username', 'petty_cash_details.amount', 'petty_cash_details.payment_type', 'petty_cash_details.transfer_type', 'petty_cash_details.created_at', 'cuser.name as created_by' );
		$query->selectRaw('UPPER(petty_cash_details.payment_type) as payment_type');
		$query->selectRaw('CASE WHEN (petty_cash_details.from_route_manager = 1) THEN "Yes" ELSE "No" END as from_manager');
		//$query->selectRaw('CASE WHEN (petty_cash_details.from_route_manager = 1) THEN usr.name ELSE 0 END as from_username');
		$query->leftJoin('users as usr', 'usr.id', '=' ,'petty_cash_details.route_user_id');
		$query->leftJoin('users as user', 'user.id', '=' ,'petty_cash_details.collected_user_id');
		$query->leftJoin('users as cuser', 'cuser.id', '=' ,'petty_cash_details.created_by');
		$query->leftJoin('roles as rls', 'rls.user_id', '=' ,'usr.id');
		$query->leftJoin('roles as roles', 'roles.user_id', '=' ,'user.id');

		if($request->has('start_date') && $request->start_date!= ''){
			$query->whereDate('petty_cash_details.created_at', '>=' ,$request->start_date);
		}

		if($request->has('end_date') && $request->end_date!= ''){
			$query->whereDate('petty_cash_details.created_at', '<=' ,$request->end_date);
		}

		if($request->has('customer_pay_type') && $request->customer_pay_type!= ''){
			$query->where('petty_cash_details.payment_type', '=' ,$request->customer_pay_type);
		}

		$query->where('petty_cash_details.status', 1);
		$query->where('petty_cash_details.is_delete', 0);

		if($request->has('from_user_id') && $request->from_user_id!= ''){
			$from_id = $request->from_user_id;
			if($from_id!=0){
				$query->where('petty_cash_details.route_user_id', $from_id);
			}
		}

		if($request->has('to_user_id') && $request->to_user_id!= ''){
			$to_user_id = $request->to_user_id;
			if($to_user_id!=0){
				$query->where('petty_cash_details.collected_user_id', $to_user_id);
			}
		}

		$query->orderBy('petty_cash_details.created_at','DESC');
		$collection = $query->paginate(50);
		$result['petty_cash_details'] 		= $collection;
		return $this->sendResponse($result,'Petty Cash Report');

	
    }
}
