<?php

namespace App\Http\Controllers;

use DB;
use App\Models\Routes;
use App\Models\User; 
use App\Models\Role; 
use App\Models\Customers;
use App\Models\Products;
use App\Models\DeliveryDetails;
use App\Models\RouteEmployees;
use Illuminate\Http\Request;
use Validator;
use Exception;
use Illuminate\Support\Facades\Log;
use App\Http\Controllers\BaseController as BaseController;
use App\Http\Resources\Routes as RouteResource;

class RoutesController extends BaseController
{

    public $successStatus = 200;

    private $objCustomers;

    public function __construct(Customers $customers){
         $this->objCustomers = $customers;
     }


    public function routeListData()
    {
        $routes = Routes::select('routes.*',  'users.email', DB::raw('(SELECT COUNT(customers.id) FROM customers WHERE customers.route_id = routes.id AND customers.is_delete = 0) as customer_count'), DB::raw('(SELECT COUNT(customers.id) FROM customers WHERE customers.route_id = routes.id AND customers.is_delete = 0 AND customers.status = 0) as inactive_customer_count'))->where('routes.status', 1)->where('routes.is_delete', 0)
        ->groupBy("routes.id")
        ->leftJoin('customers', 'customers.route_id', '=', 'routes.id')->orderBy('route_name')
        ->leftJoin('users', 'users.id', '=', 'routes.route_user_id')
        ->orderBy('route_name')->paginate(50);

        //$routes = Routes::where('status', 0)->orderBy('route_name')->paginate(5);
        return $this->sendResponse($routes,'Route List');
    }

    public function routeList()
    {
        $routes = Routes::leftJoin('customers', 'customers.route_id', '=', 'routes.id')
        ->select('routes.*', 'routes.route_name', 'users.email', DB::raw('(SELECT COUNT(customers.id) FROM customers WHERE customers.route_id = routes.id AND customers.is_delete = 0 ) as customer_count'), DB::raw('(SELECT COUNT(customers.id) FROM customers WHERE customers.route_id = routes.id AND customers.is_delete = 0 AND customers.status = 1) as inactive_customer_count'))
        ->leftJoin('users', 'users.id', '=', 'routes.route_user_id')
        ->where('routes.status', 1)
        ->where('routes.is_delete', 0)
        ->groupBy("routes.id")->get();
        
        
        return $this->sendResponse($routes,'Route List');
    }

    public function routeListWithPrefix()
    {
        $routes = Routes::leftJoin('customers', 'customers.route_id', '=', 'routes.id')
        ->select(DB::raw('(CONCAT("R-",routes.id)) as from_id'), 'routes.route_name as from_name')
        ->leftJoin('users', 'users.id', '=', 'routes.route_user_id')
        ->where('routes.status', 1)
        ->where('routes.is_delete', 0)
        ->groupBy("routes.id")->get()->toArray();

        print("<pre>"); print_r($routes); print("</pre>"); exit;
        
        
        return $this->sendResponse($routes,'Route List With Prefix');
    }

    public function routeManagersListWithPrefix(){
        $routes_managers = User::select(DB::raw('(CONCAT("M-",users.id)) as from_id'),'users.name as from_name')->where('routes.status', 1)->where('routes.is_delete', 0)
        ->groupBy("users.id")
        ->leftJoin('route_employees', 'route_employees.user_id', '=', 'users.id')
        ->leftJoin('routes', 'routes.id', '=', 'route_employees.route_id')
        ->orderBy('users.name')->get();

        return $this->sendResponse($routes_managers,'Route Managers List With Prefix');
    }


    public function getPettyRouteUsersList(){
        $routes_suppliers = Routes::select('routes.route_user_id as from_id', 'routes.route_name as from_name')
        ->leftJoin('users', 'users.id', '=', 'routes.route_user_id')
        ->where('routes.status', 1)
        ->where('routes.is_delete', 0)
        ->groupBy("routes.id")->get()->toArray();

        $routes_managers = User::select('users.id as from_id','users.name as from_name')->where('routes.status', 1)->where('routes.is_delete', 0)
        ->groupBy("users.id")
        ->leftJoin('route_employees', 'route_employees.user_id', '=', 'users.id')
        ->leftJoin('routes', 'routes.id', '=', 'route_employees.route_id')
        ->orderBy('users.name')->get()->toArray();

        $all_list = [['from_id'=>0, 'from_name'=>'All']];
        $user_list = array_merge($all_list, $routes_suppliers, $routes_managers);
        return $this->sendResponse($user_list,'Route Users List');

    }

    public function getPettyCollectedUsersList(){
        $admin_users = User::select('users.id', 'users.name')
        ->whereIn('roles.role', ['admin','manager'])
        ->where('users.is_delete', 0)
        ->leftJoin('roles', 'roles.user_id', '=', 'users.id')
        ->orderBy('users.name')->get()->toArray();

        $routes_managers = User::select('users.id as id','users.name as name')->where('routes.status', 1)->where('routes.is_delete', 0)
        ->groupBy("users.id")
        ->leftJoin('route_employees', 'route_employees.user_id', '=', 'users.id')
        ->leftJoin('routes', 'routes.id', '=', 'route_employees.route_id')
        ->orderBy('users.name')->get()->toArray();

        $all_list = [['id'=>0, 'name'=>'All']];
        $user_list = array_merge($all_list, $admin_users, $routes_managers);
        return $this->sendResponse($user_list,'Collected Users List');

    }



    public function viewRoute($id)
    {
        $routes = Routes::select('routes.id', 'routes.route_user_id', 'routes.route_name', 'users.email', 'routes.petty_cash', DB::raw("COUNT(customers.id) as 'customer_count'"), DB::raw('(SELECT COUNT(customers.id) FROM customers WHERE customers.route_id = routes.id AND customers.is_delete = 0 AND customers.status = 0) as inactive_customer_count'))

        ->leftJoin('customers', function($join){
            $join->on('customers.route_id', '=', 'routes.id');
                    //$join->where('customers.status', 1);
            $join->where('customers.is_delete', 0);
        })
        ->leftJoin('users', 'users.id', '=', 'routes.route_user_id')
        ->where('routes.id', $id)
        ->groupBy("users.id", "routes.route_user_id")
        ->orderBy('route_name')
        ->get();

        if($routes){
            return $this->sendResponse($routes->toArray(),'Route Details');
        }else{
            return $this->sendError('Product Not Found');  
        }
    }

    public function routeCustomers($route_id){
        $customers = Customers::select('id', 'customer_name')->where('is_delete', 0)->where('route_id', $route_id)->orderBy('route_position')->get();
        return $this->sendResponse($customers,'Route Customer List');
    }


    public function store(Request $request)
    {
        $validator = Validator::make($request->all(), [ 
            'route_name' => 'required', 
            'route_name' => 'required|unique:routes,route_name|max:255', 
            //'address' => 'required|max:255', 
            //'route_user_id' => 'required', 
        ]);

        if ($validator->fails()) { 
            return response()->json(['error'=>$validator->errors()], 401);            
        }

        $input = $request->all(); 
        if ($route = Routes::create($input)){
            return $this->sendResponse('','Route Successfully Added');
            //return response()->json(['success'=>true,'message' => 'Route Successfully Added'], $this->successStatus);
        }else{
            return response()->json(['success'=>false, 'message'=>'Error'], 401);    
        }

    }

    public function addRoute(Request $request) { 

        $messages = array(
            'email.unique' => 'Username already exist',
            'name.unique' => 'Route Name already exist'
        );

        $validator = Validator::make($request->all(), [ 
            'name' => 'required|unique:users,name', 
            'email' => 'required|unique:users,email', 
            'password' => 'required', 
            'role' => 'required', 
        ], $messages);

        if ($validator->fails()) { 
            return $this->sendValidationErrors($validator->errors());          
        }

        $password = $request->password;
        $input = $request->all(); 
        $input['password'] = bcrypt($input['password']); 
        $user = User::create($input); 
        if($user->email){
            $role_model             = new Role;
            $role_model->user_id    = $user->id;
            $role_model->role       = $request->role;
            $role_model->save();

            $route_model            = new Routes;
            $route_model->route_user_id     = $user->id;
            $route_model->route_name = $user->name;
            $route_model->save();

            //return response()->json(['success'=>true, 'message'=>'Route Successfully Added', 'status'=>200], 200);  

            return $this->sendResponse('','Route Successfully Added'); 

        }else{
            return $this->sendError('Route Not Added'); 
            //return response()->json(['success'=>false, 'message'=>'Route Not Added', 'status'=>500], 200);   
        }
    }


    public function updateRoute(Request $request) { 

        $messages = array(
            'email.unique' => 'Username already exist',
            'name.unique' => 'Route Name already exist'
        );

        $input  = $request->all(); 
        $id     = isset($input['id'])?$input['id']:0;
        $route  = Routes::find($id);
        if($route){
            $validator = Validator::make($input, Routes::rules($id, $route->route_user_id), $messages);
            if ($validator->fails()) { 
                return $this->sendValidationErrors($validator->errors()); 
            }

            $route_user  = User::find($route->route_user_id);
            $route_user->name = $input['name'];
            $route_user->email = $input['email'];
            if(isset($input['password']) && $input['password']!=null){
                $route_user->password = bcrypt($input['password']); 
            }

            $route->route_name = $input['name'];
            if($route_user->save() && $route->save()){

                $routes = Routes::select('routes.id', 'routes.route_name', 'users.email')
                ->groupBy("users.id", "routes.route_user_id")
                ->leftJoin('users', 'users.id', '=', 'routes.route_user_id')->where('routes.id', $id)->get();

                return $this->sendResponse($routes,'Route Successfully Updated');
            }else{
                return $this->sendError('Route Not Updated');  
            }
            
        }else{
            return $this->sendError('Route Not Found');  
        }
    }


    public function deleteRoute($id)
    {
        $route = Routes::find($id);
        if($route){
            $route->is_delete = 1;
            if($route->save()){
                return $this->sendResponse('','Route Successfully Deleted');
            }else{
                return $this->sendError('Route Not Deleted');  
            }
        }else{
            return $this->sendError('Route Not Found');  
        }
    }


    public function addRouteEmployee(Request $request) { 

        $messages = array(
            'email.unique' => 'Username already exist',
        );

        $validator = Validator::make($request->all(), [ 
            'name'=>'required',
            'email' => 'required|unique:users,email', 
            'password' => 'required', 
            'route_ids'=>'required'
        ], $messages);

        if ($validator->fails()) { 
            return $this->sendValidationErrors($validator->errors());          
        }

        $input = $request->all(); 

        $post_route_ids  = json_decode($input['route_ids'], true); 
        $user_role = ($request->user_role)?$request->user_role:'route_manager';
        $password = $request->password; 
        $login_user_id = auth('api')->user()->id;
        $input['password'] = bcrypt($input['password']); 
        $user = User::create($input); 
        if($user->email){
            $role_model             = new Role;
            $role_model->user_id    = $user->id;
            $role_model->role       = $user_role;
            $role_model->save();

            foreach($post_route_ids['routes'] as $route_id){
                $exist = RouteEmployees::where('user_id',$user->id)->where('route_id',$route_id)->where('status',1)->exists();
                
                if(!$exist){
                    $route_data['route_id'] = $route_id;
                    $route_data['user_id'] = $user->id;
                    $route_data['status'] = 1;
                    $route_data['user_type'] = $user_role;
                    $route_data['created_by'] = $login_user_id ;
                    $route_employee = RouteEmployees::create($route_data);
                }
            }
            return $this->sendResponse('','Route Manager Successfully Added'); 

        }else{
            return $this->sendError('Route Manager Not Added'); 
        }
    }

    public function updateRouteEmployee(Request $request)
    {
        $input  = $request->all(); 
        $id     = isset($input['id'])?$input['id']:0;
        
        $route_employee = RouteEmployees::where('user_id',$id)->first();
        if($route_employee){
            $validator = Validator::make($input, RouteEmployees::rules($route_employee->user_id));
            if ($validator->fails()) { 
                return $this->sendValidationErrors($validator->errors()); 
            }

            $user = User::find($route_employee->user_id);
            $user_role = ($request->user_role)?$request->user_role:'route_manager';
            $login_user_id = auth('api')->user()->id;
            $post_route_ids  = json_decode($input['route_ids'], true); 

            $user_data['name'] = $input['name'];
            if(isset($input['password']) && $input['password']!=null){
                $user_data['password'] = bcrypt($input['password']); 
            }
            $updated = $user->fill($user_data)->save();
            $old_route_ids = array();
            
            if($updated){

                $user_role = ($request->user_role!='')?$request->user_role:'route_manager';
                if($user_role!=''){
                    $role_model = Role::where('user_id', $user->id)->update(['role'=>$user_role]);
                }
                

                $old_route_ids = DB::table('route_employees')->where('user_id', $route_employee->user_id)->where('status', 1)->pluck('route_id')->toArray();
                //save route employees
                if(!empty($post_route_ids)){
                    $exist_ids = [];
                    foreach ($post_route_ids['routes'] as $route_id) {
                        if(in_array($route_id, $old_route_ids)){

                            $data['status'] = 1;
                            $route_emp = DB::table('route_employees')->where('route_id', $route_id)->where('user_id', $route_employee->user_id)->update($data);
                            $exist_ids[] = $route_id;
                        }else{
                            $route_data['route_id'] = $route_id;
                            $route_data['user_id'] = $user->id;
                            $route_data['status'] = 1;
                            $route_data['user_type'] = $user_role;
                            $route_data['created_by'] = $login_user_id ;
                            RouteEmployees::create($route_data);
                        }
                    }


                    $deleted_ids = array_diff($old_route_ids, $exist_ids);
                    $deleted_items = DB::table('route_employees')->whereIn('route_id', $deleted_ids)->where('user_id', $user->id)->delete(); 

                }

                $route_employee_data = User::select('users.*', 'roles.*', 'route_employees.*')->where('users.id', $user->id)->where('users.is_delete', 0)
                            ->leftJoin('roles', 'roles.user_id', '=', 'users.id')
                            ->leftJoin('route_employees', 'route_employees.user_id', '=', 'users.id')
                            ->first();

                return $this->sendResponse($route_employee_data,'Route Manager Successfully Updated');
            }else{
                return $this->sendError('Route Manager Not Updated');  
            }
        }else{
            return $this->sendError('Route Manager Not Found');  
        }
    }

    public function employeeRouteList($user_id){
        $employee_route_list = RouteEmployees::select('route_employees.route_id', 'routes.route_name', DB::raw('IFNULL((SELECT SUM(petty_cash_details.amount) FROM petty_cash_details WHERE petty_cash_details.route_user_id = routes.route_user_id AND petty_cash_details.status = 1 AND petty_cash_details.is_delete = 0 AND petty_cash_details.collected_user_id= '.$user_id.' GROUP BY petty_cash_details.route_user_id),0) as total_collected_amount'))
                            ->where('route_employees.user_id', $user_id)
                            ->where('route_employees.status', 1)
                            ->leftJoin('routes', 'routes.id', '=', 'route_employees.route_id')
                           // ->leftJoin('petty_cash_details', 'petty_cash_details.route_user_id', '=', 'routes.route_user_id')
                            ->leftJoin('petty_cash_details', function($join){
                                $join->on('petty_cash_details.route_user_id', '=', 'routes.route_user_id');
                                $join->on('petty_cash_details.collected_user_id', '=', 'routes.route_user_id');
                            })
                            ->groupBy('routes.route_name')
                            ->get();
            $user_data = User::select('users.id as user_id','users.name', 'users.email', DB::raw('IFNULL((SELECT COUNT(route_employees.route_id) FROM route_employees WHERE route_employees.route_id = routes.id AND route_employees.status = 1),0) as routes_count'), DB::raw('IFNULL((SELECT SUM(petty_cash_details.amount) FROM petty_cash_details WHERE  petty_cash_details.status = 1 AND petty_cash_details.is_delete = 0 AND petty_cash_details.route_user_id= '.$user_id.'),0) as total_transffered_amount'),

                DB::raw('IFNULL((SELECT SUM(petty_cash_details.amount) FROM petty_cash_details WHERE petty_cash_details.status = 1 AND petty_cash_details.is_delete = 0 AND petty_cash_details.collected_user_id= '.$user_id.' AND petty_cash_details.status=1),0) as total_collected_amount'),

                (DB::raw('IFNULL((SELECT SUM(petty_cash_details.amount) FROM petty_cash_details WHERE petty_cash_details.status = 1 AND petty_cash_details.is_delete = 0 AND petty_cash_details.collected_user_id= '.$user_id.' AND petty_cash_details.status=1),0) - IFNULL((SELECT SUM(petty_cash_details.amount) FROM petty_cash_details WHERE  petty_cash_details.status = 1 AND petty_cash_details.is_delete = 0 AND petty_cash_details.route_user_id= '.$user_id.'),0) as balance_amount')) 
                )
                    ->where('routes.status', 1)->where('routes.is_delete', 0)
                    ->groupBy("users.id")
                    ->leftJoin('route_employees', 'route_employees.user_id', '=', 'users.id')
                    ->leftJoin('routes', 'routes.id', '=', 'route_employees.route_id')
                    ->leftJoin('petty_cash_details', 'petty_cash_details.route_user_id', '=', 'users.id')
                    ->orderBy('users.name')->where('users.id',$user_id)->get();
                    $response['user_details'] = $user_data;
                    $response['employee_route_list'] = $employee_route_list;

            return $this->sendResponse($response,'Route Manager Route List');
    }

    public function routeManagerDetailsForManager($user_id){
        $employee_route_list = RouteEmployees::select('route_employees.route_id', 'routes.route_name', DB::raw('IFNULL((SELECT SUM(petty_cash_details.amount) FROM petty_cash_details WHERE petty_cash_details.route_user_id = routes.route_user_id AND petty_cash_details.status = 1 AND petty_cash_details.is_delete = 0 AND petty_cash_details.collected_user_id= '.$user_id.' GROUP BY petty_cash_details.route_user_id),0) as total_collected_amount'))
                            ->where('route_employees.user_id', $user_id)
                            ->where('route_employees.status', 1)
                            ->leftJoin('routes', 'routes.id', '=', 'route_employees.route_id')
                           // ->leftJoin('petty_cash_details', 'petty_cash_details.route_user_id', '=', 'routes.route_user_id')
                            ->leftJoin('petty_cash_details', function($join){
                                $join->on('petty_cash_details.route_user_id', '=', 'routes.route_user_id');
                                $join->on('petty_cash_details.collected_user_id', '=', 'routes.route_user_id');
                            })
                            ->groupBy('routes.route_name')
                            ->get();
            $user_data = User::select('users.id as user_id','users.name', 'users.email', DB::raw('IFNULL((SELECT COUNT(route_employees.route_id) FROM route_employees WHERE route_employees.route_id = routes.id AND route_employees.status = 1),0) as routes_count'), DB::raw('IFNULL((SELECT SUM(petty_cash_details.amount) FROM petty_cash_details WHERE  petty_cash_details.status = 1 AND petty_cash_details.is_delete = 0 AND petty_cash_details.route_user_id= '.$user_id.'),0) as total_transffered_amount'),

                DB::raw('IFNULL((SELECT SUM(petty_cash_details.amount) FROM petty_cash_details WHERE petty_cash_details.status = 1 AND petty_cash_details.is_delete = 0 AND petty_cash_details.collected_user_id= '.$user_id.' AND petty_cash_details.status=1),0) as total_collected_amount'),

                (DB::raw('IFNULL((SELECT SUM(petty_cash_details.amount) FROM petty_cash_details WHERE petty_cash_details.status = 1 AND petty_cash_details.is_delete = 0 AND petty_cash_details.collected_user_id= '.$user_id.' AND petty_cash_details.status=1),0) - IFNULL((SELECT SUM(petty_cash_details.amount) FROM petty_cash_details WHERE  petty_cash_details.status = 1 AND petty_cash_details.is_delete = 0 AND petty_cash_details.route_user_id= '.$user_id.'),0) as balance_amount')) 
                )
                    ->where('routes.status', 1)->where('routes.is_delete', 0)
                    ->groupBy("users.id")
                    ->leftJoin('route_employees', 'route_employees.user_id', '=', 'users.id')
                    ->leftJoin('routes', 'routes.id', '=', 'route_employees.route_id')
                    ->leftJoin('petty_cash_details', 'petty_cash_details.route_user_id', '=', 'users.id')
                    ->orderBy('users.name')->where('users.id',$user_id)->get();
                    $response['user_details'] = $user_data;
                    $response['employee_route_list'] = $employee_route_list;

            return $this->sendResponse($response,'Route Manager Route List');
    }

    public function routeManagersList(){
        $routes_managers = User::select('users.id as user_id','users.name', 'users.email', DB::raw('(SELECT COUNT(route_employees.route_id) FROM route_employees WHERE route_employees.user_id = users.id AND route_employees.status = 1) as routes_count'), 

            DB::raw('IFNULL((SELECT SUM(petty_cash_details.amount) FROM petty_cash_details WHERE petty_cash_details.status = 1 AND petty_cash_details.is_delete = 0 AND petty_cash_details.collected_user_id= users.id AND petty_cash_details.status=1),0) - IFNULL((SELECT SUM(petty_cash_details.amount) FROM petty_cash_details WHERE  petty_cash_details.status = 1 AND petty_cash_details.is_delete = 0 AND petty_cash_details.route_user_id= users.id),0) as petty_cash'))
        ->where('routes.status', 1)->where('routes.is_delete', 0)
        ->groupBy("users.id")
        ->leftJoin('route_employees', 'route_employees.user_id', '=', 'users.id')
        ->leftJoin('routes', 'routes.id', '=', 'route_employees.route_id')
        ->leftJoin('petty_cash_details', 'petty_cash_details.route_user_id', '=', 'users.id')
        ->orderBy('users.name')->paginate(50);

        return $this->sendResponse($routes_managers,'Route Managers List');
    }

    public function routeManagersListForManager(){
        $login_user_id = auth('api')->user()->id;
        $routes_managers = User::select('users.id as user_id','users.name', 'users.email', DB::raw('(SELECT COUNT(route_employees.route_id) FROM route_employees WHERE route_employees.user_id = users.id AND route_employees.status = 1) as routes_count'), 

            DB::raw('IFNULL((SELECT SUM(petty_cash_details.amount) FROM petty_cash_details WHERE petty_cash_details.status = 1 AND petty_cash_details.is_delete = 0 AND petty_cash_details.collected_user_id= users.id AND petty_cash_details.status=1),0) - IFNULL((SELECT SUM(petty_cash_details.amount) FROM petty_cash_details WHERE  petty_cash_details.status = 1 AND petty_cash_details.is_delete = 0 AND petty_cash_details.route_user_id= users.id),0) as petty_cash'))
        ->groupBy("users.id")
        ->whereNotIn('users.id', [$login_user_id])
        ->where('roles.role', 'route_manager')
        ->leftJoin('route_employees', 'route_employees.user_id', '=', 'users.id')
        ->leftJoin('roles', 'roles.user_id', '=', 'users.id')
        ->leftJoin('routes', 'routes.id', '=', 'route_employees.route_id')
        ->leftJoin('petty_cash_details', 'petty_cash_details.route_user_id', '=', 'users.id')
        ->orderBy('users.name')->paginate(50);

        return $this->sendResponse($routes_managers,'Route Managers List');
    }

    public function routeManagers(Request $request){
        $query = User::query();

        $query->select('users.id as user_id','users.name', 'users.email', DB::raw('(SELECT COUNT(route_employees.route_id) FROM route_employees WHERE route_employees.route_id = routes.id AND route_employees.status = 1) as routes_count'), DB::raw('(SELECT SUM(petty_cash_details.amount) FROM petty_cash_details WHERE petty_cash_details.collected_user_id=users.id AND petty_cash_details.transfer_type = "manager_transfer" AND petty_cash_details.status = 1 AND petty_cash_details.is_delete = 0) as collected_amount'))
        ->where('routes.status', 1)->where('routes.is_delete', 0)
        ->groupBy("users.id")
        ->leftJoin('route_employees', 'route_employees.user_id', '=', 'users.id')
        ->leftJoin('petty_cash_details', 'petty_cash_details.collected_user_id', '=', 'users.id')
        ->leftJoin('routes', 'routes.id', '=', 'route_employees.route_id')
        ->orderBy('users.name');
        if($request->has('route_id') && $request->route_id!= ''){
            $query = $query->where('route_employees.route_id', '=' ,$request->route_id);
        }
        $routes_managers= $query->get();

        return $this->sendResponse($routes_managers,'Route Managers List');
    }

    public function routeCustomerListOld(){

        $day = strtolower(date("D"));
        $date = date('Y-m-d');
        $login_user_id = auth('api')->user()->id;
        if($login_user_id){
            $customers = Customers::select('customers.id', 'customers.customer_name', 'customers.phone_number', 'customers.route_position', 'customers.customer_pay_type', DB::raw("MOD(DATEDIFF('".$date."',customers.start_date), 2) AS reminder"))
            
            ->join('routes', 'routes.id', '=', 'customers.route_id')
            ->where('customers.is_delete', 0)
            ->where(function ($query) use ($day, $date) {
                $query->where([ ['customers.customer_type', '=', 'daily']])
                ->orWhere([ ['customers.customer_type', '=', 'custom'],[$day, '=', 1] ])
                ->orWhere([ ['customers.customer_type', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', customers.start_date), 2)"), '=', 0] ]);

                $query->orWhere('customers.status', 1)->orWhere([ ['customers.status', '=', 0], ['customers.inactive_till_date', '<', date('Y-m-d')] ]);
            })
            ->where('routes.route_user_id', $login_user_id)
            //->with('products')
            ->with(['products' => function($query) {
                $query->with('product_details');

            }])

            ->leftJoin('delivery_details', function($join){
                $join->on('delivery_details.customer_id', '=', 'customers.id');
                $join->whereDate('delivery_details.created_at', '=', date('Y-m-d'));
            })
            ->whereNull('delivery_details.customer_id')
            ->orderBy('route_position')
            ->get();

            $customer_ids = $customers->pluck('id')->all();

            $products = Products::select('products.id', 'products.product_name', DB::raw("SUM(customer_products.quantity) AS quantity"))
            ->join('customer_products', 'customer_products.product_id', '=', 'products.id')
            ->whereIn('customer_products.customer_id', $customer_ids)
            ->groupBy('products.id')
            ->get();

            if(!empty($customers) && count($customers)>0){
                $result['delivery_date'] = date('Y-m-d');
                $result['product_summary'] = $products;
                $result['customer_list'] = $customers;
            }else{
                $next_date = date('Y-m-d', strtotime(' +1 day'));
                $next_day_summary = $this->routeProducts($next_date);

                $result['product_summary_date'] = $next_date;
                $result['product_summary'] = $next_day_summary['product_summary'];
            }
            
            return $this->sendResponse($result,'Route Customer List');
        }
        return $this->sendError('Invalid Route'); 
        
    }


    public function routeCustomerList(Request $request){

        if($request->has('delivery_date') && $request->delivery_date!= ''){
            $date   = $request->delivery_date;
            $day    = strtolower(date("D", strtotime($date)));
        }else{
            $day = strtolower(date("D"));
            $date = date('Y-m-d');
        }

        $is_delivered = 0;
        $login_user_id = auth('api')->user()->id;
        if($login_user_id){

            $route_model = Routes::where('route_user_id', $login_user_id)->first();
            $products = DeliveryDetails::join('customers', 'customers.id', '=', 'delivery_details.customer_id')
            ->join('products', 'products.id', '=', 'delivery_details.product_id')
            ->select('products.id AS id' ,'products.product_name', DB::raw("SUM(delivery_details.quantity) as quantity"))
            ->where('customers.is_delete', 0)
            ->where('customers.route_id', '=' ,$route_model->id)
            ->whereDate('delivery_details.created_at', '=' ,$date)
            ->groupBy('products.id')->get();

            $delivered_list = $products->toArray();
            if(!empty($delivered_list)){
                $is_delivered = 1;
            }


            $query = Customers::query();
            $query->select('customers.id', 'customers.customer_name', 'customers.phone_number', 'customers.route_position', 'customers.customer_pay_type','customers.status','customers.inactive_date', 'customers.inactive_from_date', 'customers.inactive_till_date', 'customers.latitude', 'customers.longitude')
            ->leftJoin('customer_products', 'customer_products.customer_id', '=', 'customers.id')
            ->join('routes', 'routes.id', '=', 'customers.route_id')
            ->when([['customers.status', '=', 0]], function($query) use($day, $date){

            })
            ->with(['products'=> function($query) use ($day, $date){

                $query->select('product_id', 'customer_id', DB::raw("SUM(customer_products.quantity) AS quantity"))
                //->where([ ['customer_products.delivery_mode', '=', 'daily']])
                //->orWhere([ ['customer_products.delivery_mode', '=', 'custom'],['customer_products.'.$day, '=', 1] ])
                //->orWhere([ ['customer_products.delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', customer_products.delivery_start_date), 2)"), '=', 0] ])
                
                 ->where(function ($query)  use ($day, $date){
                    $query->where([ ['customer_products.delivery_mode', '=', 'daily']])
                    ->where('customers.status', '=', 1)
                    ->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]])
                    ->orWhere([['customers.status', '=', 0 ], ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date]]);
                })
                ->orWhere(function ($query)  use ($day, $date){
                    $query->where([ ['customer_products.delivery_mode', '=', 'custom'],['customer_products.'.$day, '=', 1] ])
                    ->where('customers.status', '=', 1)
                    ->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]])
                    ->orWhere([['customers.status', '=', 0 ], ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date]]);
                })
                ->orWhere(function ($query)  use ($day, $date){
                    $query->where([ ['customer_products.delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', customer_products.delivery_start_date), 2)"), '=', 0] ])
                    ->where('customers.status', '=', 1)
                    ->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]])
                    ->orWhere([['customers.status', '=', 0 ], ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date]]);
                })
                ->orWhere([ ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date] ])
                ->groupBy('customer_products.product_id', 'customer_products.customer_id')
                ;

                // $query->select('product_id', 'customer_id', DB::raw("SUM(customer_products.quantity) AS quantity"))
                // ->where([ ['customer_products.delivery_mode', '=', 'daily']])
                // ->orWhere([ ['customer_products.delivery_mode', '=', 'custom'],[$day, '=', 1] ])
                // ->orWhere([ ['customer_products.delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', customer_products.delivery_start_date), 2)"), '=', 0] ])
                // ->orWhere([ ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date] ])
                // ->groupBy('customer_products.product_id', 'customer_products.customer_id')
                // ;
            }])
            ->whereHas('product', function($query) use ($day, $date){
                $query->where([ ['delivery_mode', '=', 'daily']])
                ->orWhere([ ['delivery_mode', '=', 'custom'],[$day, '=', 1] ])
                ->orWhere([ ['delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', delivery_start_date), 2)"), '=', 0] ])
                ->orWhere([ ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date] ]);
            })
            ->where('routes.route_user_id', $login_user_id)
            ->leftJoin('delivery_details', function($join){
                $join->on('delivery_details.customer_id', '=', 'customers.id');
                $join->whereDate('delivery_details.created_at', '=', date('Y-m-d'));
            })
            ->whereNull('delivery_details.customer_id')
            ->where('customers.is_delete', 0)
            ->where(function ($query)  use ($date){
                $query->where('customers.status', '=', 1)
                ->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]])
                ->orWhere([['customers.status', '=', 0 ], ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date]]);
            })
            ->groupBy('customers.id')
            ->orderBy('route_position');
            // ->get();

            $customers = $query->get();

            $customer_ids = $customers->pluck('id')->all();
           // print("<pre>"); print_r($customer_ids); print("</pre>"); exit;

            $products = Products::select('products.id', 'products.product_name', DB::raw("SUM(customer_products.quantity) AS quantity"))
            ->join('customer_products', 'customer_products.product_id', '=', 'products.id')
            ->leftJoin('customers', 'customers.id', '=', 'customer_products.customer_id')
            ->whereIn('customer_products.customer_id', $customer_ids)
            ->where(function ($query) use ($day, $date) {

                $query->where(function ($query) use ($day,$date){
                    $query->where([['customer_products.delivery_mode', '=', 'daily']])
                        ->where(function ($query)  use ($date){
                        $query->where('customers.status', '=', 1)
                        ->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]])
                        ->orWhere([['customers.status', '=', 0 ], ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date]]);
                    })->orWhere([['customer_products.delivery_mode', '=', 'custom'],['customer_products.'.$day, '=', 1]])
                        ->where(function ($query)  use ($date){
                            $query->where('customers.status', '=', 1)
                                    ->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]])
                                    ->orWhere([['customers.status', '=', 0 ], ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date]]);
                    })->orWhere([['customer_products.delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', customer_products.delivery_start_date), 2)"), '=', 0]])
                        ->where(function ($query)  use ($date){
                            $query->where('customers.status', '=', 1)
                                    ->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]])
                                    ->orWhere([['customers.status', '=', 0 ], ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date]]);
                    })->orWhere([['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date]])
                        ->where(function ($query)  use ($date){
                            $query->where('customers.status', '=', 1)
                                    ->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]])
                                    ->orWhere([['customers.status', '=', 0 ], ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date]]);
                    });
                });

                // $query->where([ ['customer_products.delivery_mode', '=', 'daily']])
                // ->orWhere([ ['customer_products.delivery_mode', '=', 'custom'],['customer_products.'.$day, '=', 1] ])
                // ->orWhere([ ['customer_products.delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', customer_products.delivery_start_date), 2)"), '=', 0] ])
                // ->orWhere([ ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date] ])
                // ->where(function ($query)  use ($date){
                // $query->where('customers.status', '=', 1)
                // ->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]])
                // ->orWhere([['customers.status', '=', 0 ], ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date]]);
                // });

            })
            ->groupBy('products.id')
            ->get();

            if(!empty($customers) && count($customers)>0){
                $result['delivery_date'] = $date;
                $result['product_summary'] = $products;
                $result['customer_list'] = $customers;
            }else{
                $next_date = date('Y-m-d', strtotime(' +1 day'));
                $next_day_summary = $this->routeProducts($next_date);

                $result['product_summary_date'] = $next_date;
                $result['product_summary'] = $next_day_summary['product_summary'];
            }
            
            return $this->sendResponse($result,'Route Customer List');
        }
    }


    public function routeCustomerList1(){

        $day = strtolower(date("D"));
        $date = date('Y-m-d');
        $login_user_id = auth('api')->user()->id;
        if($login_user_id){
            $customers = Customers::select('customers.id', 'customers.customer_name', 'customers.phone_number', 'customers.route_position', 'customers.customer_pay_type')
            
            ->join('routes', 'routes.id', '=', 'customers.route_id')
           // ->join('customer_products', 'customer_products.customer_id', '=', 'customers.id')
            

            // ->leftJoin('customer_products', function($join) use ($day, $date){
            //         $join->on('customer_products.customer_id', '=', 'customers.id');
            //         $join->where([ ['customer_products.delivery_mode', '=', 'daily']])
            //             ->orWhere([ ['customer_products.delivery_mode', '=', 'custom'],['customer_products.'.$day, '=', 1] ])
            //             ->orWhere([ ['customer_products.delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', customer_products.delivery_start_date), 2)"), '=', 0] ]);
            // })
            ->where('customers.is_delete', 0)
            

            // ->where(function ($query) use ($day, $date) {
            //     // $query->where([ ['customer_products.delivery_mode', '=', 'daily']])
            //     //         ->orWhere([ ['customer_products.delivery_mode', '=', 'custom'],['customer_products.'.$day, '=', 1] ])
            //     //         ->orWhere([ ['customer_products.delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', customer_products.delivery_start_date), 2)"), '=', 0] ]);

            //     //$query->orWhere('customers.status', 1)->orWhere([ ['customers.status', '=', 0], ['customers.inactive_till_date', '<', date('Y-m-d')] ]);
            // })
            ->where('routes.route_user_id', $login_user_id)
            ->whereHas(['products' => function($query) use ($day, $date){
                $query->where([ ['delivery_mode', '=', 'daily']])
                ->orWhere([ ['delivery_mode', '=', 'custom'],[$day, '=', 1] ])
                ->orWhere([ ['delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', delivery_start_date), 2)"), '=', 0] ]);
            }, 'products.product_details' => function ($query) {
                $query->select('id', 'product_name');
            }])



            // ->with('products:product_id,customer_id,quantity', 'products.product_details:id,product_name,price')


            // ->with(['products' => function($query) use ($day, $date){
            //     $query->select('customer_products.customer_id')->where([ ['customer_products.delivery_mode', '=', 'daily']])
            //             ->orWhere([ ['customer_products.delivery_mode', '=', 'custom'],['customer_products.'.$day, '=', 1] ])
            //             ->orWhere([ ['customer_products.delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', customer_products.delivery_start_date), 2)"), '=', 0] ]);
            // }])

            ->leftJoin('delivery_details', function($join){
                $join->on('delivery_details.customer_id', '=', 'customers.id');
                $join->whereDate('delivery_details.created_at', '=', date('Y-m-d'));
            })
            ->whereNull('delivery_details.customer_id')
            ->orderBy('route_position')
            ->groupBy('customers.id')
            ->get();

            $customer_ids = $customers->pluck('id')->all();

            $products = Products::select('products.id', 'products.product_name', DB::raw("SUM(customer_products.quantity) AS quantity"))
            ->join('customer_products', 'customer_products.product_id', '=', 'products.id')
            ->whereIn('customer_products.customer_id', $customer_ids)
            ->groupBy('products.id')
            ->get();

            if(!empty($customers) && count($customers)>0){
                $result['delivery_date'] = date('Y-m-d');
                $result['product_summary'] = $products;
                $result['customer_list'] = $customers;
            }else{
                $next_date = date('Y-m-d', strtotime(' +1 day'));
                $next_day_summary = $this->routeProducts($next_date);

                $result['product_summary_date'] = $next_date;
                $result['product_summary'] = $next_day_summary['product_summary'];
            }
            
            return $this->sendResponse($result,'Route Customer List');
        }
        return $this->sendError('Invalid Route'); 
        
    }


    public function routeProductSummaryOld(Request $request){

        if($request->has('delivery_date') && $request->delivery_date!= ''){
            $date   = $request->delivery_date;
            $day    = strtolower(date("D", strtotime($date)));
        }else{
            $date   = date('Y-m-d');
            $day    = strtolower(date("D"));
        }
        
        
        $login_user_id = auth('api')->user()->id;
        if($login_user_id){

            $products = Products::select('products.id', 'products.product_name', DB::raw("SUM(customer_products.quantity) AS quantity"))
            ->join('customer_products', 'customer_products.product_id', '=', 'products.id')
            ->join('customers', 'customer_products.customer_id', '=', 'customers.id')
            ->join('routes', 'customers.route_id', '=', 'routes.id')
            ->where('customers.is_delete', 0)
            ->where('customers.status', 1)
                //->whereIn('customer_products.customer_id', $customer_ids)
            ->where(function ($query) use ($day, $date) {
                $query->where([ ['customers.customer_type', '=', 'daily'] ])->orWhere([ ['customers.customer_type', '=', 'custom'],[$day, '=', 1] ])->orWhere([ ['customers.customer_type', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', customers.start_date), 2)"), '=', 0] ]);
            })
            ->where('routes.route_user_id', $login_user_id)
            ->groupBy('products.id')
            ->get();

            $result['delivery_date'] = $date;
            $result['product_summary'] = $products;
            
            return $this->sendResponse($result,'Route Product Summary');
        }
        return $this->sendError('Invalid Route'); 
        
    }

    public function routeProductSummary(Request $request){

        $delivery_flag = 0;
        $c_date = date('Y-m-d');
        if($request->has('delivery_date') && $request->delivery_date!= ''){
            $date   = $request->delivery_date;
            $day    = strtolower(date("D", strtotime($date)));
            Log::channel('product_summary')->info("Product Summary Date : ".$date);

            if($date <= $c_date){
                $delivery_flag = 1;
            }
        }else{
            $date   = date('Y-m-d');
            $day    = strtolower(date("D"));
        }
        
        Log::channel('product_summary')->info("Product Summary Date : ".$date);
        
        $login_user_id = auth('api')->user()->id;
        if($login_user_id){

            $supply_mode = 'invalid';
            $delivered_list = [];
            //previous date delivery
            if($delivery_flag == 1){


                $route_model = Routes::where('route_user_id', $login_user_id)->first();
                $products = DeliveryDetails::join('customers', 'customers.id', '=', 'delivery_details.customer_id')
                ->join('products', 'products.id', '=', 'delivery_details.product_id')
                ->select('products.id AS id' ,'products.product_name', DB::raw("SUM(delivery_details.quantity) as quantity"))
                ->where('customers.is_delete', 0)
                ->where('customers.route_id', '=' ,$route_model->id)
                ->whereDate('delivery_details.created_at', '=' ,$date)
                ->groupBy('products.id')->get();

                $delivered_list = $products->toArray();
                if(!empty($delivered_list)){
                    $supply_mode = 'delivered';
                }

            }

            if(($delivery_flag == 0 || empty($delivered_list)) && $date >= $c_date){
                $supply_mode = 'future';
                $products = Products::select('products.id', 'products.product_name', DB::raw("SUM(customer_products.quantity) AS quantity"))
                ->join('customer_products', 'customer_products.product_id', '=', 'products.id')
                ->join('customers', 'customer_products.customer_id', '=', 'customers.id')
                ->join('routes', 'customers.route_id', '=', 'routes.id')
                ->where('customers.is_delete', 0)
                //->where('customers.status', 1)
                //->whereIn('customer_products.customer_id', $customer_ids)
                ->where(function ($query) use ($day, $date) {

                    $query->where(function ($query) use ($day,$date){
                        $query->where([['customer_products.delivery_mode', '=', 'daily']])
                            ->where(function ($query)  use ($date){
                            $query->where('customers.status', '=', 1)
                            ->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]])
                            ->orWhere([['customers.status', '=', 0 ], ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date]]);
                        })->orWhere([['customer_products.delivery_mode', '=', 'custom'],['customer_products.'.$day, '=', 1]])
                            ->where(function ($query)  use ($date){
                                $query->where('customers.status', '=', 1)
                                        ->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]])
                                        ->orWhere([['customers.status', '=', 0 ], ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date]]);
                        })->orWhere([['customer_products.delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', customer_products.delivery_start_date), 2)"), '=', 0]])
                            ->where(function ($query)  use ($date){
                                $query->where('customers.status', '=', 1)
                                        ->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]])
                                        ->orWhere([['customers.status', '=', 0 ], ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date]]);
                        })->orWhere([['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date]])
                            ->where(function ($query)  use ($date){
                                $query->where('customers.status', '=', 1)
                                        ->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]])
                                        ->orWhere([['customers.status', '=', 0 ], ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date]]);
                        });
                    });
                    /*
                    $query->where([ ['customer_products.delivery_mode', '=', 'daily'] ])->orWhere([ ['customer_products.delivery_mode', '=', 'custom'],['customer_products.'.$day, '=', 1] ])->orWhere([ ['customer_products.delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', customer_products.delivery_start_date), 2)"), '=', 0] ]);
                    */
                })
                ->where('routes.route_user_id', $login_user_id)
                ->where(function ($query)  use ($date){

                    $query->where('customers.status', '=', 1)
                        ->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]])
                        ->orWhere([['customers.status', '=', 0 ], ['customer_products.delivery_mode', '=', 'order'],['customer_products.delivery_start_date', '=', $date]]);
                
                    /*
                    $query->where('customers.status', '=', 1)
                    ->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]]);
                    */
                })
                ->groupBy('products.id')
                ->get();
            }
            

            $result['delivery_date'] = $date;
            $result['supply_mode'] = $supply_mode;
            $result['product_summary'] = $products;
            
            return $this->sendResponse($result,'Route Product Summary');
        }
        return $this->sendError('Invalid Route'); 
        
    }

    
    public function routeProducts($delivery_date){

        $date   = $delivery_date;
        $day    = strtolower(date("D", strtotime($date)));
        $login_user_id = auth('api')->user()->id;
        if($login_user_id){

            $products = Products::select('products.id', 'products.product_name', DB::raw("SUM(customer_products.quantity) AS quantity"))
            ->join('customer_products', 'customer_products.product_id', '=', 'products.id')
            ->join('customers', 'customer_products.customer_id', '=', 'customers.id')
            ->join('routes', 'customers.route_id', '=', 'routes.id')
            ->where('customers.is_delete', 0)
            ->where(function ($query)  use ($date){
                $query->where('customers.status', '=', 1)
                ->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]]);
            })
                //->whereIn('customer_products.customer_id', $customer_ids)
            ->where(function ($query) use ($day, $date) {
                $query->where([ ['customer_products.delivery_mode', '=', 'daily'] ])->orWhere([ ['customer_products.delivery_mode', '=', 'custom'],['customer_products.'.$day, '=', 1] ])->orWhere([ ['customer_products.delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', customer_products.delivery_start_date), 2)"), '=', 0] ]);
            })
            ->where('routes.route_user_id', $login_user_id)
            ->groupBy('products.id')
            ->get();

            $result['product_summary_date'] = $date;
            $result['product_summary'] = $products;
            return $result;
        }
    }


    public function custometPaymentList(){

        $login_user_id = auth('api')->user()->id;

        $last_month_end = date('Y-m-d', strtotime('last day of last month'));
        if($login_user_id){
            $customers = Customers::
            select('customers.id', 'customers.customer_name', 'customers.phone_number', 'customers.route_position', 'customers.customer_pay_type', DB::raw("(SUM(delivery_details.amount) - customers.advance_amount) as pending_amount"), 'customers.status', 'customers.inactive_date', 'customers.inactive_from_date', 'customers.inactive_till_date','customers.inactive_type', 'customers.latitude', 'customers.longitude')
            ->leftJoin('routes', function($join) use ($login_user_id){
                $join->on('routes.id', '=', 'customers.route_id');
            })

            // ->join('customer_payments', function($join){
            //     $join->on('customer_payments.customer_id', '=', 'customers.id')
            //     //$join->select('customer_payments.created_at');
            //     ->where('customer_payments.created_at', '>=', date('Y-m-d').' 00:00:00');


            // })
            ->with(['today_payment'])
            ->leftJoin('delivery_details', function($join) use($last_month_end){
                $join->on('delivery_details.customer_id', '=', 'customers.id');
                $join->where('delivery_details.payment_status', 0);
                $join->whereDate('delivery_details.created_at', '<=', $last_month_end);
            })
            ->where('routes.route_user_id', $login_user_id)
            ->where('customers.is_delete', 0)
            
            ->orderBy('route_position')
            ->groupBy('customers.id')
            ->get();

            $result['delivery_date'] = date('Y-m-d');
            $result['payment_date'] = $last_month_end;
            $result['customer_list'] = $customers;
            return $this->sendResponse($result,'Route Customer Payment List');
        }
        return $this->sendError('Invalid Route'); 
        
    }


    //individual pending amount details
    public function customerPaymentDetails(Request $request){

        $login_user_id = auth('api')->user()->id;

        if($request->has('customer_id') && $request->customer_id!= ''){
            $customer_id = $request->customer_id;
        }
        if($request->has('end_date') && $request->end_date!= ''){
            $pending_date = $request->end_date;
        }else{
            $pending_date = date('Y-m-d', strtotime('last day of last month'));
        }

        
        if($login_user_id && isset($customer_id)){
            $customers = Customers::
            select('customers.id', 'customers.customer_name', 'customers.phone_number', 'customers.advance_amount', 'customers.route_position', 'customers.customer_pay_type', DB::raw("(SUM(delivery_details.amount) - customers.advance_amount) as pending_amount"))
            
            // ->leftJoin('routes', function($join) use ($login_user_id){
            //     $join->on('routes.id', '=', 'customers.route_id');
            // })

            ->leftJoin('delivery_details', function($join) use($pending_date){
                $join->on('delivery_details.customer_id', '=', 'customers.id');
                $join->where('delivery_details.payment_status', 0);
                $join->whereDate('delivery_details.created_at', '<=', $pending_date);
            })
            //->where('routes.route_user_id', $login_user_id)
            //->where('customers.status', 1)
            ->where('customers.is_delete', 0)
            ->where('customers.id', $customer_id)
            ->orderBy('route_position')
            ->groupBy('customers.id')
            ->first();

            $result['customer_details'] = $customers;
            return $this->sendResponse($result,'Route Customer Payment Details');
        }
        return $this->sendError('Invalid Customer'); 
        
    }

    //individual pending amount details
    public function customerPaymentDetailsForManager(Request $request){

        $login_user_id = auth('api')->user()->id;

        if($request->has('customer_id') && $request->customer_id!= ''){
            $customer_id = $request->customer_id;
        }
        if($request->has('end_date') && $request->end_date!= ''){
            $pending_date = $request->end_date;
        }else{
            $pending_date = date('Y-m-d', strtotime('last day of last month'));
        }

        
        if($login_user_id && isset($customer_id)){
            $customers = Customers::
            select('customers.id', 'customers.customer_name', 'customers.phone_number', 'customers.advance_amount', 'customers.route_position', 'customers.customer_pay_type', DB::raw("(SUM(delivery_details.amount) - customers.advance_amount) as pending_amount"))
            
            // ->leftJoin('routes', function($join) use ($login_user_id){
            //     $join->on('routes.id', '=', 'customers.route_id');
            // })

            ->leftJoin('delivery_details', function($join) use($pending_date){
                $join->on('delivery_details.customer_id', '=', 'customers.id');
                $join->where('delivery_details.payment_status', 0);
                $join->whereDate('delivery_details.created_at', '<=', $pending_date);
            })
            //->where('routes.route_user_id', $login_user_id)
            //->where('customers.status', 1)
            ->where('customers.is_delete', 0)
            ->where('customers.id', $customer_id)
            ->orderBy('route_position')
            ->groupBy('customers.id')
            ->first();

            $result['customer_details'] = $customers;
            return $this->sendResponse($result,'Route Customer Payment Details');
        }
        return $this->sendError('Invalid Customer'); 
        
    }

    public function routeManagerRoutes(){

        $route_manager_user_id = auth('api')->user()->id;
        $routes_list = RouteEmployees::select('route_employees.route_id', 'routes.route_name', 'routes.petty_cash', 'users.email', DB::raw('IFNULL((SELECT SUM(petty_cash_details.amount) FROM petty_cash_details WHERE petty_cash_details.route_user_id = routes.route_user_id AND petty_cash_details.status = 1 AND petty_cash_details.is_delete = 0 AND petty_cash_details.collected_user_id= '.$route_manager_user_id.' GROUP BY petty_cash_details.route_user_id),0) as total_collected_amount'), DB::raw('(SELECT COUNT(customers.id) FROM customers WHERE customers.route_id = routes.id AND customers.is_delete = 0) as customer_count'), DB::raw('(SELECT COUNT(customers.id) FROM customers WHERE customers.route_id = routes.id AND customers.is_delete = 0 AND customers.status = 0) as inactive_customer_count'))
                            ->where('route_employees.user_id', $route_manager_user_id)
                            ->where('route_employees.status', 1)
                            ->leftJoin('routes', 'routes.id', '=', 'route_employees.route_id')
                            ->leftJoin('users', 'users.id', '=', 'routes.route_user_id')
                            ->leftJoin('petty_cash_details', function($join){
                                $join->on('petty_cash_details.route_user_id', '=', 'routes.route_user_id');
                                $join->on('petty_cash_details.collected_user_id', '=', 'routes.route_user_id');
                            })
                            ->groupBy('routes.route_name')
                            ->paginate(50);
            return $this->sendResponse($routes_list,'Route Manager Route List');
    }


    public function routeManagerRoutesList(){

        $route_manager_user_id = auth('api')->user()->id;
        $routes = Routes::leftJoin('customers', 'customers.route_id', '=', 'routes.id')
        ->select('routes.*', 'routes.route_name', 'users.email', DB::raw('(SELECT COUNT(customers.id) FROM customers WHERE customers.route_id = routes.id AND customers.is_delete = 0 ) as customer_count'), DB::raw('(SELECT COUNT(customers.id) FROM customers WHERE customers.route_id = routes.id AND customers.is_delete = 0 AND customers.status = 1) as inactive_customer_count'))
        ->leftJoin('users', 'users.id', '=', 'routes.route_user_id')
        ->join('route_employees', 'route_employees.route_id', '=', 'routes.id')
        ->where('routes.status', 1)
        ->where('route_employees.user_id', '=', $route_manager_user_id)
        ->where('routes.is_delete', 0)
        ->groupBy("routes.id")->get();

        return $this->sendResponse($routes,'Route Manager Route List');
    }

    public function viewManagerRoute($id)
    {
        $routes = Routes::select('routes.id', 'routes.route_user_id', 'routes.route_name', 'users.email', 'routes.petty_cash', DB::raw("COUNT(customers.id) as 'customer_count'"), DB::raw('(SELECT COUNT(customers.id) FROM customers WHERE customers.route_id = routes.id AND customers.is_delete = 0 AND customers.status = 0) as inactive_customer_count'))

        ->leftJoin('customers', function($join){
            $join->on('customers.route_id', '=', 'routes.id');
                    //$join->where('customers.status', 1);
            $join->where('customers.is_delete', 0);
        })
        ->leftJoin('users', 'users.id', '=', 'routes.route_user_id')
        ->where('routes.id', $id)
        ->groupBy("users.id", "routes.route_user_id")
        ->orderBy('route_name')
        ->get();

        if($routes){
            return $this->sendResponse($routes->toArray(),'Route Details');
        }else{
            return $this->sendError('Route Not Found');  
        }
    }

    


}
