<?php

namespace App\Http\Controllers;

use DB;
use Illuminate\Http\Request;
use App\Models\Routes;
use App\Models\Customers;
use App\Models\Products;
use App\Models\DeliveryDetails;
use App\Models\DeliverySummary;
use Maatwebsite\Excel\Facades\Excel;
use App\Exports\Delivery;
use App\Http\Traits\ProductTrait;

class SupportController extends Controller
{
	use ProductTrait;

	public function routeList()
	{
		$routes = Routes::where('is_delete', 0)->orderBy('route_name')->pluck('route_name', 'id');
		$products = Products::where('is_delete', 0)->orderBy('product_name')->pluck('product_name', 'id');
		return view('support.index', compact('routes', 'products'));
	}


	public function checkDuplicate($delivery_date)
	{
		$results = DB::select("select customers.route_id,delivery_details.customer_id,delivery_details.product_id,delivery_details.quantity, DATE(delivery_details.created_at), count(*) as NumDuplicates from delivery_details  LEFT JOIN customers ON customers.id = delivery_details.customer_id WHERE DATE(delivery_details.created_at) = '$delivery_date' group by delivery_details.customer_id,delivery_details.product_id,delivery_details.quantity, DATE(delivery_details.created_at) having NumDuplicates > 1 ORDER BY DATE(delivery_details.created_at) DESC");

		$customer_ids = array();
		print("<pre>"); print_r($results); print("</pre>"); 
		
		foreach($results as $key=>$data){
			$customer_ids[] = $data->customer_id;
		}

		echo implode(',', $customer_ids);
		//print("<pre>"); print_r($customer_ids); print("</pre>"); exit;
		exit;
	}  

	public function routeCheck(Request $request){
		$validated = $request->validate([
			'route_id' => 'required',
			'delivery_date' => 'required',
			'product_id' => 'required',
		]
	);

		return \Redirect::route('route.supply_details', ['route_id'=>$request->route_id, 'delivery_date' => $request->delivery_date, 'product_id' => $request->product_id]);
	}

	public function routeDetails($route_id, $delivery_date, $product_id){

		$route = Routes::find($route_id);
		$product = Products::find($product_id);

		$delivery_date = date('Y-m-d', strtotime($delivery_date));
		$customers = Customers::select('customers.*', 'delivery_details.product_id', 'products.product_name', 'delivery_details.quantity', 'delivery_details.payment_status')
		->leftJoin('delivery_details', function($join) use($delivery_date, $product_id) {
			$join->on('customers.id', '=', 'delivery_details.customer_id');
			$join->whereDate('delivery_details.created_at', '=' ,$delivery_date);
			$join->join('products','products.id', '=','delivery_details.product_id' );
			$join->where('delivery_details.product_id', '=', $product_id);

		})
		->join('customer_products', 'customer_products.customer_id', '=', 'customers.id')
		->where('customers.route_id', '=' ,$route_id)
		->where('customers.is_delete', '=' ,0)
    		//->where('customer_products.product_id', '=', $product_id)
		->groupBy('customer_products.customer_id')
		->orderBy('customers.route_position')
		->get();
    		//print("<pre>"); print_r($customers); print("</pre>"); exit;

		return view('support.delivery', compact('customers', 'route', 'product'));
	}


	public function submitDeliveryOld(Request $request)
	{
		$data = $request->all();
		$quantity 		= isset($data['quantity'])?$data['quantity']:0;
		$delivery_date 	= date('Y-m-d', strtotime($data['delivery_date']));
		$customer_id 	= $data['customer_id'];
		$product_id 	= $data['product_id'];
		$route_id 		= $data['route_id'];

		$product_price = Products::all()->keyBy('id')->toArray();

		$delivery = DeliveryDetails::whereDate('created_at', '=', $delivery_date)->where('customer_id', $customer_id)->where('product_id', $product_id)->first();
		if(empty($delivery)){
			$delivery = new DeliveryDetails();
		}
		$delivery->customer_id  = $customer_id;
		$delivery->product_id   = $product_id;
		$delivery->quantity     = $quantity;
		$delivery->amount       = isset($product_price[$product_id]['price'])?($product_price[$product_id]['price'] * $quantity):0;
		$delivery->created_at   = $delivery_date;
		$delivery->payment_status = 0;
		if($delivery->save()){
			return \Redirect::route('route.supply_details', ['route_id'=>$route_id, 'delivery_date' => $data['delivery_date'], 'product_id' => $product_id])->with('success', 'Delivery updated successfully');
		}else{
			return \Redirect::route('route.supply_details', ['route_id'=>$route_id, 'delivery_date' => $data['delivery_date'], 'product_id' => $product_id])->with('success', 'Delivery not updated');
		}
		
	}


	public function submitDelivery(Request $request)
	{
		$data = $request->all();
		$quantity 		= isset($data['quantity'])?$data['quantity']:0;
		$delivery_date 	= date('Y-m-d', strtotime($data['delivery_date']));
		$customer_id 	= $data['customer_id'];
		$product_id 	= $data['product_id'];
		$route_id 		= $data['route_id'];

		$route_model = Routes::where('id', $route_id)->first();

		$product_price = Products::all()->keyBy('id')->toArray();

		$delivery = DeliveryDetails::whereDate('created_at', '=', $delivery_date)->where('customer_id', $customer_id)->where('product_id', $product_id)->first();
		if(empty($delivery)){
			$delivery = new DeliveryDetails();
		}
		$delivery->customer_id  = $customer_id;
		$delivery->product_id   = $product_id;
		$delivery->quantity     = $quantity;
		$delivery->amount       = isset($product_price[$product_id]['price'])?($product_price[$product_id]['price'] * $quantity):0;
		$delivery->created_at   = $delivery_date;
		$delivery->payment_status = 0;
		if($delivery->save()){


			$delivery_details = $this->routeDeliveryProductCount($route_id, $delivery_date);

			$date   = $delivery_date;
			$delivery_date_time = $delivery_date." ".date('H:i:s');

			$products = $this->routeDayProducts($date, $route_model->route_user_id);
			$actual_products = $products->toArray();

			$actual_product_ids = array_column($actual_products, 'id');
        	$delivered_product_ids = array_keys($delivery_details); //key of this array

        	$all_product_ids = array_unique(array_merge($actual_product_ids, $delivered_product_ids), SORT_REGULAR);


        	foreach ($all_product_ids as $pid) {

        		$index = array_search($pid, array_column($actual_products, 'id'));
        		if(false !== $index){
        			$actual_product_quantity = $actual_products[$index]['quantity'];
        		}else{
        			$actual_product_quantity = 0;
        		}
        		$delivered_quantity = isset($delivery_details[$pid])?$delivery_details[$pid]:0;

        		$summary_exist = DeliverySummary::where([ 
        			['route_id', '=', $route_model->id],
        			['product_id', '=', $pid],
        		])
        		->whereDate('created_at', '=' ,$delivery_date)->first();
        		if(!$summary_exist){
        			$summary_model = new DeliverySummary;
        			$summary_model->route_id = $route_model->id;
        			$summary_model->product_id = $pid;
        			$summary_model->delivered_quantity = $delivered_quantity;
        			$summary_model->actual_quantity = $actual_product_quantity;
        			$summary_model->created_at = $delivery_date_time;
        			$summary_model->save();
        		}else{
        			$summary_exist->delivered_quantity = $delivered_quantity;
        			$summary_exist->actual_quantity = $actual_product_quantity;
        			$summary_exist->save();
        		}

        	}


        	return response()->json(['success'=>'Delivery Saved']);
        }else{
        	return response()->json(['success'=>'Delivery Not Saved']);
        }

    }

    public function submitRowDelivery(Request $request)
    {
    	print("<pre>"); print_r($request->all()); print("</pre>"); exit;
    }


    public function excel(){

    	return Excel::download(new Delivery, 'users.xlsx')->download('xlsx');
    }

    public function deliveryRouteList()
    {
    	$routes = Routes::where('is_delete', 0)->orderBy('route_name')->pluck('route_name', 'id');
    	return view('support.delivery_route_list', compact('routes'));
    } 

    public function deleteDelivery(Request $request){
    	$validated = $request->validate([
    		'route_id' => 'required',
    		'delivery_date' => 'required',
    	]
    );

    	$route_id = $request->route_id;
    	$delivery_date = date('Y-m-d', strtotime($request->delivery_date));


    	$delivery_exist = DeliveryDetails::join('customers','customers.id', '=','delivery_details.customer_id' )
    	->whereDate('delivery_details.created_at', '=', $delivery_date)
    	->where('customers.route_id', '=' ,$route_id)
    	->where('delivery_details.payment_status', '=' ,1)
    	->first();

		//check any payments recieved
    	if(!empty($delivery_exist)){
    		return Redirect()->back()->withError('Payments received for this date and cannot delete delivery');
    	}else{
    		DB::table("delivery_details")
    		->join('customers','customers.id', '=','delivery_details.customer_id' )
    		->where('customers.route_id', '=' ,$route_id)
    		->whereDate('delivery_details.created_at', '=' ,$delivery_date)
    		->delete();

    		DB::table("delivery_summary")
    		->where('delivery_summary.route_id', '=' ,$route_id)
    		->whereDate('delivery_summary.created_at', '=' ,$delivery_date)
    		->delete();

    		return Redirect()->back()->withSuccess('Delivery deleted successfully');
    	}

    }


    public function allroutelist()
    {
    	$routes = Routes::where('is_delete', 0)->orderBy('route_name')->get();
    	return view('support.allroutelist', compact('routes'));
    }


    public function payDeleteDelivery(Request $request){

    	$routes = Routes::where('is_delete', 0)->orderBy('route_name')->get();
    	$route_id           = $request->route_id;
    	$delivery_date      = $request->delivery_date;
    	if($delivery_date != NULL){
    		$delivery_date = date('Y-m-d', strtotime($delivery_date))." 23:59:59";

    		$result = DB::table("delivery_details")
    		->join('customers','customers.id', '=','delivery_details.customer_id' )
    		->where('customers.route_id', '=' ,$route_id)
    		->whereDate('delivery_details.created_at', '<=', $delivery_date)
    		->delete();

    		DB::table("delivery_summary")
    		->where('delivery_summary.route_id', '=' ,$route_id)
    		->whereDate('delivery_summary.created_at', '<=', $delivery_date)
    		->delete();

    		return Redirect()->back()->with('routes', $routes)->withSuccess('Delivery Details Deleted');
    	}else{
    		return Redirect()->back()->with('routes', $routes)->withError('Select Date');
    	}
    }

    public function payDeleteDeliverySummary(Request $request){

    	$routes = Routes::where('is_delete', 0)->orderBy('route_name')->get();
    	$route_id           = $request->route_id;
    	$delivery_date      = $request->delivery_date;
    	if($delivery_date != NULL){
    		$delivery_date = date('Y-m-d', strtotime($delivery_date))." 23:59:59";

    		DB::table("delivery_summary")
    		->where('delivery_summary.route_id', '=' ,$route_id)
    		->whereDate('delivery_summary.created_at', '<=', $delivery_date)
    		->delete();


    		return Redirect()->back()->with('routes', $routes)->withSuccess('Delivery Summary Details Deleted');
    	}else{
    		return Redirect()->back()->with('routes', $routes)->withError('Select Date');
    	}

    }

    public function payDeletePayments(Request $request){

    	$routes = Routes::where('is_delete', 0)->orderBy('route_name')->get();
    	$route_id           = $request->route_id;
    	$delivery_date      = $request->delivery_date;
    	if($delivery_date != NULL){
    		$delivery_date = date('Y-m-d', strtotime($delivery_date))." 23:59:59";

    		DB::table("customer_payments")
    		->join('customers','customers.id', '=','customer_payments.customer_id' )
    		->where('customers.route_id', '=' ,$route_id)
    		->whereDate('customer_payments.created_at', '<=', $delivery_date)
    		->delete();

			//set customer advance amount to 0
    		Customers::where('route_id', $route_id)->update(['advance_amount' => 0]);

			//delete petty cash amount
    		$route_model = Routes::find($route_id);
    		DB::table("petty_cash_details")
    		->where('petty_cash_details.route_user_id', '=' ,$route_model->route_user_id)
    		->whereDate('petty_cash_details.created_at', '<=', $delivery_date)
    		->delete();


    		return Redirect()->back()->with('routes', $routes)->withSuccess('Payment Details Deleted');
    	}else{
    		return Redirect()->back()->with('routes', $routes)->withError('Select Date');
    	}
    }




    public function deliveryList()
    {
        $routes = Routes::where('is_delete', 0)->orderBy('route_name')->pluck('route_name', 'id');
        $products = Products::where('is_delete', 0)->orderBy('product_name')->pluck('product_name', 'id');
        return view('support.route_delivery_index', compact('routes', 'products'));
    } 

    public function deliveryCheck(Request $request){
        $validated = $request->validate([
            'route_id' => 'required',
            'delivery_month' => 'required',
            'product_id' => 'required',
        ]
    );

        return \Redirect::route('route.route_delivery', ['route_id'=>$request->route_id, 'delivery_month' => $request->delivery_month, 'product_id' => $request->product_id]);
    }

    public function routeDelivery($route_id, $delivery_month, $product_id){

        $route = Routes::find($route_id);
        $product = Products::find($product_id);

        $month_start    = date('Y-m-d', strtotime("01-".$delivery_month));
        $month_end      = date('Y-m-t', strtotime($month_start));

        $customers = Customers::select('customers.*', 'delivery_details.product_id', 'products.product_name', 'delivery_details.quantity', 'delivery_details.payment_status')
        ->leftJoin('delivery_details', function($join) use($month_start, $month_end, $product_id) {
            $join->on('customers.id', '=', 'delivery_details.customer_id');
            $join->whereDate('delivery_details.created_at', '>=' ,$month_start);
            $join->whereDate('delivery_details.created_at', '<=' ,$month_end);
            $join->join('products','products.id', '=','delivery_details.product_id' );
            $join->where('delivery_details.product_id', '=', $product_id);

        })
        ->join('customer_products', 'customer_products.customer_id', '=', 'customers.id')
        ->where('customers.route_id', '=' ,$route_id)
        ->where('customers.is_delete', '=' ,0)
            //->where('customer_products.product_id', '=', $product_id)
        ->groupBy('customer_products.customer_id')
        ->orderBy('customers.route_position')
        ->get();

        $customer_list = $customers->toArray();
        //print("<pre>"); print_r($customers); print("</pre>"); exit;
        foreach ($customer_list as $key => $customer) {
            $delivery_data[$customer['id']]['id'] = $customer['id'];
            $delivery_data[$customer['id']]['name'] = $customer['customer_name'];
            $delivery_data[$customer['id']]['status'] = $customer['status'];

            $delivery_start = $month_start;
            $delivery_end = $month_end;
            while (strtotime($delivery_start) <= strtotime($delivery_end)) {

              $exist = DeliveryDetails::where([ 
                ['customer_id', '=', $customer['id']],
                ['product_id', '=', $product_id],
                ])->whereDate('created_at', '=' ,$delivery_start)->first();
                  
                if($exist){
                    $delivery_data[$customer['id']]['delivery'][$delivery_start] = $exist->quantity;
                }else{
                    $delivery_data[$customer['id']]['delivery'][$delivery_start] = 0;
                }
                $delivery_start = date ("Y-m-d", strtotime("+1 days", strtotime($delivery_start)));
            }
    }
    //print("<pre>"); print_r($delivery_data); print("</pre>"); exit;

    return view('support.route_delivery', compact('customers', 'route', 'product', 'delivery_data'), ['month_start'=> $month_start, 'month_end'=>$month_end, 'product_id'=> $product_id]);
}

    public function cDelivery($route_id, $delivered_date, $new_date){

        $delivery_details = DeliveryDetails::join('customers','customers.id', '=','delivery_details.customer_id' )
        ->select('delivery_details.*')
        ->whereDate('delivery_details.created_at', '=', $delivered_date)
        ->where('customers.route_id', '=' ,$route_id)
        ->get();

        //print("<pre>"); print_r($delivery_details->toArray()); print("</pre>"); exit;

        $details = $delivery_details->toArray();
        if(!empty($details)){
            $count = 0;
            foreach ($details as $key => $delivery) {

                $model = DeliveryDetails::where([ 
                    ['customer_id', '=', $delivery['customer_id']],
                    ['product_id', '=', $delivery['product_id']],
                    ['quantity', '=', $delivery['quantity']],
                ])
                ->whereDate('created_at', '=' ,$new_date)->first();

                if(empty($model)){
                    $model = new DeliveryDetails;
                }

                $model->customer_id = $delivery['customer_id'];
                $model->product_id  = $delivery['product_id'];
                $model->quantity    = $delivery['quantity'];
                $model->amount      = $delivery['amount'];
                $model->payment_status = 0;
                $model->created_at = $new_date." 10:00:00";
                $model->save();
                $count++;
            }
        }

        echo "Data saved successfully";
        echo "Count : ".$count; 
        

    }

}
