<?php

namespace App\Http\Controllers;

use DB;
use App\Models\User; 
use App\Models\Role; 
use App\Models\Routes;
use App\Models\UserPermissions;
use App\Models\PettyCashDetails;
use App\Rules\MatchPassword;
use Validator;
use Exception;
use GuzzleHttp\Client;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth; 
use Laravel\Passport\Client as OClient; 
use Illuminate\Support\Facades\URL;
use App\Http\Controllers\BaseController as BaseController;

class UserController extends BaseController
{
	public $successStatus = 200;

	public function login() { 
		if (Auth::attempt(['email' => request('email'), 'password' => request('password')])) { 
			//print("<pre>"); print_r(request('password')); print("</pre>"); exit;
			$user = Auth::user();
			$userRole = $user->role()->first();

			if ($userRole) {
				$this->scope = $userRole->role;

				if($this->scope == 'supplier'){
					$model = Routes::where('route_user_id', $user->id)->first();
					$user['petty_cash'] = $model['petty_cash'];
				}

				if($this->scope == 'route_manager'){
					$petty_cash_details = PettyCashDetails::select(DB::raw('IFNULL((SELECT SUM(petty_cash_details.amount) FROM petty_cash_details WHERE petty_cash_details.status = 1 AND petty_cash_details.is_delete = 0 AND petty_cash_details.collected_user_id= '.$user->id.' AND petty_cash_details.status=1),0) - IFNULL((SELECT SUM(petty_cash_details.amount) FROM petty_cash_details WHERE  petty_cash_details.status = 1 AND petty_cash_details.is_delete = 0 AND petty_cash_details.route_user_id= '.$user->id.' ),0) as petty_cash'))
						->where('petty_cash_details.status', 1)->where('petty_cash_details.is_delete', 0)->first();

						// $query = PettyCashDetails::select(DB::raw('IFNULL((SELECT SUM(petty_cash_details.amount) FROM petty_cash_details WHERE petty_cash_details.status = 1 AND petty_cash_details.collected_user_id= '.$user->id.' AND petty_cash_details.status=1),0) - IFNULL((SELECT SUM(petty_cash_details.amount) FROM petty_cash_details WHERE  petty_cash_details.status = 1 AND petty_cash_details.route_user_id= '.$user->id.' ),0) as petty_cash'))
						// ->where('petty_cash_details.status', 1)->toSql();
						// print("<pre>"); print_r($query); print("</pre>"); exit;

					$user['petty_cash'] = $petty_cash_details->petty_cash;
				}
			}

			$user_permissions = UserPermissions::where('user_id',$user->id)->get();



			$oClient = OClient::where('password_client', 1)->first();
			return $this->getTokenAndRefreshToken($oClient, request('email'), request('password'), $this->scope, $user, $user_permissions);
		} 
		else { 
			return $this->sendError('Invalid username or password', 403);    

			//return response()->json(['success'=>true, 'error'=>'Invalid username or password', 'status'=>200], 401); 
		} 
	}

	public function addUser(Request $request) { 

		
		$validator = Validator::make($request->all(), [ 
			'name' => 'required', 
			'email' => 'required|unique:users', 
			'password' => 'required', 
			'c_password' => 'required|same:password', 
			'role' => 'required', 
		]);


		if ($validator->fails()) { 

			return $this->sendValidationErrors($validator->errors()); 
			//return response()->json(['success'=>false, 'error'=>$validator->errors(),'status'=>200], 401);            
		}

		$password = $request->password;
		$input = $request->all(); 
		$input['password'] = bcrypt($input['password']); 
		$user = User::create($input); 
		if($user->email){
			$role_model 			= new Role;
			$role_model->user_id 	= $user->id;
			$role_model->role 		= $request->role;
			$role_model->save();
		}

		$post_permissions  = json_decode($request->permissions, true); 
		if(in_array('routeSalesConsolidateReport', $post_permissions)){
			$post_permissions[] = 'routeSalesConsolidateReportPdf';
		}

		if(in_array('itemConsolidatedReport', $post_permissions)){
			$post_permissions[] = 'itemConsolidatedReportPdf';
		}

        foreach($post_permissions as $permission){
        	$exist = UserPermissions::where('user_id',$user->id)->where('permission',$permission)->exists();
        	$user_input['user_id'] = $user->id;
        	$user_input['permission'] = $permission;
        	$user_input['status'] = 1;
       		$user_input['created_at'] = date('Y-m-d H:i:s');
       		$user_input['created_by'] = auth('api')->user()->id;
        	
	        if(!$exist){
	        	$permission = UserPermissions::create($user_input);
	        }
        }


		return $this->sendResponse('','User Successfully Added');
		//return response()->json(['success'=>true, 'message'=>'User Successfully Added','status'=>200], 200);    
		//$oClient = OClient::where('password_client', 1)->first();
		//return $this->getTokenAndRefreshToken($oClient, $user->email, $password);
	}

	

	public function getTokenAndRefreshToken(OClient $oClient, $email, $password, $scope, $user, $user_permissions) { 
		$oClient = OClient::where('password_client', 1)->first();
		$http = new Client;
		$response = $http->request('POST', URL::to('/').'/oauth/token', [
			'form_params' => [
				'grant_type' => 'password',
				'client_id' => $oClient->id,
				'client_secret' => $oClient->secret,
				'username' => $email,
				'password' => $password,
				'scope' => $scope,
			],
		]);

		$result = json_decode((string) $response->getBody(), true);
		$result['role'] = $scope;
		$result['profile'] = $user;
		$result['user_permissions'] = $user_permissions;

		return $this->sendResponse($result,'Token'); 
		//return response()->json($result, $this->successStatus);
	}

	public function profile(Request $request) { 

		if (Auth::user()) { 
			$user = Auth::user();

			$model = Routes::where('route_user_id', $user->id)->first();
			$user['petty_cash'] = $model['petty_cash'];
			return $this->sendResponse($user,'User Details'); 
		} 
		else { 
			return $this->sendError('Invalid User', 403);    
		} 
	} 

	public function logout(Request $request) {
		$request->user()->token()->revoke();
		return $this->sendResponse('','Successfully logged out'); 
		//return response()->json(['message' => 'Successfully logged out']);
	}

	public function unauthorized() { 

		return $this->sendError('Unauthorized', 401);  

		//return response()->json("unauthorized", 401); 
	} 

	public function refreshToken(Request $request) { 

		$refresh_token = $request->header('Refreshtoken');
		$oClient = OClient::where('password_client', 1)->first();
		$http = new Client;

		try {
			$response = $http->request('POST', URL::to('/').'/oauth/token', [
				'form_params' => [
					'grant_type' => 'refresh_token',
					'refresh_token' => $refresh_token,
					'client_id' => $oClient->id,
					'client_secret' => $oClient->secret,
					'scope' => '',
				],
			]);
			$result = json_decode((string) $response->getBody(), true);
			return $this->sendResponse($result,'Token'); 
		} catch (Exception $e) {
			return $this->sendError('Unauthorized'); 
			//return response()->json("unauthorized", 401); 
		}
	}


	public function changePassword(Request $request){

		$validator = Validator::make($request->all(), [ 
			'password' => ['required', new MatchPassword],
			'new_password' => 'required',
			'retype_password' => 'same:new_password',
		]);

		
		if ($validator->fails()) { 
			return $this->sendValidationErrors($validator->errors()); 
		}

		$input = $request->all(); 

		$user = User::find(auth('api')->user()->id);
		if($user){

			$user->password = bcrypt($input['new_password']);
			$user->save();
			// User::find(auth('api')->user()->id)->update(['password'=> bcrypt($input['password'])]);

			return $this->sendResponse($user,'Password Successfully Updated');
		}else{
			return $this->sendError('Password Not Updated');  
		}

	}

	public function userList()
    {
        $users = User::select('users.*', 'roles.role')
        ->whereIn('roles.role', ['admin','manager'])
        ->where('users.is_delete', 0)
        ->leftJoin('roles', 'roles.user_id', '=', 'users.id')
        ->orderBy('users.name')->paginate(50);

        return $this->sendResponse($users,'User List');
    }


    public function getAllUserList()
    {
        $users = User::select('users.id as to_user_id', 'users.name as to_user_name')
        ->whereIn('roles.role', ['admin','route_manager','super_admin'])
        ->where('users.is_delete', 0)
        ->leftJoin('roles', 'roles.user_id', '=', 'users.id')
        ->orderBy('users.name')->get()->toArray();

        $all_list = [['to_user_id'=>0, 'to_user_name'=>'All']];
        $user_list = array_merge($all_list, $users);
        return $this->sendResponse($user_list,'All User List');
    }

    public function viewUser($id)
    {
        $user = User::select('users.*', 'roles.role')
        			->where('users.is_delete', 0)
        			->where('users.id', $id)
        			->leftJoin('roles', 'roles.user_id', '=', 'users.id')
    				->get();
    	$permissions = UserPermissions::where('user_id',$id)->where('status',1)->get();

    	$result = ['user_details'=>$user->toArray(), 'user_permissions'=>$permissions];
        

        if($user){
        	return $this->sendResponse($result, 'User Details');
        }else{
            return $this->sendError('User Not Found');  
        }
    }

    public function deleteUser($id)
    {
        $user = User::find($id);
        if($user){
            $user->is_delete = 1;
            if($user->save()){
                return $this->sendResponse('','User Successfully Deleted');
            }else{
                return $this->sendError('User Not Deleted');  
            }
        }else{
            return $this->sendError('User Not Found');  
        }
    }

    public function updateUser(Request $request)
    {

        $input  = $request->all(); 
        $id     = isset($input['id'])?$input['id']:0;
        $user 	= User::find($id);
        if($user){
        	//print("<pre>"); print_r($user); print("</pre>"); exit;

        	if(isset($input['password']) && $input['password']!=null){
                $validator = Validator::make($input, User::passRules($id));
                $input['password'] = bcrypt($input['password']); 
            }else{
            	$validator = Validator::make($input, User::rules($id));
            }

            if ($validator->fails()) { 
                return $this->sendValidationErrors($validator->errors()); 
            }
            //print("<pre>"); print_r($input); print("</pre>"); exit;


            
            $updated = $user->fill($input)->save();
            $post_permissions = array();
            $removed_permissions = array();
            $exist_permission = array();

            if($updated){

            	Role::where('user_id', '=', $user->id)->update(['role'=>$input['role']]);

            	if(isset($request->permission) && $request->permission!=null){
            		$post_permissions  = json_decode($request->permission, true); 
            	}
            	
            	$existing_data = UserPermissions::select('permission')->where('user_id',$user->id)->get();
            	$exist_permissions = $existing_data->pluck('permission')->toArray();
            	

            	$removed_permissions = array_diff($exist_permissions, $post_permissions);
            	if(!in_array('routeSalesConsolidateReport', $removed_permissions)){
					$removed_permissions = array_diff($removed_permissions,array('routeSalesConsolidateReportPdf'));
				}

				if(!in_array('itemConsolidatedReport', $removed_permissions)){
					$removed_permissions = array_diff($removed_permissions,array('itemConsolidatedReportPdf'));
				}
            	if(!empty($removed_permissions)){
            		foreach($removed_permissions as $permission){
            			$model = UserPermissions::where('user_id',$user->id)->where('permission',$permission)->first();
            			$model->delete();
            		}
            	}
            	if(!empty($post_permissions)){

            		if(in_array('routeSalesConsolidateReport', $post_permissions)){
						$post_permissions[] = 'routeSalesConsolidateReportPdf';
					}

					if(in_array('itemConsolidatedReport', $post_permissions)){
						$post_permissions[] = 'itemConsolidatedReportPdf';
					}
					
            		foreach($post_permissions as $permission){
			        	$exist = UserPermissions::where('user_id',$user->id)->where('permission',$permission)->exists();
			        	$user_input['user_id'] = $user->id;
			        	$user_input['permission'] = $permission;
			        	$user_input['status'] = 1;
			       		$user_input['created_at'] = date('Y-m-d H:i:s');
			       		$user_input['created_by'] = auth('api')->user()->id;
			        	
				        if(!$exist){
				        	$permission = UserPermissions::create($user_input);
				        }
			        }
            	}
		        

                return $this->sendResponse($user,'User Successfully Updated');
            }else{
                return $this->sendError('User Not Updated');  
            }
        }else{
            return $this->sendError('User Not Found');  
        }
        
    }

    public function addPermission(Request $request)
    {

        $input = $request->all(); 
        $input['status'] = 1;
       	$input['created_at'] = date('Y-m-d H:i:s');
       	$input['created_by'] = auth('api')->user()->id;

        $validator = Validator::make($input, UserPermissions::rules());

        if ($validator->fails()) { 
            return $this->sendValidationErrors($validator->errors()); 
        }

        $post_permissions  = json_decode($input['permission'], true); 

        foreach($post_permissions['permissions'] as $permission){
        	$exist = UserPermissions::where('user_id',$input['user_id'])->where('permission',$permission)->exists();
        	$input['permission'] = $permission;
	        if(!$exist){
	        	$permission = UserPermissions::create($input);
	        }
        }
        /*
        $exist = UserPermissions::where('user_id',$input['user_id'])->where('permission',$input['permission'])->exists();
        if(!$exist){
        	$permission = UserPermissions::create($input);
        	return $this->sendResponse('','Permission Successfully Added');
        }else{
        	return $this->sendError('Permission Exist'); 
        }
        */
        return $this->sendResponse('','Permission Successfully Added');

    }

	
}
