<?php

namespace App\Http\Traits;

use DB;
use App\Models\Products;
use App\Models\Routes;
use App\Models\DeliveryDetails;
use App\Models\CustomerPayments;
use App\Models\Customers;

trait ProductTrait {

	//actual product count for a date in a route
	public function routeDayProducts($date, $route_user_id){

		$day    = strtolower(date("D", strtotime($date)));
		$products = Products::select('products.id', DB::raw("SUM(customer_products.quantity) AS quantity"))
		->join('customer_products', 'customer_products.product_id', '=', 'products.id')
		->join('customers', 'customer_products.customer_id', '=', 'customers.id')
		->join('routes', 'customers.route_id', '=', 'routes.id')
		->where('customers.is_delete', 0)
		->where(function ($query)  use ($date){
                $query->where('customers.status', '=', 1)
                ->orWhere([['customers.status', '=', 0 ], ['customers.inactive_till_date', '<', $date]]);
            })
		->where(function ($query) use ($day, $date) {
			$query->where([ ['customer_products.delivery_mode', '=', 'daily'] ])->orWhere([ ['customer_products.delivery_mode', '=', 'custom'],['customer_products.'.$day, '=', 1] ])->orWhere([ ['customer_products.delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', customer_products.delivery_start_date), 2)"), '=', 0] ]);
		})
		->where('routes.route_user_id', $route_user_id)
		->groupBy('products.id')
		->get();

		return $products;
	}


	public function routeDeliveryProductCount($route_id, $delivery_date){
		$summary_query = DeliveryDetails::join('customers', 'customers.id', '=', 'delivery_details.customer_id')
		->join('products', 'products.id', '=', 'delivery_details.product_id')
		->select('products.id AS product_id', DB::raw("SUM(delivery_details.quantity) as delivered_quantity"))
		->where('customers.is_delete', 0)
		->where('customers.route_id', '=' ,$route_id)
		->whereDate('delivery_details.created_at', '=' ,$delivery_date)
		->groupBy('products.id')->get();

		$delivered_list = $summary_query->pluck('delivered_quantity', 'product_id');
		return $delivered_list->toArray();

	}

	public function customerPending($customer_id){
		if($customer_id){

			$total_pending_amount = DeliveryDetails::where('delivery_details.payment_status', 0)->where('delivery_details.customer_id', $customer_id)->sum('amount');
			$result['total_pending_amount'] = $total_pending_amount - $customer_data->advance_amount;
			
			return $this->sendResponse($result,'Customer Pending Amount');
		}else{
			return $this->sendError('Invalid Customer'); 
		}
	}

	public function updateCustomerPayments($customer_id,$recent_payment_id){
		DB::table('customer_payments')->where('customer_id',$customer_id)->whereNotIn('id',[$recent_payment_id])->update(['payment_flag'=>0]);
	}

	public function updateCustomerRecentPayment($customer_id){
		$model = CustomerPayments::where('customer_id', $customer_id)->where('status', 'A')->orderBy('created_at', 'desc')->first();
		if($model){
			$model->payment_flag = 1;
			$model->save();
		}
	}


	public function unpaidDelivery($customer_id,$payment_id){
		DB::table('delivery_details')->where('customer_id',$customer_id)->where('payment_id',$payment_id)->update(['payment_status'=>0]);
		DB::table('delivery_details')->where('customer_id',$customer_id)->where('payment_id',$payment_id)->update(['payment_id'=>0]);
	}

	public function sendSms($customer_id,$amount, $last_paid_date, $advance_amount){

		$customer = Customers::select('id', 'customer_name', 'phone_number')->where('id', $customer_id)->first();

		// $data = array(
  //           'username' => 'alrayan',
  //           'password' => '12345',
  //           'mobile' => $customer['phone_number'],
  //           'sendername' => 'RNMILK',
  //           'message' => 'Hello '.$customer['customer_name'].', Your payment '.$amount.' has been received. Thank You.',
  //           'routetype'=>1,
  //           'tid'=>rand(0, 99999)

  //       );
        $data = array(
            'username' => 'alrayan',
            'password' => '12345',
            'mobile' => $customer['phone_number'],
            'sendername' => 'RNMILK',
            'message' => "Dear Sir.Greeting from Grameen Milk. We've received payment RS.".$amount." till ".$last_paid_date.". Advance Collected RS.".$advance_amount."Thank you for shopping with usRayyanMilk",
            'routetype'=>1,
            'tid'=>1607100000000176907

        );
        $url = "http://sapteleservices.com/SMS_API/sendsms.php?";

        $ch = curl_init();
        curl_setopt_array($ch, array(
           CURLOPT_URL => $url,
           CURLOPT_RETURNTRANSFER =>true,
           CURLOPT_POST => true,
           CURLOPT_POSTFIELDS => $data,
        ));

       curl_setopt($ch,CURLOPT_SSL_VERIFYHOST,0);
       curl_setopt($ch,CURLOPT_SSL_VERIFYPEER,0);

       $output = curl_exec($ch);

       if(curl_errno($ch)){

           echo 'error :'. curl_error($ch);

       }

       curl_close($ch);
	}


}


?>