<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Validation\Rule; 
use Illuminate\Support\Facades\Input;
use DB;

class Customers extends Model
{
	use HasFactory;

	protected $deliveryDate;


	public $timestamps = false;

	protected $fillable = [
		'customer_name', 'route_position', 'route_id', 'phone_number', 'address', 'customer_type', 'status', 'sun', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat', 'sun', 'start_date', 'customer_pay_type', 'inactive_till_date','inactive_type'
	];   

	public function route() {
		return $this->belongsTo('App\Models\Routes', 'route_id');
	}

	public function product() {
		return $this->hasMany('App\Models\CustomerProducts', 'customer_id');
	}


	public function order() {
		return $this->hasMany('App\Models\CustomerOrders', 'customer_id');
	}

	public function orders() {
		return $this->hasMany('App\Models\CustomerOrders', 'customer_id');
	}

	public function customer_product() {
		return $this->hasMany('App\Models\CustomerProducts', 'customer_id');
	}


	public function products() {

		return $this->hasMany('App\Models\CustomerProducts', 'customer_id')
					->leftJoin('customers', 'customers.id', '=', 'customer_products.customer_id');
		// $day = strtolower(date("D"));
  //       $date = date('Y-m-d');
		// return $this->hasMany('App\Models\CustomerProducts', 'customer_id')
		// ->select('product_id', 'customer_id', DB::raw("SUM(customer_products.quantity) AS quantity"))
		// ->where([ ['customer_products.delivery_mode', '=', 'daily']])
  //       ->orWhere([ ['customer_products.delivery_mode', '=', 'custom'],[$day, '=', 1] ])
  //       ->orWhere([ ['customer_products.delivery_mode', '=', 'alternative'], [DB::raw("MOD(DATEDIFF('".$date."', customer_products.delivery_start_date), 2)"), '=', 0] ])->groupBy('customer_products.product_id', 'customer_products.customer_id');;
	}


	public function product_summary() {
		return $this->hasMany('App\Models\CustomerProducts', 'customer_id')
				->leftJoin('customers', 'customers.id', '=', 'customer_products.customer_id');
	}

	public function delivery() {
		return $this->hasMany('App\Models\DeliveryDetails', 'customer_id', 'id');
	}

	public function cproducts() 
	{
		return $this->hasManyThrough('App\Models\CustomerProducts', 'App\Models\Products');
	}

	public function custproducts()
	{
		return $this->hasMany('App\Models\CustomerProducts', 'customer_id', 'id');
	}

	public function delivery_details() {
		return $this->hasMany('App\Models\DeliveryDetails', 'customer_id', 'id')
		->leftJoin('products', 'products.id', '=', 'delivery_details.product_id')
		->select('customer_id','delivery_details.product_id', 'products.product_name', 'delivery_details.quantity', 'delivery_details.actual_quantity', 'delivery_details.amount')
		->orderBy('products.product_name', 'asc');
	}

	public function customerdelivery() {
		return $this->hasMany('App\Models\DeliveryDetails', 'customer_id', 'id')
		->leftJoin('products', 'products.id', '=', 'delivery_details.product_id')
		->select('customer_id','delivery_details.product_id', 'products.product_name', 'delivery_details.quantity', 'delivery_details.amount')
		->orderBy('products.product_name', 'asc');
	}

	public function payments() {
		return $this->hasMany('App\Models\CustomerPayments', 'customer_id');
	}

	public function today_payment() {
		return $this->hasOne('App\Models\CustomerPayments', 'customer_id', 'id')->orderBy('created_at', 'desc')->whereDate('customer_payments.created_at', '>=', date('Y-m-d').' 00:00:00');
	}


	public static function rules ($id=0, $route_id, $merge=[]) {
		return array_merge(
			[
				//'customer_name'  => 'required',
				'customer_name' => [
					'required',Rule::unique('customers')->where(function($query) use ($id, $route_id) {
						$query->where('id', '!=', $id)
						->where('route_id', '=', $route_id);
					})
				],
				'address' => 'max:255', 
				'phone_number' => [
					'required','regex:/^([0-9\s\-\+\(\)]*)$/','min:10',Rule::unique('customers')->where(function($query) use ($id, $route_id) {
						$query->where('is_delete', '=', '0')
						->where('id', '!=', $id);
					})
				],
				'customer_pay_type' =>'required|in:online,direct',
				'status' =>'required',
				'route_id' =>'required|exists:routes,id',
				'start_date' => 'required_if:customer_type,alternative',
				'products.*.product_id' => 'required|exists:products,id',
				'products.*.quantity' => 'required',
				'products.*.delivery_mode' => 'required',
				'products.*.delivery_start_date' => 'required_if:products.*.delivery_mode,alternative',

			], 
			$merge);
	}

	public static function pro_rules ($id=0, $merge=[]) {
		return array_merge(
			[
				'*.product_id' => 'required|exists:products,id',
				'*.quantity' => 'required',
				'*.delivery_mode' => 'required',
				//'*.delivery_start_date' => 'required_if:*.delivery_mode,alternative',

			], 
			$merge);
	}

	public static function location_rules ($id=0, $merge=[]) {
		return array_merge(
			[
				'customer_id' => 'required|exists:customers,id',
				'latitude' => 'required',
				'longitude' => 'required',
			], 
			$merge);
	}

	public function specialdelivery() {
		$date = "2021-05-31";

		return $this->hasOne('App\Models\DeliveryDetails', 'customer_id', 'id')
		->select('delivery_details.quantity')
		->whereDate('delivery_details.created_at', '=', $date);
	}
}
