<?php

namespace App\Models;

use Illuminate\Contracts\Auth\MustVerifyEmail;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Laravel\Passport\HasApiTokens;
use App\Models\Roles;
use App\Models\Routes;

class User extends Authenticatable
{
    use HasFactory, Notifiable, HasApiTokens;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'name',
        'email',
        'password',

    ];

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = [
        'password',
        'remember_token',
    ];

    /**
     * The attributes that should be cast to native types.
     *
     * @var array
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
    ];

    public function role() {
        return $this->hasOne(Role::class);
    }

    public function route() {
        return $this->hasOne(Routes::class);
    }

    public static function rules ($id=0, $merge=[]) {
        return array_merge(
            [
                'email'  => 'required|unique:users,email' . ($id ? ",$id" : ''),
                'name' => 'required', 
                //'email' => 'required|unique:users' .($id ? ",$id" : ''), 
                'role' => 'required', 

            ], 
            $merge);
    }

    public static function passRules ($id=0, $merge=[]) {
        return array_merge(
            [
                'name' => 'required', 
                'email'  => 'required|unique:users,email' . ($id ? ",$id" : ''),
                'password' => 'required', 
                'c_password' => 'required|same:password', 
                'role' => 'required', 

            ], 
            $merge);
    }
}
