<?php

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Route;
use App\Http\Controllers\UserController;
use App\Http\Controllers\RoutesController;
use App\Http\Controllers\CustomersController;
use App\Http\Controllers\ProductsController;
use App\Models\User; 

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
|
| Here is where you can register API routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| is assigned the "api" middleware group. Enjoy building your API!
|
*/

Route::post('login', 'App\Http\Controllers\UserController@login');


Route::get('/excel', 'App\Http\Controllers\SupportController@excel');

Route::get('/deliveredCount', 'App\Http\Controllers\CustomersController@deliveredCount');

Route::middleware('auth:api')->get('/user', function (Request $request) {
	return $request->user();
});


Route::get('/unauthorized', 'App\Http\Controllers\UserController@unauthorized');

Route::post('updateProduct', 'App\Http\Controllers\ProductsController@updateProduct')->middleware(["auth:api", "scope:Administrator"]);


Route::middleware(['auth:api', 'role'])->group(function() {

    // List users
    Route::middleware(['scope:admin'])->group(function() {
    	Route::post('add_route', 'App\Http\Controllers\RoutesController@add_route');
    });

    // Route::get('/users', function (Request $request) {

    //     return User::get();
    // })->middleware(['scope:admin']);


});

Route::group(['middleware' => ['auth:api']], function() {
	Route::get('routeList', 'App\Http\Controllers\RoutesController@routeList')->middleware(['scope:admin,super_admin']);
	Route::get('routeListWithPrefix', 'App\Http\Controllers\RoutesController@routeListWithPrefix')->middleware(['scope:admin,super_admin']);
	Route::get('routeCustomers/{route_id}', 'App\Http\Controllers\RoutesController@routeCustomers')->middleware(['scope:admin,super_admin']);
	Route::get('productListData', 'App\Http\Controllers\ProductsController@productListData')->middleware(['scope:admin,super_admin,supplier']);
	Route::get('routeManagers', 'App\Http\Controllers\RoutesController@routeManagers')->middleware(['scope:admin,super_admin']);
	Route::get('routeManagerRoutes', 'App\Http\Controllers\RoutesController@routeManagerRoutes')->middleware(['scope:super_admin,route_manager']);

	//like routeList
	Route::get('routeManagerRoutesList', 'App\Http\Controllers\RoutesController@routeManagerRoutesList')->middleware(['scope:super_admin,route_manager']);

	Route::get('viewManagerRoute/{id}', 'App\Http\Controllers\RoutesController@viewManagerRoute')->middleware(['scope:route_manager']);

	Route::post('managerPettyCashReceive', 'App\Http\Controllers\PettyCashController@managerPettyCashReceive')->middleware(['scope:route_manager']);

	Route::get('customerListForManager', 'App\Http\Controllers\CustomersController@customerListForManager')->middleware(['scope:route_manager']);

	Route::get('viewCustomerForManager/{id}', 'App\Http\Controllers\CustomersController@viewCustomerForManager')->middleware(['scope:route_manager']);

	Route::post('submitManagerPayment', 'App\Http\Controllers\CustomersController@submitManagerPayment')->middleware(['scope:route_manager']);

	//list routes under manager
	Route::get('salesReportForManagers', 'App\Http\Controllers\ReportsController@salesReportForManagers')->middleware(['scope:route_manager']);

	Route::get('supplyDetailsForManager', 'App\Http\Controllers\ReportsController@supplyDetailsForManager')->middleware(['scope:route_manager']);

	Route::get('customerPaymentDetailsForManager', 'App\Http\Controllers\RoutesController@customerPaymentDetailsForManager')->middleware(['scope:route_manager']);

	Route::get('routeCollectionForManager', 'App\Http\Controllers\ReportsController@routeCollectionForManager')->middleware(['scope:route_manager']);

	Route::get('routeManagersListForManager', 'App\Http\Controllers\RoutesController@routeManagersListForManager')->middleware(['scope:route_manager']);

	Route::get('routeManagerDetailsForManager/{id}', 'App\Http\Controllers\RoutesController@routeManagerDetailsForManager')->middleware(['scope:route_manager']);

	Route::post('receivePettyCashForManager', 'App\Http\Controllers\PettyCashController@receivePettyCashForManager')->middleware(['scope:route_manager']);


});

Route::post('refreshtoken', 'App\Http\Controllers\UserController@refreshToken');

Route::group(['middleware' => ['auth:api', 'role']], function() {



	//USER MANAGEMENT
	Route::get('userList', 'App\Http\Controllers\UserController@userList')->middleware(['scope:admin,super_admin']);
	Route::get('viewUser/{id}', 'App\Http\Controllers\UserController@viewUser')->middleware(['scope:admin,super_admin']);
	Route::delete('deleteUser/{id}', 'App\Http\Controllers\UserController@deleteUser')->middleware(['scope:admin,super_admin']);
	Route::post('updateUser', 'App\Http\Controllers\UserController@updateUser')->middleware(['scope:admin,super_admin']);
	Route::post('addUser', 'App\Http\Controllers\UserController@addUser')->middleware(['scope:super_admin']);
	Route::post('updateUser', 'App\Http\Controllers\UserController@updateUser')->middleware(['scope:admin,super_admin']);
	Route::post('addPermission', 'App\Http\Controllers\UserController@addPermission')->middleware(['scope:admin,super_admin']);
	Route::get('getAllUserList', 'App\Http\Controllers\UserController@getAllUserList')->middleware(['scope:admin,super_admin']);


	Route::post('logout', 'App\Http\Controllers\UserController@logout');
	Route::get('profile', 'App\Http\Controllers\UserController@profile')->middleware(['scope:admin,super_admin,supplier']);


	

	//PRODUCTS START
	Route::get('viewProduct/{id}', 'App\Http\Controllers\ProductsController@viewProduct')->middleware(['scope:admin,super_admin']);

	Route::get('productList', 'App\Http\Controllers\ProductsController@productList')->middleware(['scope:admin,super_admin']);

	

	Route::post('addProduct', 'App\Http\Controllers\ProductsController@addProduct')->middleware(['scope:admin,super_admin']);
	Route::post('updateProduct', 'App\Http\Controllers\ProductsController@updateProduct')->middleware(['scope:admin,super_admin']);

	Route::delete('deleteProduct/{id}', 'App\Http\Controllers\ProductsController@deleteProduct')->middleware(['scope:admin,super_admin']);
	//PRODUCTS END


	//ROUTES START
	Route::get('viewRoute/{id}', 'App\Http\Controllers\RoutesController@viewRoute')->middleware(['scope:admin,super_admin']);

	Route::post('addRoute', 'App\Http\Controllers\RoutesController@addRoute')->middleware(['scope:admin,super_admin']);

	Route::post('updateRoute', 'App\Http\Controllers\RoutesController@updateRoute')->middleware(['scope:admin,super_admin']);

	

	Route::get('routeListData', 'App\Http\Controllers\RoutesController@routeListData')->middleware(['scope:admin,super_admin']);

	

	Route::get('routeManagersList', 'App\Http\Controllers\RoutesController@routeManagersList')->middleware(['scope:admin,super_admin']);

	Route::post('addRouteEmployee', 'App\Http\Controllers\RoutesController@addRouteEmployee')->middleware(['scope:admin,super_admin']);

	Route::post('updateRouteEmployee', 'App\Http\Controllers\RoutesController@updateRouteEmployee')->middleware(['scope:admin,super_admin']);

	Route::get('employeeRouteList/{id}', 'App\Http\Controllers\RoutesController@employeeRouteList')->middleware(['scope:admin,super_admin']);

	Route::get('routeManagersListWithPrefix', 'App\Http\Controllers\RoutesController@routeManagersListWithPrefix')->middleware(['scope:admin,super_admin']);

	Route::get('getPrefixList', 'App\Http\Controllers\RoutesController@getPrefixList')->middleware(['scope:admin,super_admin']);

	Route::get('getPettyRouteUsersList', 'App\Http\Controllers\RoutesController@getPettyRouteUsersList')->middleware(['scope:admin,super_admin']);

	Route::get('getPettyCollectedUsersList', 'App\Http\Controllers\RoutesController@getPettyCollectedUsersList')->middleware(['scope:admin,super_admin']);


	

	//supplier delivery api
	Route::get('routeCustomerList', 'App\Http\Controllers\RoutesController@routeCustomerList')->middleware(['scope:supplier,admin,super_admin']);
	//for mobile product summary for supplier
	Route::get('routeProductSummary', 'App\Http\Controllers\RoutesController@routeProductSummary')->middleware(['scope:supplier,admin,super_admin']);


	//supplier customer payment list
	Route::get('custometPaymentList', 'App\Http\Controllers\RoutesController@custometPaymentList')->middleware(['scope:supplier,admin,super_admin']);

	Route::get('customerPaymentDetails', 'App\Http\Controllers\RoutesController@customerPaymentDetails')->middleware(['scope:supplier,admin,super_admin']);

	Route::delete('deleteRoute/{id}', 'App\Http\Controllers\RoutesController@deleteRoute')->middleware(['scope:admin,super_admin']);
	//ROUTES END



	//CUSTOMER START

	Route::get('viewCustomer/{id}', 'App\Http\Controllers\CustomersController@viewCustomer')->middleware(['scope:admin,super_admin']);

	Route::get('customerList', 'App\Http\Controllers\CustomersController@customerList')->middleware(['scope:admin,super_admin']);

	Route::post('addCustomer', 'App\Http\Controllers\CustomersController@addCustomer')->middleware(['scope:admin,super_admin']);

	Route::post('updateCustomer', 'App\Http\Controllers\CustomersController@updateCustomer')->middleware(['scope:admin,super_admin']);

	Route::delete('deleteCustomer/{id}', 'App\Http\Controllers\CustomersController@deleteCustomer')->middleware(['scope:admin,super_admin']);

	Route::post('updateCustomerStatus', 'App\Http\Controllers\CustomersController@updateCustomerStatus')->middleware(['scope:admin,super_admin']);

	Route::post('deleteDelivery', 'App\Http\Controllers\CustomersController@deleteDelivery')->middleware(['scope:admin,super_admin']);

	Route::post('addCustomerLocation', 'App\Http\Controllers\CustomersController@addCustomerLocation')->middleware(['scope:admin,super_admin,supplier']);

	//CUSTOMER END

	//DELIVERY START
	Route::post('submitDelivery', 'App\Http\Controllers\CustomersController@submitDelivery')->middleware(['scope:supplier,admin,super_admin']);

	Route::post('productPayment', 'App\Http\Controllers\CustomersController@productPayment')->middleware(['scope:supplier,admin,super_admin']);

	Route::post('submitPayment', 'App\Http\Controllers\CustomersController@submitPayment')->middleware(['scope:supplier,admin,super_admin']);

	//used one
	Route::post('submitSinglePayment', 'App\Http\Controllers\CustomersController@submitSinglePayment')->middleware(['scope:supplier,admin,super_admin']);

	//used one
	Route::post('submitAdminPayment', 'App\Http\Controllers\CustomersController@submitAdminPayment')->middleware(['scope:supplier,admin,super_admin']);

	Route::post('deletePayment', 'App\Http\Controllers\CustomersController@deletePayment')->middleware(['scope:admin,super_admin']);
	//DELIVERY END


	//REPORTS START
	Route::get('routeCollection', 'App\Http\Controllers\ReportsController@routeCollection')->middleware(['scope:admin,super_admin']);

	Route::get('salesReport', 'App\Http\Controllers\ReportsController@salesReport')->middleware(['scope:admin,super_admin']);

	

	Route::get('supplyDetails', 'App\Http\Controllers\ReportsController@supplyDetails')->middleware(['scope:admin,super_admin']);

	Route::get('routeSupplyDetails', 'App\Http\Controllers\ReportsController@routeSupplyDetails')->middleware(['scope:admin,super_admin']);

	Route::get('allSupplyDetails', 'App\Http\Controllers\ReportsController@allSupplyDetails')->middleware(['scope:admin,super_admin']);

	Route::get('allFutureSupplyDetails', 'App\Http\Controllers\ReportsController@allFutureSupplyDetails')->middleware(['scope:admin,super_admin']);

	Route::get('routeSupplyDetailss', 'App\Http\Controllers\ReportsController@routeSupplyDetailss')->middleware(['scope:admin,super_admin']);

	Route::get('routeSalesConsolidateReport', 'App\Http\Controllers\ReportsController@routeSalesConsolidateReport')->middleware(['scope:admin,super_admin']);

	Route::get('routeSalesConsolidateReportPdf', 'App\Http\Controllers\ReportsController@routeSalesConsolidateReportPdf')->middleware(['scope:admin,super_admin']);
	
	Route::get('itemConsolidatedReport', 'App\Http\Controllers\ReportsController@itemConsolidatedReport')->middleware(['scope:admin,super_admin']);

	Route::get('itemConsolidatedReportPdf', 'App\Http\Controllers\ReportsController@itemConsolidatedReportPdf')->middleware(['scope:admin,super_admin']);

	Route::get('pettyCashReport', 'App\Http\Controllers\ReportsController@pettyCashReport')->middleware(['scope:admin,super_admin']);
	//REPORTS END


	//PETTY CASH START
	Route::get('viewPettyCash/{id}', 'App\Http\Controllers\PettyCashController@viewPettyCash')->middleware(['scope:admin,super_admin']);

	Route::get('pettyCashList', 'App\Http\Controllers\PettyCashController@pettyCashList')->middleware(['scope:admin,super_admin']);

	//used one
	Route::post('receivePettyCash', 'App\Http\Controllers\PettyCashController@receivePettyCash')->middleware(['scope:admin,super_admin,route_manager']);

	Route::post('updatePettyCash', 'App\Http\Controllers\PettyCashController@updatePettyCash')->middleware(['scope:admin,super_admin']);

	Route::delete('deletePettyCash/{id}', 'App\Http\Controllers\PettyCashController@deletePettyCash')->middleware(['scope:admin,super_admin']);

	Route::delete('deletePettyCashDetails/{id}', 'App\Http\Controllers\PettyCashController@deletePettyCashDetails')->middleware(['scope:admin,super_admin,manager']);
	//PETTY CASH END

	Route::post('changePassword', 'App\Http\Controllers\UserController@changePassword')->middleware(['scope:admin,super_admin']);


	//CUSTOMER ORDER START
	Route::get('viewOrder/{id}', 'App\Http\Controllers\CustomerOrderController@viewOrder')->middleware(['scope:admin,super_admin']);

	Route::get('orderList', 'App\Http\Controllers\CustomerOrderController@orderList')->middleware(['scope:admin,super_admin']);

	Route::post('addOrder', 'App\Http\Controllers\CustomerOrderController@addOrder')->middleware(['scope:admin,super_admin']);

	Route::post('updateOrder', 'App\Http\Controllers\CustomerOrderController@updateOrder')->middleware(['scope:admin,super_admin']);

	Route::get('deleteOrder/{id}', 'App\Http\Controllers\CustomerOrderController@deleteOrder')->middleware(['scope:admin,super_admin']);
	//CUSTOMER ORDER END






});

